@@ -25,17 +25,19 @@ import (
 	"fmt"
 )
 
-// EncodeDigest encodes a hash algorithm and a digest to be put in an assertion header.
+// EncodeDigest encodes the digest from hash algorithm to be put in an assertion header.
 func EncodeDigest(hash crypto.Hash, hashDigest []byte) (string, error) {
 	algo := ""
 	switch hash {
-	case crypto.SHA256:
-		algo = "sha256"
+	case crypto.SHA512:
+		algo = "sha512"
+	case crypto.SHA3_384:
+		algo = "sha3-384"
 	default:
 		return "", fmt.Errorf("unsupported hash")
 	}
 	if len(hashDigest) != hash.Size() {
 		return "", fmt.Errorf("hash digest by %s should be %d bytes", algo, hash.Size())
 	}
-	return fmt.Sprintf("%s %s", algo, base64.RawURLEncoding.EncodeToString(hashDigest)), nil
+	return base64.RawURLEncoding.EncodeToString(hashDigest), nil
 }
