@@ -33,6 +33,7 @@ import (
 	"github.com/snapcore/snapd/bootloader"
 	"github.com/snapcore/snapd/bootloader/assets"
 	"github.com/snapcore/snapd/bootloader/bootloadertest"
+	"github.com/snapcore/snapd/bootloader/grubenv"
 	"github.com/snapcore/snapd/bootloader/ubootenv"
 	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/gadget"
@@ -79,6 +80,7 @@ func makeSnapWithFiles(c *C, name, yaml string, revno snap.Revision, files [][]s
 }
 
 func (s *makeBootableSuite) TestMakeBootable(c *C) {
+	bootloader.Force(nil)
 	model := boottest.MakeMockModel()
 
 	grubCfg := []byte("#grub cfg")
@@ -117,14 +119,11 @@ version: 4.0
 	c.Assert(err, IsNil)
 
 	// check the bootloader config
-	m, err := s.bootloader.GetBootVars("snap_kernel", "snap_core", "snap_menuentry")
-	c.Assert(err, IsNil)
-	c.Check(m["snap_kernel"], Equals, "pc-kernel_5.snap")
-	c.Check(m["snap_core"], Equals, "core18_3.snap")
-	c.Check(m["snap_menuentry"], Equals, "My Model")
-
-	// kernel was extracted as needed
-	c.Check(s.bootloader.ExtractKernelAssetsCalls, DeepEquals, []snap.PlaceInfo{kernelInfo})
+	seedGenv := grubenv.NewEnv(filepath.Join(s.rootdir, "boot/grub/grubenv"))
+	c.Assert(seedGenv.Load(), IsNil)
+	c.Check(seedGenv.Get("snap_kernel"), Equals, "pc-kernel_5.snap")
+	c.Check(seedGenv.Get("snap_core"), Equals, "core18_3.snap")
+	c.Check(seedGenv.Get("snap_menuentry"), Equals, "My Model")
 
 	// check symlinks from snap blob dir
 	kernelBlob := filepath.Join(dirs.SnapBlobDirUnder(s.rootdir), kernelInfo.Filename())
@@ -173,6 +172,7 @@ func (s *makeBootable20UbootSuite) SetUpTest(c *C) {
 }
 
 func (s *makeBootable20Suite) TestMakeBootable20(c *C) {
+	bootloader.Force(nil)
 	model := boottest.MakeMockUC20Model()
 
 	unpackedGadgetDir := c.MkDir()
@@ -228,7 +228,8 @@ version: 5.0
 	// ensure only a single file got copied (the grub.cfg)
 	files, err := filepath.Glob(filepath.Join(s.rootdir, "EFI/ubuntu/*"))
 	c.Assert(err, IsNil)
-	c.Check(files, HasLen, 1)
+	// grub.cfg and grubenv
+	c.Check(files, HasLen, 2)
 	// check that the recovery bootloader configuration was installed with
 	// the correct content
 	c.Check(filepath.Join(s.rootdir, "EFI/ubuntu/grub.cfg"), testutil.FileEquals, grubRecoveryCfgAsset)
@@ -237,13 +238,13 @@ version: 5.0
 	c.Check(filepath.Join(s.rootdir, "boot"), testutil.FileAbsent)
 
 	// ensure the correct recovery system configuration was set
-	c.Check(s.bootloader.RecoverySystemDir, Equals, recoverySystemDir)
-	c.Check(s.bootloader.RecoverySystemBootVars, DeepEquals, map[string]string{
-		"snapd_recovery_kernel": "/snaps/pc-kernel_5.snap",
-	})
-	c.Check(s.bootloader.BootVars, DeepEquals, map[string]string{
-		"snapd_recovery_system": label,
-	})
+	seedGenv := grubenv.NewEnv(filepath.Join(s.rootdir, "EFI/ubuntu/grubenv"))
+	c.Assert(seedGenv.Load(), IsNil)
+	c.Check(seedGenv.Get("snapd_recovery_system"), Equals, label)
+
+	systemGenv := grubenv.NewEnv(filepath.Join(s.rootdir, recoverySystemDir, "grubenv"))
+	c.Assert(systemGenv.Load(), IsNil)
+	c.Check(systemGenv.Get("snapd_recovery_kernel"), Equals, "/snaps/pc-kernel_5.snap")
 }
 
 func (s *makeBootable20Suite) TestMakeBootable20UnsetRecoverySystemLabelError(c *C) {
@@ -369,7 +370,8 @@ version: 5.0
 	}
 
 	// set up observer state
-	obs, err := boot.TrustedAssetsInstallObserverForModel(model, unpackedGadgetDir)
+	useEncryption := true
+	obs, err := boot.TrustedAssetsInstallObserverForModel(model, unpackedGadgetDir, useEncryption)
 	c.Assert(obs, NotNil)
 	c.Assert(err, IsNil)
 	runBootStruct := &gadget.LaidOutStructure{
@@ -389,10 +391,12 @@ version: 5.0
 
 	// set encryption key
 	myKey := secboot.EncryptionKey{}
+	myKey2 := secboot.EncryptionKey{}
 	for i := range myKey {
 		myKey[i] = byte(i)
+		myKey2[i] = byte(128 + i)
 	}
-	obs.ChosenEncryptionKey(myKey)
+	obs.ChosenEncryptionKeys(myKey, myKey2)
 
 	// set a mock recovery kernel
 	readSystemEssentialCalls := 0
@@ -410,10 +414,20 @@ version: 5.0
 	defer restore()
 
 	// set mock key sealing
-	sealKeyCalls := 0
-	restore = boot.MockSecbootSealKey(func(key secboot.EncryptionKey, params *secboot.SealKeyParams) error {
-		sealKeyCalls++
-		c.Check(key, DeepEquals, myKey)
+	sealKeysCalls := 0
+	restore = boot.MockSecbootSealKeys(func(keys []secboot.SealKeyRequest, params *secboot.SealKeysParams) error {
+		sealKeysCalls++
+		switch sealKeysCalls {
+		case 1:
+			c.Check(keys, HasLen, 1)
+			c.Check(keys[0].Key, DeepEquals, myKey)
+		case 2:
+			c.Check(keys, HasLen, 2)
+			c.Check(keys[0].Key, DeepEquals, myKey)
+			c.Check(keys[1].Key, DeepEquals, myKey2)
+		default:
+			c.Errorf("unexpected additional call to secboot.SealKeys (call # %d)", sealKeysCalls)
+		}
 		c.Assert(params.ModelParams, HasLen, 1)
 
 		shim := bootloader.NewBootFile("", filepath.Join(s.rootdir,
@@ -428,14 +442,27 @@ version: 5.0
 		kernel := bootloader.NewBootFile("/var/lib/snapd/seed/snaps/pc-kernel_1.snap", "kernel.efi", bootloader.RoleRecovery)
 		runKernel := bootloader.NewBootFile(filepath.Join(s.rootdir, "var/lib/snapd/snaps/pc-kernel_5.snap"), "kernel.efi", bootloader.RoleRunMode)
 
-		c.Assert(params.ModelParams[0].EFILoadChains, DeepEquals, []*secboot.LoadChain{
-			secboot.NewLoadChain(shim, secboot.NewLoadChain(grub, secboot.NewLoadChain(kernel))),
-			secboot.NewLoadChain(shim, secboot.NewLoadChain(grub, secboot.NewLoadChain(runGrub, secboot.NewLoadChain(runKernel)))),
-		})
-		c.Assert(params.ModelParams[0].KernelCmdlines, DeepEquals, []string{
-			"snapd_recovery_mode=recover snapd_recovery_system=20191216 console=ttyS0 console=tty1 panic=-1",
-			"snapd_recovery_mode=run console=ttyS0 console=tty1 panic=-1",
-		})
+		switch sealKeysCalls {
+		case 1:
+			c.Assert(params.ModelParams[0].EFILoadChains, DeepEquals, []*secboot.LoadChain{
+				secboot.NewLoadChain(shim, secboot.NewLoadChain(grub, secboot.NewLoadChain(kernel))),
+				secboot.NewLoadChain(shim, secboot.NewLoadChain(grub, secboot.NewLoadChain(runGrub, secboot.NewLoadChain(runKernel)))),
+			})
+			c.Assert(params.ModelParams[0].KernelCmdlines, DeepEquals, []string{
+				"snapd_recovery_mode=recover snapd_recovery_system=20191216 console=ttyS0 console=tty1 panic=-1",
+				"snapd_recovery_mode=run console=ttyS0 console=tty1 panic=-1",
+			})
+		case 2:
+			c.Assert(params.ModelParams[0].EFILoadChains, DeepEquals, []*secboot.LoadChain{
+				secboot.NewLoadChain(shim, secboot.NewLoadChain(grub, secboot.NewLoadChain(kernel))),
+			})
+			c.Assert(params.ModelParams[0].KernelCmdlines, DeepEquals, []string{
+				"snapd_recovery_mode=recover snapd_recovery_system=20191216 console=ttyS0 console=tty1 panic=-1",
+			})
+		default:
+			c.Errorf("unexpected additional call to secboot.SealKeys (call # %d)", sealKeysCalls)
+		}
+
 		c.Assert(params.ModelParams[0].Model.DisplayName(), Equals, "My Model")
 
 		return nil
@@ -517,8 +544,8 @@ current_trusted_recovery_boot_assets={"bootx64.efi":["39efae6545f16e39633fbfbef0
 	c.Check(copiedRecoveryGrubBin, testutil.FileEquals, "recovery grub content")
 	c.Check(copiedRecoveryShimBin, testutil.FileEquals, "recovery shim content")
 
-	// make sure SealKey was called
-	c.Check(sealKeyCalls, Equals, 1)
+	// make sure SealKey was called for the run object and the fallback object
+	c.Check(sealKeysCalls, Equals, 2)
 
 	// make sure the marker file for sealed key was created
 	c.Check(filepath.Join(dirs.SnapFDEDirUnder(boot.InstallHostWritableDir), "sealed-keys"), testutil.FilePresent)
@@ -582,9 +609,9 @@ version: 5.0
 		UnpackedGadgetDir: unpackedGadgetDir,
 	}
 
-	// no grub cfg in gadget directory raises an error
+	// no grub marker in gadget directory raises an error
 	err = boot.MakeBootable(model, s.rootdir, bootWith, nil)
-	c.Assert(err, ErrorMatches, "cannot install boot config with a mismatched gadget")
+	c.Assert(err, ErrorMatches, "internal error: cannot identify run system bootloader: cannot determine bootloader")
 
 	// set up grub.cfg in gadget
 	grubCfg := []byte("#grub cfg")
@@ -685,7 +712,8 @@ version: 5.0
 	}
 
 	// set up observer state
-	obs, err := boot.TrustedAssetsInstallObserverForModel(model, unpackedGadgetDir)
+	useEncryption := true
+	obs, err := boot.TrustedAssetsInstallObserverForModel(model, unpackedGadgetDir, useEncryption)
 	c.Assert(obs, NotNil)
 	c.Assert(err, IsNil)
 	runBootStruct := &gadget.LaidOutStructure{
@@ -705,10 +733,12 @@ version: 5.0
 
 	// set encryption key
 	myKey := secboot.EncryptionKey{}
+	myKey2 := secboot.EncryptionKey{}
 	for i := range myKey {
 		myKey[i] = byte(i)
+		myKey2[i] = byte(128 + i)
 	}
-	obs.ChosenEncryptionKey(myKey)
+	obs.ChosenEncryptionKeys(myKey, myKey2)
 
 	// set a mock recovery kernel
 	readSystemEssentialCalls := 0
@@ -726,10 +756,20 @@ version: 5.0
 	defer restore()
 
 	// set mock key sealing
-	sealKeyCalls := 0
-	restore = boot.MockSecbootSealKey(func(key secboot.EncryptionKey, params *secboot.SealKeyParams) error {
-		sealKeyCalls++
-		c.Check(key, DeepEquals, myKey)
+	sealKeysCalls := 0
+	restore = boot.MockSecbootSealKeys(func(keys []secboot.SealKeyRequest, params *secboot.SealKeysParams) error {
+		sealKeysCalls++
+		switch sealKeysCalls {
+		case 1:
+			c.Check(keys, HasLen, 1)
+			c.Check(keys[0].Key, DeepEquals, myKey)
+		case 2:
+			c.Check(keys, HasLen, 2)
+			c.Check(keys[0].Key, DeepEquals, myKey)
+			c.Check(keys[1].Key, DeepEquals, myKey2)
+		default:
+			c.Errorf("unexpected additional call to secboot.SealKeys (call # %d)", sealKeysCalls)
+		}
 		c.Assert(params.ModelParams, HasLen, 1)
 
 		shim := bootloader.NewBootFile("", filepath.Join(s.rootdir,
@@ -759,10 +799,11 @@ version: 5.0
 	defer restore()
 
 	err = boot.MakeBootable(model, s.rootdir, bootWith, obs)
-	c.Assert(err, ErrorMatches, "cannot seal the encryption key: seal error")
+	c.Assert(err, ErrorMatches, "cannot seal the encryption keys: seal error")
 }
 
 func (s *makeBootable20UbootSuite) TestUbootMakeBootable20TraditionalUbootenvFails(c *C) {
+	bootloader.Force(nil)
 	model := boottest.MakeMockUC20Model()
 
 	unpackedGadgetDir := c.MkDir()
@@ -810,7 +851,7 @@ version: 5.0
 
 	// TODO:UC20: enable this use case
 	err = boot.MakeBootable(model, s.rootdir, bootWith, nil)
-	c.Assert(err, ErrorMatches, fmt.Sprintf("cannot find boot config in %q", unpackedGadgetDir))
+	c.Assert(err, ErrorMatches, "non-empty uboot.env not supported on UC20 yet")
 }
 
 func (s *makeBootable20UbootSuite) TestUbootMakeBootable20BootScr(c *C) {
@@ -869,6 +910,7 @@ version: 5.0
 
 	c.Check(s.bootloader.BootVars, DeepEquals, map[string]string{
 		"snapd_recovery_system": label,
+		"snapd_recovery_mode":   "install",
 	})
 
 	// ensure the correct recovery system configuration was set
@@ -882,7 +924,7 @@ version: 5.0
 	)
 }
 
-func (s *makeBootable20UbootSuite) TestUbootMakeBootable20RunModeBootScr(c *C) {
+func (s *makeBootable20UbootSuite) TestUbootMakeBootable20RunModeBootSel(c *C) {
 	bootloader.Force(nil)
 
 	model := boottest.MakeMockUC20Model()
@@ -898,7 +940,7 @@ func (s *makeBootable20UbootSuite) TestUbootMakeBootable20RunModeBootScr(c *C) {
 	c.Assert(err, IsNil)
 	c.Assert(env.Save(), IsNil)
 
-	// uboot on ubuntu-boot
+	// uboot on ubuntu-boot (as if it was installed when creating the partition)
 	mockBootUbootBootSel := filepath.Join(boot.InitramfsUbuntuBootDir, "uboot/ubuntu/boot.sel")
 	err = os.MkdirAll(filepath.Dir(mockBootUbootBootSel), 0755)
 	c.Assert(err, IsNil)
@@ -906,6 +948,9 @@ func (s *makeBootable20UbootSuite) TestUbootMakeBootable20RunModeBootScr(c *C) {
 	c.Assert(err, IsNil)
 	c.Assert(env.Save(), IsNil)
 
+	unpackedGadgetDir := c.MkDir()
+	c.Assert(ioutil.WriteFile(filepath.Join(unpackedGadgetDir, "uboot.conf"), nil, 0644), IsNil)
+
 	baseFn, baseInfo := makeSnap(c, "core20", `name: core20
 type: base
 version: 5.0
@@ -934,8 +979,8 @@ version: 5.0
 		KernelPath:        kernelInSeed,
 		Kernel:            kernelInfo,
 		Recovery:          false,
+		UnpackedGadgetDir: unpackedGadgetDir,
 	}
-
 	err = boot.MakeBootable(model, s.rootdir, bootWith, nil)
 	c.Assert(err, IsNil)
 
