@@ -84,14 +84,12 @@ func (client *Client) Known(assertTypeName string, headers map[string]string, op
 		q.Set("remote", "true")
 	}
 
-	ctx, cancel := context.WithTimeout(context.Background(), doTimeout)
-	defer cancel()
-	response, err := client.raw(ctx, "GET", path, q, nil, nil)
+	response, cancel, err := client.rawWithTimeout(context.Background(), "GET", path, q, nil, nil, nil)
 	if err != nil {
 		fmt := "failed to query assertions: %w"
 		return nil, xerrors.Errorf(fmt, err)
 	}
-
+	defer cancel()
 	defer response.Body.Close()
 	if response.StatusCode != 200 {
 		return nil, parseError(response)
