@@ -195,7 +195,7 @@ func (cs *clientSuite) TestClientConnect(c *check.C) {
 }
 
 func (cs *clientSuite) TestClientDisconnectCallsEndpoint(c *check.C) {
-	cs.cli.Disconnect("producer", "plug", "consumer", "slot")
+	cs.cli.Disconnect("producer", "plug", "consumer", "slot", nil)
 	c.Check(cs.req.Method, check.Equals, "POST")
 	c.Check(cs.req.URL.Path, check.Equals, "/v2/interfaces")
 }
@@ -208,7 +208,8 @@ func (cs *clientSuite) TestClientDisconnect(c *check.C) {
 		"result": { },
                 "change": "42"
 	}`
-	id, err := cs.cli.Disconnect("producer", "plug", "consumer", "slot")
+	opts := &client.DisconnectOptions{Forget: false}
+	id, err := cs.cli.Disconnect("producer", "plug", "consumer", "slot", opts)
 	c.Assert(err, check.IsNil)
 	c.Check(id, check.Equals, "42")
 	var body map[string]interface{}
@@ -231,3 +232,37 @@ func (cs *clientSuite) TestClientDisconnect(c *check.C) {
 		},
 	})
 }
+
+func (cs *clientSuite) TestClientDisconnectForget(c *check.C) {
+	cs.status = 202
+	cs.rsp = `{
+		"type": "async",
+                "status-code": 202,
+		"result": { },
+                "change": "42"
+	}`
+	opts := &client.DisconnectOptions{Forget: true}
+	id, err := cs.cli.Disconnect("producer", "plug", "consumer", "slot", opts)
+	c.Assert(err, check.IsNil)
+	c.Check(id, check.Equals, "42")
+	var body map[string]interface{}
+	decoder := json.NewDecoder(cs.req.Body)
+	err = decoder.Decode(&body)
+	c.Check(err, check.IsNil)
+	c.Check(body, check.DeepEquals, map[string]interface{}{
+		"action": "disconnect",
+		"forget": true,
+		"plugs": []interface{}{
+			map[string]interface{}{
+				"snap": "producer",
+				"plug": "plug",
+			},
+		},
+		"slots": []interface{}{
+			map[string]interface{}{
+				"snap": "consumer",
+				"slot": "slot",
+			},
+		},
+	})
+}
