@@ -22,8 +22,9 @@
 
 void sc_cleanup_string(char **ptr)
 {
-	if (ptr != NULL) {
+	if (ptr != NULL && *ptr != NULL) {
 		free(*ptr);
+		*ptr = NULL;
 	}
 }
 
@@ -31,6 +32,7 @@ void sc_cleanup_file(FILE ** ptr)
 {
 	if (ptr != NULL && *ptr != NULL) {
 		fclose(*ptr);
+		*ptr = NULL;
 	}
 }
 
@@ -38,6 +40,7 @@ void sc_cleanup_endmntent(FILE ** ptr)
 {
 	if (ptr != NULL && *ptr != NULL) {
 		endmntent(*ptr);
+		*ptr = NULL;
 	}
 }
 
@@ -45,6 +48,7 @@ void sc_cleanup_closedir(DIR ** ptr)
 {
 	if (ptr != NULL && *ptr != NULL) {
 		closedir(*ptr);
+		*ptr = NULL;
 	}
 }
 
@@ -52,5 +56,6 @@ void sc_cleanup_close(int *ptr)
 {
 	if (ptr != NULL && *ptr != -1) {
 		close(*ptr);
+		*ptr = -1;
 	}
 }
