@@ -112,8 +112,11 @@ static const char *nvidia_globs[] = {
 	"libnvidia-ifr.so*",
 	"libnvidia-ml.so*",
 	"libnvidia-opencl.so*",
+	"libnvidia-opticalflow.so*",
 	"libnvidia-ptxjitcompiler.so*",
+	"libnvidia-rtcore.so*",
 	"libnvidia-tls.so*",
+	"libnvoptix.so*",
 	"tls/libnvidia-tls.so*",
 	"vdpau/libvdpau_nvidia.so*",
 };
