@@ -35,8 +35,6 @@ import (
 
 type appOpSuite struct {
 	BaseSnapSuite
-
-	restoreAll func()
 }
 
 var _ = check.Suite(&appOpSuite{})
@@ -44,16 +42,13 @@ var _ = check.Suite(&appOpSuite{})
 func (s *appOpSuite) SetUpTest(c *check.C) {
 	s.BaseSnapSuite.SetUpTest(c)
 
-	restoreClientRetry := client.MockDoRetry(time.Millisecond, 10*time.Millisecond)
+	restoreClientRetry := client.MockDoTimings(time.Millisecond, 100*time.Millisecond)
 	restorePollTime := snap.MockPollTime(time.Millisecond)
-	s.restoreAll = func() {
-		restoreClientRetry()
-		restorePollTime()
-	}
+	s.AddCleanup(restoreClientRetry)
+	s.AddCleanup(restorePollTime)
 }
 
 func (s *appOpSuite) TearDownTest(c *check.C) {
-	s.restoreAll()
 	s.BaseSnapSuite.TearDownTest(c)
 }
 
