@@ -26,21 +26,24 @@ import (
 )
 
 type cmdUnalias struct {
+	waitMixin
 	Positionals struct {
 		AliasOrSnap aliasOrSnap `required:"yes"`
 	} `positional-args:"true"`
 }
 
-var shortUnaliasHelp = i18n.G("Unalias a manual alias or an entire snap")
+var shortUnaliasHelp = i18n.G("Remove a manual alias, or the aliases for an entire snap")
 var longUnaliasHelp = i18n.G(`
-The unalias command tears down a manual alias when given one or disables all aliases of a snap, removing also all manual ones, when given a snap name.
+The unalias command removes a single alias if the provided argument is a manual
+alias, or disables all aliases of a snap, including manual ones, if the
+argument is a snap name.
 `)
 
 func init() {
 	addCommand("unalias", shortUnaliasHelp, longUnaliasHelp, func() flags.Commander {
 		return &cmdUnalias{}
-	}, nil, []argDesc{
-		// TRANSLATORS: This needs to be wrapped in <>s.
+	}, waitDescs.also(nil), []argDesc{
+		// TRANSLATORS: This needs to begin with < and end with >
 		{name: i18n.G("<alias-or-snap>")},
 	})
 }
@@ -50,19 +53,17 @@ func (x *cmdUnalias) Execute(args []string) error {
 		return ErrExtraArgs
 	}
 
-	cli := Client()
-	id, err := cli.Unalias(string(x.Positionals.AliasOrSnap))
+	id, err := x.client.Unalias(string(x.Positionals.AliasOrSnap))
 	if err != nil {
 		return err
 	}
-
-	chg, err := wait(cli, id)
+	chg, err := x.wait(id)
 	if err != nil {
-		return err
-	}
-	if err := showAliasChanges(chg); err != nil {
+		if err == noWait {
+			return nil
+		}
 		return err
 	}
 
-	return nil
+	return showAliasChanges(chg)
 }
