@@ -43,8 +43,7 @@ func (x *cmdWatch) Execute(args []string) error {
 	if len(args) > 0 {
 		return ErrExtraArgs
 	}
-	cli := Client()
-	id, err := x.GetChangeID(cli)
+	id, err := x.GetChangeID()
 	if err != nil {
 		if err == noChangeFoundOK {
 			return nil
@@ -54,7 +53,9 @@ func (x *cmdWatch) Execute(args []string) error {
 
 	// this is the only valid use of wait without a waitMixin (ie
 	// without --no-wait), so we fake it here.
-	_, err = waitMixin{skipAbort: true}.wait(cli, id)
+	wmx := &waitMixin{skipAbort: true}
+	wmx.client = x.client
+	_, err = wmx.wait(id)
 
 	return err
 }
