@@ -106,8 +106,8 @@ func (s remoteSnapName) Complete(match string) []flags.Completion {
 		return nil
 	}
 	snaps, _, err := mkClient().Find(&client.FindOptions{
-		Prefix: true,
 		Query:  match,
+		Prefix: true,
 	})
 	if err != nil {
 		return nil
@@ -508,7 +508,7 @@ func (snapshotID) Complete(match string) []flags.Completion {
 func (s snapshotID) ToUint() (uint64, error) {
 	setID, err := strconv.ParseUint((string)(s), 10, 64)
 	if err != nil {
-		return 0, fmt.Errorf(i18n.G("invalid argument for set id: expected a non-negative integer argument"))
+		return 0, fmt.Errorf(i18n.G("invalid argument for snapshot set id: expected a non-negative integer argument (see 'snap help saved')"))
 	}
 	return setID, nil
 }
