@@ -384,3 +384,19 @@ func MockDownloadDirect(f func(snapName string, revision snap.Revision, dlOpts i
 		downloadDirect = old
 	}
 }
+
+func MockSnapdAPIInterval(t time.Duration) (restore func()) {
+	old := snapdAPIInterval
+	snapdAPIInterval = t
+	return func() {
+		snapdAPIInterval = old
+	}
+}
+
+func MockSnapdWaitForFullSystemReboot(t time.Duration) (restore func()) {
+	old := snapdWaitForFullSystemReboot
+	snapdWaitForFullSystemReboot = t
+	return func() {
+		snapdWaitForFullSystemReboot = old
+	}
+}
