@@ -185,7 +185,7 @@ int ensure_fusesquashfs_inside_container(const char *normal_dir)
 			fprintf(stderr, "cannot open %s: %m\n", fname);
 			return 2;
 		}
-		fprintf(f, "[Mount]\nType=%s\n", fstype);
+		fprintf(f, "[Mount]\nType=%s\nOptions=nodev,ro,x-gdu.hide,allow_other\nLazyUnmount=yes\n", fstype);
 	}
 	
 	return 0;
