@@ -113,6 +113,8 @@ var api = []*Command{
 	serialModelCmd,
 	systemsCmd,
 	systemsActionCmd,
+	routineConsoleConfStartCmd,
+	systemRecoveryKeysCmd,
 }
 
 var servicestateControl = servicestate.Control
@@ -569,7 +571,7 @@ func searchStore(c *Command, r *http.Request, user *auth.UserState) Response {
 				Status: 400,
 			}, nil)
 		}
-		if e, ok := err.(*httputil.PerstistentNetworkError); ok {
+		if e, ok := err.(*httputil.PersistentNetworkError); ok {
 			return SyncResponse(&resp{
 				Type:   ResponseTypeError,
 				Result: &errorResult{Message: e.Error(), Kind: client.ErrorKindDNSFailure},
@@ -796,6 +798,7 @@ type snapInstruction struct {
 	JailMode         bool `json:"jailmode"`
 	Classic          bool `json:"classic"`
 	IgnoreValidation bool `json:"ignore-validation"`
+	IgnoreRunning    bool `json:"ignore-running"`
 	Unaliased        bool `json:"unaliased"`
 	Purge            bool `json:"purge,omitempty"`
 	// dropping support temporarely until flag confusion is sorted,
@@ -831,6 +834,10 @@ func (inst *snapInstruction) installFlags() (snapstate.Flags, error) {
 	if inst.Unaliased {
 		flags.Unaliased = true
 	}
+	if inst.IgnoreRunning {
+		flags.IgnoreRunning = true
+	}
+
 	return flags, nil
 }
 
@@ -884,6 +891,7 @@ var (
 	snapshotRestore = snapshotstate.Restore
 	snapshotSave    = snapshotstate.Save
 	snapshotExport  = snapshotstate.Export
+	snapshotImport  = snapshotstate.Import
 
 	assertstateRefreshSnapDeclarations = assertstate.RefreshSnapDeclarations
 )
@@ -1025,6 +1033,9 @@ func snapUpdate(inst *snapInstruction, st *state.State) (string, []*state.TaskSe
 	if inst.IgnoreValidation {
 		flags.IgnoreValidation = true
 	}
+	if inst.IgnoreRunning {
+		flags.IgnoreRunning = true
+	}
 	if inst.Amend {
 		flags.Amend = true
 	}
@@ -1423,6 +1434,7 @@ func postSnaps(c *Command, r *http.Request, user *auth.UserState) Response {
 	flags.RemoveSnapPath = true
 
 	flags.Unaliased = isTrue(form, "unaliased")
+	flags.IgnoreRunning = isTrue(form, "ignore-running")
 
 	// find the file for the "snap" form field
 	var snapBody multipart.File
@@ -2146,12 +2158,13 @@ func readyToBuy(c *Command, r *http.Request, user *auth.UserState) Response {
 }
 
 func runSnapctl(c *Command, r *http.Request, user *auth.UserState) Response {
-	var snapctlOptions client.SnapCtlOptions
-	if err := jsonutil.DecodeWithNumber(r.Body, &snapctlOptions); err != nil {
+	var snapctlPostData client.SnapCtlPostData
+
+	if err := jsonutil.DecodeWithNumber(r.Body, &snapctlPostData); err != nil {
 		return BadRequest("cannot decode snapctl request: %s", err)
 	}
 
-	if len(snapctlOptions.Args) == 0 {
+	if len(snapctlPostData.Args) == 0 {
 		return BadRequest("snapctl cannot run without args")
 	}
 
@@ -2162,8 +2175,17 @@ func runSnapctl(c *Command, r *http.Request, user *auth.UserState) Response {
 
 	// Ignore missing context error to allow 'snapctl -h' without a context;
 	// Actual context is validated later by get/set.
-	context, _ := c.d.overlord.HookManager().Context(snapctlOptions.ContextID)
-	stdout, stderr, err := ctlcmdRun(context, snapctlOptions.Args, uid)
+	context, _ := c.d.overlord.HookManager().Context(snapctlPostData.ContextID)
+
+	// make the data read from stdin available for the hook
+	// TODO: use a forwarded stdin here
+	if snapctlPostData.Stdin != nil {
+		context.Lock()
+		context.Set("stdin", snapctlPostData.Stdin)
+		context.Unlock()
+	}
+
+	stdout, stderr, err := ctlcmdRun(context, snapctlPostData.Args, uid)
 	if err != nil {
 		if e, ok := err.(*ctlcmd.UnsuccessfulError); ok {
 			result := map[string]interface{}{
@@ -2399,7 +2421,7 @@ func getLogs(c *Command, r *http.Request, user *auth.UserState) Response {
 		serviceNames[i] = appInfo.ServiceName()
 	}
 
-	sysd := systemd.New(dirs.GlobalRootDir, systemd.SystemMode, progress.Null)
+	sysd := systemd.New(systemd.SystemMode, progress.Null)
 	reader, err := sysd.LogReader(serviceNames, n, follow)
 	if err != nil {
 		return InternalError("cannot get logs: %v", err)
