@@ -21,6 +21,8 @@ package gadget
 
 import (
 	"errors"
+
+	"github.com/snapcore/snapd/gadget/quantity"
 )
 
 var errNotImplemented = errors.New("not implemented")
@@ -29,7 +31,7 @@ func FindDeviceForStructure(ps *LaidOutStructure) (string, error) {
 	return "", errNotImplemented
 }
 
-func findDeviceForStructureWithFallback(ps *LaidOutStructure) (string, Size, error) {
+func findDeviceForStructureWithFallback(ps *LaidOutStructure) (string, quantity.Size, error) {
 	return "", 0, errNotImplemented
 }
 
