@@ -23,7 +23,6 @@ import (
 	"errors"
 	"fmt"
 	"io/ioutil"
-	"math"
 	"os"
 	"path/filepath"
 	"regexp"
@@ -34,6 +33,7 @@ import (
 
 	"github.com/snapcore/snapd/asserts"
 	"github.com/snapcore/snapd/gadget/edition"
+	"github.com/snapcore/snapd/gadget/quantity"
 	"github.com/snapcore/snapd/metautil"
 	"github.com/snapcore/snapd/snap"
 	"github.com/snapcore/snapd/snap/naming"
@@ -50,6 +50,7 @@ const (
 	SystemBoot = "system-boot"
 	SystemData = "system-data"
 	SystemSeed = "system-seed"
+	SystemSave = "system-save"
 
 	bootImage  = "system-boot-image"
 	bootSelect = "system-boot-select"
@@ -107,13 +108,13 @@ type VolumeStructure struct {
 	// Label provides the filesystem label
 	Label string `yaml:"filesystem-label"`
 	// Offset defines a starting offset of the structure
-	Offset *Size `yaml:"offset"`
+	Offset *quantity.Size `yaml:"offset"`
 	// OffsetWrite describes a 32-bit address, within the volume, at which
 	// the offset of current structure will be written. The position may be
 	// specified as a byte offset relative to the start of a named structure
 	OffsetWrite *RelativeOffset `yaml:"offset-write"`
 	// Size of the structure
-	Size Size `yaml:"size"`
+	Size quantity.Size `yaml:"size"`
 	// Type of the structure, which can be 2-hex digit MBR partition,
 	// 36-char GUID partition, comma separated <mbr>,<guid> for hybrid
 	// partitioning schemes, or 'bare' when the structure is not considered
@@ -186,14 +187,14 @@ type VolumeContent struct {
 	// for a 'bare' type structure
 	Image string `yaml:"image"`
 	// Offset the image is written at
-	Offset *Size `yaml:"offset"`
+	Offset *quantity.Size `yaml:"offset"`
 	// OffsetWrite describes a 32-bit address, within the volume, at which
 	// the offset of current image will be written. The position may be
 	// specified as a byte offset relative to the start of a named structure
 	OffsetWrite *RelativeOffset `yaml:"offset-write"`
 	// Size of the image, when empty size is calculated by looking at the
 	// image
-	Size Size `yaml:"size"`
+	Size quantity.Size `yaml:"size"`
 
 	Unpack bool `yaml:"unpack"`
 }
@@ -405,6 +406,7 @@ type validationState struct {
 	SystemSeed *VolumeStructure
 	SystemData *VolumeStructure
 	SystemBoot *VolumeStructure
+	SystemSave *VolumeStructure
 }
 
 func validateVolume(name string, vol *Volume, model Model) error {
@@ -423,12 +425,12 @@ func validateVolume(name string, vol *Volume, model Model) error {
 	structures := make([]LaidOutStructure, len(vol.Structure))
 
 	state := &validationState{}
-	previousEnd := Size(0)
+	previousEnd := quantity.Size(0)
 	for idx, s := range vol.Structure {
 		if err := validateVolumeStructure(&s, vol); err != nil {
 			return fmt.Errorf("invalid structure %v: %v", fmtIndexAndName(idx, s.Name), err)
 		}
-		var start Size
+		var start quantity.Size
 		if s.Offset != nil {
 			start = *s.Offset
 		} else {
@@ -471,6 +473,11 @@ func validateVolume(name string, vol *Volume, model Model) error {
 				return fmt.Errorf("cannot have more than one partition with system-boot role")
 			}
 			state.SystemBoot = &vol.Structure[idx]
+		case SystemSave:
+			if state.SystemSave != nil {
+				return fmt.Errorf("cannot have more than one partition with system-save role")
+			}
+			state.SystemSave = &vol.Structure[idx]
 		}
 
 		previousEnd = end
@@ -489,7 +496,7 @@ func validateVolume(name string, vol *Volume, model Model) error {
 func ensureVolumeConsistencyNoConstraints(state *validationState) error {
 	switch {
 	case state.SystemSeed == nil && state.SystemData == nil:
-		return nil
+		// happy so far
 	case state.SystemSeed != nil && state.SystemData == nil:
 		return fmt.Errorf("the system-seed role requires system-data to be defined")
 	case state.SystemSeed == nil && state.SystemData != nil:
@@ -501,6 +508,11 @@ func ensureVolumeConsistencyNoConstraints(state *validationState) error {
 			return err
 		}
 	}
+	if state.SystemSave != nil {
+		if err := ensureSystemSaveConsistency(state); err != nil {
+			return err
+		}
+	}
 	return nil
 }
 
@@ -510,7 +522,6 @@ func ensureVolumeConsistencyWithConstraints(state *validationState, model Model)
 		if wantsSystemSeed(model) {
 			return fmt.Errorf("model requires system-seed partition, but no system-seed or system-data partition found")
 		}
-		return nil
 	case state.SystemSeed != nil && state.SystemData == nil:
 		return fmt.Errorf("the system-seed role requires system-data to be defined")
 	case state.SystemSeed == nil && state.SystemData != nil:
@@ -519,7 +530,7 @@ func ensureVolumeConsistencyWithConstraints(state *validationState, model Model)
 			return fmt.Errorf("model requires system-seed structure, but none was found")
 		}
 		// without SystemSeed, system-data label must be implicit or writable
-		if state.SystemData != nil && state.SystemData.Label != "" && state.SystemData.Label != implicitSystemDataLabel {
+		if state.SystemData.Label != "" && state.SystemData.Label != implicitSystemDataLabel {
 			return fmt.Errorf("system-data structure must have an implicit label or %q, not %q",
 				implicitSystemDataLabel, state.SystemData.Label)
 		}
@@ -532,6 +543,11 @@ func ensureVolumeConsistencyWithConstraints(state *validationState, model Model)
 			return err
 		}
 	}
+	if state.SystemSave != nil {
+		if err := ensureSystemSaveConsistency(state); err != nil {
+			return err
+		}
+	}
 	return nil
 }
 
@@ -549,12 +565,21 @@ func ensureSeedDataLabelsUnset(state *validationState) error {
 	if state.SystemSeed.Label != "" {
 		return fmt.Errorf("system-seed structure must not have a label")
 	}
+	return nil
+}
 
+func ensureSystemSaveConsistency(state *validationState) error {
+	if state.SystemData == nil || state.SystemSeed == nil {
+		return fmt.Errorf("system-save requires system-seed and system-data structures")
+	}
+	if state.SystemSave.Label != "" {
+		return fmt.Errorf("system-save structure must not have a label")
+	}
 	return nil
 }
 
 func validateCrossVolumeStructure(structures []LaidOutStructure, knownStructures map[string]*LaidOutStructure) error {
-	previousEnd := Size(0)
+	previousEnd := quantity.Size(0)
 	// cross structure validation:
 	// - relative offsets that reference other structures by name
 	// - laid out structure overlap
@@ -598,6 +623,31 @@ func validateCrossVolumeStructure(structures []LaidOutStructure, knownStructures
 	return nil
 }
 
+var (
+	reservedLabels = []string{
+		// 2020-12-02 disabled because of customer gadget hotfix
+		/*ubuntuBootLabel,*/
+		ubuntuSeedLabel,
+		ubuntuDataLabel,
+		ubuntuSaveLabel,
+	}
+)
+
+func validateReservedLabels(vs *VolumeStructure) error {
+	if vs.Role != "" {
+		// structure specifies a role, its labels will be checked later
+		return nil
+	}
+	if vs.Label == "" {
+		return nil
+	}
+	if strutil.ListContains(reservedLabels, vs.Label) {
+		// a structure without a role uses one of reserved labels
+		return fmt.Errorf("label %q is reserved", vs.Label)
+	}
+	return nil
+}
+
 func validateVolumeStructure(vs *VolumeStructure, vol *Volume) error {
 	if vs.Size == 0 {
 		return errors.New("missing size")
@@ -635,6 +685,10 @@ func validateVolumeStructure(vs *VolumeStructure, vol *Volume) error {
 		return err
 	}
 
+	if err := validateReservedLabels(vs); err != nil {
+		return err
+	}
+
 	// TODO: validate structure size against sector-size; ubuntu-image uses
 	// a tmp file to find out the default sector size of the device the tmp
 	// file is created on
@@ -720,7 +774,7 @@ func validateRole(vs *VolumeStructure, vol *Volume) error {
 	}
 
 	switch vsRole {
-	case SystemData, SystemSeed:
+	case SystemData, SystemSeed, SystemSave:
 		// roles have cross dependencies, consistency checks are done at
 		// the volume level
 	case schemaMBR:
@@ -783,91 +837,14 @@ func validateStructureUpdate(up *VolumeUpdate, vs *VolumeStructure) error {
 	return nil
 }
 
-// Size describes the size of a structure item or an offset within the
-// structure.
-type Size uint64
-
 const (
-	SizeKiB = Size(1 << 10)
-	SizeMiB = Size(1 << 20)
-	SizeGiB = Size(1 << 30)
-
 	// SizeMBR is the maximum byte size of a structure of role 'mbr'
-	SizeMBR = Size(446)
+	SizeMBR = quantity.Size(446)
 	// SizeLBA48Pointer is the byte size of a pointer value written at the
 	// location described by 'offset-write'
-	SizeLBA48Pointer = Size(4)
+	SizeLBA48Pointer = quantity.Size(4)
 )
 
-func (s *Size) UnmarshalYAML(unmarshal func(interface{}) error) error {
-	var gs string
-	if err := unmarshal(&gs); err != nil {
-		return errors.New(`cannot unmarshal gadget size`)
-	}
-
-	var err error
-	*s, err = parseSize(gs)
-	if err != nil {
-		return fmt.Errorf("cannot parse size %q: %v", gs, err)
-	}
-	return err
-}
-
-// parseSize parses a string expressing size in gadget declaration. The
-// accepted format is one of: <bytes> | <bytes/2^20>M | <bytes/2^30>G.
-func parseSize(gs string) (Size, error) {
-	number, unit, err := strutil.SplitUnit(gs)
-	if err != nil {
-		return 0, err
-	}
-	if number < 0 {
-		return 0, errors.New("size cannot be negative")
-	}
-	var size Size
-	switch unit {
-	case "M":
-		// MiB
-		size = Size(number) * SizeMiB
-	case "G":
-		// GiB
-		size = Size(number) * SizeGiB
-	case "":
-		// straight bytes
-		size = Size(number)
-	default:
-		return 0, fmt.Errorf("invalid suffix %q", unit)
-	}
-	return size, nil
-}
-
-func (s *Size) String() string {
-	if s == nil {
-		return "unspecified"
-	}
-	return fmt.Sprintf("%d", *s)
-}
-
-// IECString formats the size using multiples from IEC units (i.e. kibibytes,
-// mebibytes), that is as multiples of 1024. Printed values are truncated to 2
-// decimal points.
-func (s *Size) IECString() string {
-	maxFloat := float64(1023.5)
-	r := float64(*s)
-	unit := "B"
-	for _, rangeUnit := range []string{"KiB", "MiB", "GiB", "TiB", "PiB"} {
-		if r < maxFloat {
-			break
-		}
-		r /= 1024
-		unit = rangeUnit
-	}
-	precision := 0
-	if math.Floor(r) != r {
-		precision = 2
-	}
-	return fmt.Sprintf("%.*f %s", precision, r, unit)
-}
-
 // RelativeOffset describes an offset where structure data is written at.
 // The position can be specified as byte-offset relative to the start of another
 // named structure.
@@ -876,7 +853,7 @@ type RelativeOffset struct {
 	// address write will be calculated.
 	RelativeTo string
 	// Offset is a 32-bit value
-	Offset Size
+	Offset quantity.Size
 }
 
 func (r *RelativeOffset) String() string {
@@ -904,11 +881,11 @@ func parseRelativeOffset(grs string) (*RelativeOffset, error) {
 		return nil, errors.New("missing offset")
 	}
 
-	size, err := parseSize(sizeSpec)
+	size, err := quantity.ParseSize(sizeSpec)
 	if err != nil {
 		return nil, fmt.Errorf("cannot parse offset %q: %v", sizeSpec, err)
 	}
-	if size > 4*SizeGiB {
+	if size > 4*quantity.SizeGiB {
 		return nil, fmt.Errorf("offset above 4G limit")
 	}
 
@@ -969,12 +946,6 @@ func IsCompatible(current, new *Info) error {
 // PositionedVolumeFromGadget takes a gadget rootdir and positions the
 // partitions as specified.
 func PositionedVolumeFromGadget(gadgetRoot string) (*LaidOutVolume, error) {
-	// TODO:UC20: since this is unconstrained via the model, it returns an
-	//            err == nil and an empty info when the gadgetRoot does not
-	//            actually contain the required gadget.yaml file (for example
-	//            when you have a typo in the args to snap-bootstrap
-	//            create-partitions). anyways just verify this more because
-	//            otherwise it's unhelpful :-/
 	info, err := ReadInfo(gadgetRoot, nil)
 	if err != nil {
 		return nil, err
@@ -985,7 +956,7 @@ func PositionedVolumeFromGadget(gadgetRoot string) (*LaidOutVolume, error) {
 	}
 
 	constraints := LayoutConstraints{
-		NonMBRStartOffset: 1 * SizeMiB,
+		NonMBRStartOffset: 1 * quantity.SizeMiB,
 		SectorSize:        512,
 	}
 
