@@ -27,6 +27,7 @@ import (
 	. "gopkg.in/check.v1"
 
 	"github.com/snapcore/snapd/gadget"
+	"github.com/snapcore/snapd/gadget/quantity"
 	"github.com/snapcore/snapd/testutil"
 )
 
@@ -75,8 +76,7 @@ echo '{
         "size": 2457600,
         "type": "C12A7328-F81F-11D2-BA4B-00A0C93EC93B",
         "uuid": "44C3D5C3-CAE1-4306-83E8-DF437ACDB32F",
-        "name": "Recovery",
-        "attrs": "GUID:59"
+        "name": "Recovery"
       }
     ]
   }
@@ -136,6 +136,11 @@ const gadgetContent = `volumes:
         content:
           - source: grubx64.efi
             target: EFI/boot/grubx64.efi
+      - name: Save
+        role: system-save
+        filesystem: ext4
+        type: 83,0FC63DAF-8483-4772-8E79-3D69D8477DE4
+        size: 128M
       - name: Writable
         role: system-data
         filesystem: ext4
@@ -143,21 +148,39 @@ const gadgetContent = `volumes:
         size: 1200M
 `
 
-var mockOnDiskStructureWritable = gadget.OnDiskStructure{
-	Node:                 "/dev/node3",
-	CreatedDuringInstall: true,
+var mockOnDiskStructureSave = gadget.OnDiskStructure{
+	Node: "/dev/node3",
 	LaidOutStructure: gadget.LaidOutStructure{
 		VolumeStructure: &gadget.VolumeStructure{
-			Name:       "Writable",
-			Size:       1258291200,
+			Name:       "Save",
+			Size:       128 * quantity.SizeMiB,
 			Type:       "83,0FC63DAF-8483-4772-8E79-3D69D8477DE4",
-			Role:       "system-data",
-			Label:      "ubuntu-data",
+			Role:       "system-save",
+			Label:      "ubuntu-save",
 			Filesystem: "ext4",
 		},
 		StartOffset: 1260388352,
 		Index:       3,
 	},
+	Size: 128 * quantity.SizeMiB,
+}
+
+var mockOnDiskStructureWritable = gadget.OnDiskStructure{
+	Node: "/dev/node4",
+	LaidOutStructure: gadget.LaidOutStructure{
+		VolumeStructure: &gadget.VolumeStructure{
+			Name:       "Writable",
+			Size:       1200 * quantity.SizeMiB,
+			Type:       "83,0FC63DAF-8483-4772-8E79-3D69D8477DE4",
+			Role:       "system-data",
+			Label:      "ubuntu-data",
+			Filesystem: "ext4",
+		},
+		StartOffset: 1394606080,
+		Index:       4,
+	},
+	// expanded to fill the disk
+	Size: 2*quantity.SizeGiB + 717*quantity.SizeMiB + 1031680,
 }
 
 func (s *ondiskTestSuite) TestDeviceInfoGPT(c *C) {
@@ -170,7 +193,7 @@ func (s *ondiskTestSuite) TestDeviceInfoGPT(c *C) {
 	dl, err := gadget.OnDiskVolumeFromDevice("/dev/node")
 	c.Assert(err, IsNil)
 	c.Assert(cmdSfdisk.Calls(), DeepEquals, [][]string{
-		{"sfdisk", "--json", "-d", "/dev/node"},
+		{"sfdisk", "--json", "/dev/node"},
 	})
 	c.Assert(cmdLsblk.Calls(), DeepEquals, [][]string{
 		{"lsblk", "--fs", "--json", "/dev/node1"},
@@ -180,8 +203,8 @@ func (s *ondiskTestSuite) TestDeviceInfoGPT(c *C) {
 	c.Assert(dl.Schema, Equals, "gpt")
 	c.Assert(dl.ID, Equals, "9151F25B-CDF0-48F1-9EDE-68CBD616E2CA")
 	c.Assert(dl.Device, Equals, "/dev/node")
-	c.Assert(dl.SectorSize, Equals, gadget.Size(512))
-	c.Assert(dl.Size, Equals, gadget.Size(8388575*512))
+	c.Assert(dl.SectorSize, Equals, quantity.Size(512))
+	c.Assert(dl.Size, Equals, quantity.Size(8388575*512))
 	c.Assert(len(dl.Structure), Equals, 2)
 
 	c.Assert(dl.Structure, DeepEquals, []gadget.OnDiskStructure{
@@ -227,7 +250,8 @@ echo '{
       "partitions": [
          {"node": "/dev/node1", "start": 4096, "size": 2457600, "type": "c"},
          {"node": "/dev/node2", "start": 2461696, "size": 1048576, "type": "d"},
-         {"node": "/dev/node3", "start": 3510272, "size": 1048576, "type": "d"}
+         {"node": "/dev/node3", "start": 3510272, "size": 1048576, "type": "d"},
+         {"node": "/dev/node4", "start": 4558848, "size": 1048576, "type": "d"}
       ]
    }
 }'`
@@ -239,6 +263,9 @@ echo '{
     "blockdevices": [ {"name": "node2", "fstype": "vfat", "label": "ubuntu-boot", "uuid": "A644-B808", "mountpoint": null} ]
 }'
 [ "$3" == "/dev/node3" ] && echo '{
+    "blockdevices": [ {"name": "node3", "fstype": "ext4", "label": "ubuntu-save", "mountpoint": null} ]
+}'
+[ "$3" == "/dev/node4" ] && echo '{
     "blockdevices": [ {"name": "node3", "fstype": "ext4", "label": "ubuntu-data", "mountpoint": null} ]
 }'
 exit 0`
@@ -255,12 +282,13 @@ exit 0`
 	dl, err := gadget.OnDiskVolumeFromDevice("/dev/node")
 	c.Assert(err, IsNil)
 	c.Assert(cmdSfdisk.Calls(), DeepEquals, [][]string{
-		{"sfdisk", "--json", "-d", "/dev/node"},
+		{"sfdisk", "--json", "/dev/node"},
 	})
 	c.Assert(cmdLsblk.Calls(), DeepEquals, [][]string{
 		{"lsblk", "--fs", "--json", "/dev/node1"},
 		{"lsblk", "--fs", "--json", "/dev/node2"},
 		{"lsblk", "--fs", "--json", "/dev/node3"},
+		{"lsblk", "--fs", "--json", "/dev/node4"},
 	})
 	c.Assert(cmdBlockdev.Calls(), DeepEquals, [][]string{
 		{"blockdev", "--getsz", "/dev/node"},
@@ -269,9 +297,9 @@ exit 0`
 	c.Assert(dl.ID, Equals, "")
 	c.Assert(dl.Schema, Equals, "dos")
 	c.Assert(dl.Device, Equals, "/dev/node")
-	c.Assert(dl.SectorSize, Equals, gadget.Size(512))
-	c.Assert(dl.Size, Equals, gadget.Size(12345670*512))
-	c.Assert(len(dl.Structure), Equals, 3)
+	c.Assert(dl.SectorSize, Equals, quantity.Size(512))
+	c.Assert(dl.Size, Equals, quantity.Size(12345670*512))
+	c.Assert(len(dl.Structure), Equals, 4)
 
 	c.Assert(dl.Structure, DeepEquals, []gadget.OnDiskStructure{
 		{
@@ -285,8 +313,7 @@ exit 0`
 				StartOffset: 0x200000,
 				Index:       1,
 			},
-			Node:                 "/dev/node1",
-			CreatedDuringInstall: false,
+			Node: "/dev/node1",
 		},
 		{
 			LaidOutStructure: gadget.LaidOutStructure{
@@ -299,8 +326,20 @@ exit 0`
 				StartOffset: 0x4b200000,
 				Index:       2,
 			},
-			Node:                 "/dev/node2",
-			CreatedDuringInstall: true,
+			Node: "/dev/node2",
+		},
+		{
+			LaidOutStructure: gadget.LaidOutStructure{
+				VolumeStructure: &gadget.VolumeStructure{
+					Size:       0x20000000,
+					Label:      "ubuntu-save",
+					Type:       "0D",
+					Filesystem: "ext4",
+				},
+				StartOffset: 0x6b200000,
+				Index:       3,
+			},
+			Node: "/dev/node3",
 		},
 		{
 			LaidOutStructure: gadget.LaidOutStructure{
@@ -310,11 +349,10 @@ exit 0`
 					Type:       "0D",
 					Filesystem: "ext4",
 				},
-				StartOffset: 0x6b200000,
-				Index:       3,
+				StartOffset: 0x8b200000,
+				Index:       4,
 			},
-			Node:                 "/dev/node3",
-			CreatedDuringInstall: true,
+			Node: "/dev/node4",
 		},
 	})
 }
@@ -423,9 +461,11 @@ func (s *ondiskTestSuite) TestBuildPartitionList(c *C) {
 	// the expected expanded writable partition size is:
 	// start offset = (2M + 1200M), expanded size in sectors = (8388575*512 - start offset)/512
 	sfdiskInput, create := gadget.BuildPartitionList(dl, pv)
-	c.Assert(sfdiskInput.String(), Equals, `/dev/node3 : start=     2461696, size=     5926879, type=0FC63DAF-8483-4772-8E79-3D69D8477DE4, name="Writable", attrs="GUID:59"
+	c.Assert(sfdiskInput.String(), Equals,
+		`/dev/node3 : start=     2461696, size=      262144, type=0FC63DAF-8483-4772-8E79-3D69D8477DE4, name="Save"
+/dev/node4 : start=     2723840, size=     5664735, type=0FC63DAF-8483-4772-8E79-3D69D8477DE4, name="Writable"
 `)
-	c.Assert(create, DeepEquals, []gadget.OnDiskStructure{mockOnDiskStructureWritable})
+	c.Assert(create, DeepEquals, []gadget.OnDiskStructure{mockOnDiskStructureSave, mockOnDiskStructureWritable})
 }
 
 func (s *ondiskTestSuite) TestUpdatePartitionList(c *C) {
@@ -487,142 +527,6 @@ exit 0`
 	c.Assert(dl.Structure[1].Node, Equals, "/dev/node2")
 }
 
-func (s *ondiskTestSuite) TestCreatedDuringInstallGPT(c *C) {
-	cmdLsblk := testutil.MockCommand(c, "lsblk", `echo '{ "blockdevices": [ {"fstype":"ext4", "label":null} ] }'`)
-	defer cmdLsblk.Restore()
-
-	ptable := gadget.SFDiskPartitionTable{
-		Label:    "gpt",
-		ID:       "9151F25B-CDF0-48F1-9EDE-68CBD616E2CA",
-		Device:   "/dev/node",
-		Unit:     "sectors",
-		FirstLBA: 34,
-		LastLBA:  8388574,
-		Partitions: []gadget.SFDiskPartition{
-			{
-				Node:  "/dev/node1",
-				Start: 1024,
-				Size:  1024,
-				Type:  "0fc63daf-8483-4772-8e79-3d69d8477de4",
-				UUID:  "641764aa-a680-4d36-a7ad-f7bd01fd8d12",
-				Name:  "Linux filesystem",
-			},
-			{
-				Node:  "/dev/node2",
-				Start: 2048,
-				Size:  2048,
-				Type:  "0657FD6D-A4AB-43C4-84E5-0933C84B4F4F",
-				UUID:  "7ea3a75a-3f6d-4647-8134-89ae61fe88d5",
-				Name:  "Linux swap",
-			},
-			{
-				Node:  "/dev/node3",
-				Start: 8192,
-				Size:  8192,
-				Type:  "21686148-6449-6E6F-744E-656564454649",
-				UUID:  "30a26851-4b08-4b8d-8aea-f686e723ed8c",
-				Name:  "BIOS boot partition",
-			},
-			{
-				Node:  "/dev/node4",
-				Start: 16384,
-				Size:  16384,
-				Type:  "0fc63daf-8483-4772-8e79-3d69d8477de4",
-				UUID:  "8ab3e8fd-d53d-4d72-9c5e-56146915fd07",
-				Name:  "Another Linux filesystem",
-			},
-		},
-	}
-	dl, err := gadget.OnDiskVolumeFromPartitionTable(ptable)
-	c.Assert(err, IsNil)
-	list := gadget.CreatedDuringInstall(dl)
-	c.Assert(list, HasLen, 0)
-
-	// Set attribute bit for all partitions except the last one
-	for i := 0; i < len(ptable.Partitions)-1; i++ {
-		ptable.Partitions[i].Attrs = "RequiredPartition LegacyBIOSBootable GUID:58,59"
-	}
-
-	dl, err = gadget.OnDiskVolumeFromPartitionTable(ptable)
-	c.Assert(err, IsNil)
-	list = gadget.CreatedDuringInstall(dl)
-	c.Assert(list, DeepEquals, []string{"/dev/node1", "/dev/node2"})
-}
-
-func (s *ondiskTestSuite) TestCreatedDuringInstallMBR(c *C) {
-	cmdLsblk := testutil.MockCommand(c, "lsblk", `
-what=
-shift 2
-case "$1" in
-   /dev/node1)
-      what='{"name": "node1", "fstype":"ext4", "label":"ubuntu-seed"}'
-      ;;
-   /dev/node2)
-      what='{"name": "node2", "fstype":"vfat", "label":"ubuntu-boot"}'
-      ;;
-   /dev/node3)
-      what='{"name": "node3", "fstype":null, "label":null}'
-      ;;
-   /dev/node4)
-      what='{"name": "node4", "fstype":"ext4", "label":"ubuntu-data"}'
-      ;;
-  *)
-    echo "unexpected call"
-    exit 1
-esac
-
-cat <<EOF
-{
-"blockdevices": [
-   $what
-  ]
-}
-EOF`)
-	defer cmdLsblk.Restore()
-	cmdBlockdev := testutil.MockCommand(c, "blockdev", `echo '1234567'`)
-	defer cmdBlockdev.Restore()
-
-	ptable := gadget.SFDiskPartitionTable{
-		Label:  "dos",
-		Device: "/dev/node",
-		Unit:   "sectors",
-		Partitions: []gadget.SFDiskPartition{
-			{
-				// ubuntu-seed
-				Node:  "/dev/node1",
-				Start: 1024,
-				Size:  1024,
-				Type:  "0a",
-			},
-			{
-				// ubuntu-boot
-				Node:  "/dev/node2",
-				Start: 2048,
-				Size:  2048,
-				Type:  "b",
-			},
-			{
-				// unlabeled
-				Node:  "/dev/node3",
-				Start: 8192,
-				Size:  8192,
-				Type:  "c",
-			},
-			{
-				// ubuntu-data
-				Node:  "/dev/node4",
-				Start: 16384,
-				Size:  16384,
-				Type:  "0d",
-			},
-		},
-	}
-	dl, err := gadget.OnDiskVolumeFromPartitionTable(ptable)
-	c.Assert(err, IsNil)
-	list := gadget.CreatedDuringInstall(dl)
-	c.Assert(list, DeepEquals, []string{"/dev/node2", "/dev/node4"})
-}
-
 func (s *ondiskTestSuite) TestFilesystemInfo(c *C) {
 	cmd := testutil.MockCommand(c, "lsblk", `echo '{
    "blockdevices": [
