@@ -31,6 +31,7 @@ import (
 
 	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/gadget"
+	"github.com/snapcore/snapd/gadget/quantity"
 	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/testutil"
@@ -121,19 +122,19 @@ func (u *updateTestSuite) TestCanUpdateSize(c *C) {
 		{
 			// size change
 			from: gadget.LaidOutStructure{
-				VolumeStructure: &gadget.VolumeStructure{Size: 1 * gadget.SizeMiB},
+				VolumeStructure: &gadget.VolumeStructure{Size: 1 * quantity.SizeMiB},
 			},
 			to: gadget.LaidOutStructure{
-				VolumeStructure: &gadget.VolumeStructure{Size: 1*gadget.SizeMiB + 1*gadget.SizeKiB},
+				VolumeStructure: &gadget.VolumeStructure{Size: 1*quantity.SizeMiB + 1*quantity.SizeKiB},
 			},
 			err: "cannot change structure size from [0-9]+ to [0-9]+",
 		}, {
 			// size change
 			from: gadget.LaidOutStructure{
-				VolumeStructure: &gadget.VolumeStructure{Size: 1 * gadget.SizeMiB},
+				VolumeStructure: &gadget.VolumeStructure{Size: 1 * quantity.SizeMiB},
 			},
 			to: gadget.LaidOutStructure{
-				VolumeStructure: &gadget.VolumeStructure{Size: 1 * gadget.SizeMiB},
+				VolumeStructure: &gadget.VolumeStructure{Size: 1 * quantity.SizeMiB},
 			},
 			err: "",
 		},
@@ -247,22 +248,22 @@ func (u *updateTestSuite) TestCanUpdateOffset(c *C) {
 		{
 			// explicitly declared start offset change
 			from: gadget.LaidOutStructure{
-				VolumeStructure: &gadget.VolumeStructure{Size: 1 * gadget.SizeMiB, Offset: asSizePtr(1024)},
+				VolumeStructure: &gadget.VolumeStructure{Size: 1 * quantity.SizeMiB, Offset: asSizePtr(1024)},
 				StartOffset:     1024,
 			},
 			to: gadget.LaidOutStructure{
-				VolumeStructure: &gadget.VolumeStructure{Size: 1 * gadget.SizeMiB, Offset: asSizePtr(2048)},
+				VolumeStructure: &gadget.VolumeStructure{Size: 1 * quantity.SizeMiB, Offset: asSizePtr(2048)},
 				StartOffset:     2048,
 			},
 			err: "cannot change structure offset from [0-9]+ to [0-9]+",
 		}, {
 			// explicitly declared start offset in new structure
 			from: gadget.LaidOutStructure{
-				VolumeStructure: &gadget.VolumeStructure{Size: 1 * gadget.SizeMiB, Offset: nil},
+				VolumeStructure: &gadget.VolumeStructure{Size: 1 * quantity.SizeMiB, Offset: nil},
 				StartOffset:     1024,
 			},
 			to: gadget.LaidOutStructure{
-				VolumeStructure: &gadget.VolumeStructure{Size: 1 * gadget.SizeMiB, Offset: asSizePtr(2048)},
+				VolumeStructure: &gadget.VolumeStructure{Size: 1 * quantity.SizeMiB, Offset: asSizePtr(2048)},
 				StartOffset:     2048,
 			},
 			err: "cannot change structure offset from unspecified to [0-9]+",
@@ -270,23 +271,23 @@ func (u *updateTestSuite) TestCanUpdateOffset(c *C) {
 			// explicitly declared start offset in old structure,
 			// missing from new
 			from: gadget.LaidOutStructure{
-				VolumeStructure: &gadget.VolumeStructure{Size: 1 * gadget.SizeMiB, Offset: asSizePtr(1024)},
+				VolumeStructure: &gadget.VolumeStructure{Size: 1 * quantity.SizeMiB, Offset: asSizePtr(1024)},
 				StartOffset:     1024,
 			},
 			to: gadget.LaidOutStructure{
-				VolumeStructure: &gadget.VolumeStructure{Size: 1 * gadget.SizeMiB, Offset: nil},
+				VolumeStructure: &gadget.VolumeStructure{Size: 1 * quantity.SizeMiB, Offset: nil},
 				StartOffset:     2048,
 			},
 			err: "cannot change structure offset from [0-9]+ to unspecified",
 		}, {
 			// start offset changed due to layout
 			from: gadget.LaidOutStructure{
-				VolumeStructure: &gadget.VolumeStructure{Size: 1 * gadget.SizeMiB},
-				StartOffset:     1 * gadget.SizeMiB,
+				VolumeStructure: &gadget.VolumeStructure{Size: 1 * quantity.SizeMiB},
+				StartOffset:     1 * quantity.SizeMiB,
 			},
 			to: gadget.LaidOutStructure{
-				VolumeStructure: &gadget.VolumeStructure{Size: 1 * gadget.SizeMiB},
-				StartOffset:     2 * gadget.SizeMiB,
+				VolumeStructure: &gadget.VolumeStructure{Size: 1 * quantity.SizeMiB},
+				StartOffset:     2 * quantity.SizeMiB,
 			},
 			err: "cannot change structure start offset from [0-9]+ to [0-9]+",
 		},
@@ -645,14 +646,14 @@ func updateDataSet(c *C) (oldData gadget.GadgetData, newData gadget.GadgetData,
 	// prepare the stage
 	bareStruct := gadget.VolumeStructure{
 		Name: "first",
-		Size: 5 * gadget.SizeMiB,
+		Size: 5 * quantity.SizeMiB,
 		Content: []gadget.VolumeContent{
 			{Image: "first.img"},
 		},
 	}
 	fsStruct := gadget.VolumeStructure{
 		Name:       "second",
-		Size:       10 * gadget.SizeMiB,
+		Size:       10 * quantity.SizeMiB,
 		Filesystem: "ext4",
 		Content: []gadget.VolumeContent{
 			{Source: "/second-content", Target: "/"},
@@ -660,7 +661,7 @@ func updateDataSet(c *C) (oldData gadget.GadgetData, newData gadget.GadgetData,
 	}
 	lastStruct := gadget.VolumeStructure{
 		Name:       "third",
-		Size:       5 * gadget.SizeMiB,
+		Size:       5 * quantity.SizeMiB,
 		Filesystem: "vfat",
 		Content: []gadget.VolumeContent{
 			{Source: "/third-content", Target: "/"},
@@ -688,15 +689,15 @@ func updateDataSet(c *C) (oldData gadget.GadgetData, newData gadget.GadgetData,
 	}
 
 	oldRootDir := c.MkDir()
-	makeSizedFile(c, filepath.Join(oldRootDir, "first.img"), gadget.SizeMiB, nil)
+	makeSizedFile(c, filepath.Join(oldRootDir, "first.img"), quantity.SizeMiB, nil)
 	makeSizedFile(c, filepath.Join(oldRootDir, "/second-content/foo"), 0, nil)
 	makeSizedFile(c, filepath.Join(oldRootDir, "/third-content/bar"), 0, nil)
 	oldData = gadget.GadgetData{Info: oldInfo, RootDir: oldRootDir}
 
 	newRootDir := c.MkDir()
-	makeSizedFile(c, filepath.Join(newRootDir, "first.img"), 900*gadget.SizeKiB, nil)
-	makeSizedFile(c, filepath.Join(newRootDir, "/second-content/foo"), gadget.SizeKiB, nil)
-	makeSizedFile(c, filepath.Join(newRootDir, "/third-content/bar"), gadget.SizeKiB, nil)
+	makeSizedFile(c, filepath.Join(newRootDir, "first.img"), 900*quantity.SizeKiB, nil)
+	makeSizedFile(c, filepath.Join(newRootDir, "/second-content/foo"), quantity.SizeKiB, nil)
+	makeSizedFile(c, filepath.Join(newRootDir, "/third-content/bar"), quantity.SizeKiB, nil)
 	newData = gadget.GadgetData{Info: newInfo, RootDir: newRootDir}
 
 	rollbackDir = c.MkDir()
@@ -711,8 +712,8 @@ type mockUpdateProcessObserver struct {
 }
 
 func (m *mockUpdateProcessObserver) Observe(op gadget.ContentOperation, sourceStruct *gadget.LaidOutStructure,
-	targetRootDir, relativeTargetPath string, data *gadget.ContentChange) (bool, error) {
-	return false, errors.New("unexpected call")
+	targetRootDir, relativeTargetPath string, data *gadget.ContentChange) (gadget.ContentChangeAction, error) {
+	return gadget.ChangeAbort, errors.New("unexpected call")
 }
 
 func (m *mockUpdateProcessObserver) BeforeWrite() error {
@@ -745,21 +746,21 @@ func (u *updateTestSuite) TestUpdateApplyHappy(c *C) {
 		case 0:
 			c.Check(ps.Name, Equals, "first")
 			c.Check(ps.HasFilesystem(), Equals, false)
-			c.Check(ps.Size, Equals, 5*gadget.SizeMiB)
+			c.Check(ps.Size, Equals, 5*quantity.SizeMiB)
 			c.Check(ps.IsPartition(), Equals, true)
 			// non MBR start offset defaults to 1MiB
-			c.Check(ps.StartOffset, Equals, 1*gadget.SizeMiB)
+			c.Check(ps.StartOffset, Equals, 1*quantity.SizeMiB)
 			c.Assert(ps.LaidOutContent, HasLen, 1)
 			c.Check(ps.LaidOutContent[0].Image, Equals, "first.img")
-			c.Check(ps.LaidOutContent[0].Size, Equals, 900*gadget.SizeKiB)
+			c.Check(ps.LaidOutContent[0].Size, Equals, 900*quantity.SizeKiB)
 		case 1:
 			c.Check(ps.Name, Equals, "second")
 			c.Check(ps.HasFilesystem(), Equals, true)
 			c.Check(ps.Filesystem, Equals, "ext4")
 			c.Check(ps.IsPartition(), Equals, true)
-			c.Check(ps.Size, Equals, 10*gadget.SizeMiB)
+			c.Check(ps.Size, Equals, 10*quantity.SizeMiB)
 			// foo's start offset + foo's size
-			c.Check(ps.StartOffset, Equals, (1+5)*gadget.SizeMiB)
+			c.Check(ps.StartOffset, Equals, (1+5)*quantity.SizeMiB)
 			c.Assert(ps.LaidOutContent, HasLen, 0)
 			c.Assert(ps.Content, HasLen, 1)
 			c.Check(ps.Content[0].Source, Equals, "/second-content")
@@ -850,7 +851,7 @@ func (u *updateTestSuite) TestUpdateApplyErrorLayout(c *C) {
 	// prepare the stage
 	bareStruct := gadget.VolumeStructure{
 		Name: "foo",
-		Size: 5 * gadget.SizeMiB,
+		Size: 5 * quantity.SizeMiB,
 		Content: []gadget.VolumeContent{
 			{Image: "first.img"},
 		},
@@ -889,7 +890,7 @@ func (u *updateTestSuite) TestUpdateApplyErrorLayout(c *C) {
 	err := gadget.Update(oldData, newData, rollbackDir, nil, nil)
 	c.Assert(err, ErrorMatches, `cannot lay out the new volume: cannot lay out structure #0 \("foo"\): content "first.img": .* no such file or directory`)
 
-	makeSizedFile(c, filepath.Join(newRootDir, "first.img"), gadget.SizeMiB, nil)
+	makeSizedFile(c, filepath.Join(newRootDir, "first.img"), quantity.SizeMiB, nil)
 
 	// Update does not error out when when the bare struct data of the old volume is missing
 	err = gadget.Update(oldData, newData, rollbackDir, nil, nil)
@@ -900,7 +901,7 @@ func (u *updateTestSuite) TestUpdateApplyErrorIllegalVolumeUpdate(c *C) {
 	// prepare the stage
 	bareStruct := gadget.VolumeStructure{
 		Name: "foo",
-		Size: 5 * gadget.SizeMiB,
+		Size: 5 * quantity.SizeMiB,
 		Content: []gadget.VolumeContent{
 			{Image: "first.img"},
 		},
@@ -936,8 +937,8 @@ func (u *updateTestSuite) TestUpdateApplyErrorIllegalVolumeUpdate(c *C) {
 
 	rollbackDir := c.MkDir()
 
-	makeSizedFile(c, filepath.Join(oldRootDir, "first.img"), gadget.SizeMiB, nil)
-	makeSizedFile(c, filepath.Join(newRootDir, "first.img"), 900*gadget.SizeKiB, nil)
+	makeSizedFile(c, filepath.Join(oldRootDir, "first.img"), quantity.SizeMiB, nil)
+	makeSizedFile(c, filepath.Join(newRootDir, "first.img"), 900*quantity.SizeKiB, nil)
 
 	err := gadget.Update(oldData, newData, rollbackDir, nil, nil)
 	c.Assert(err, ErrorMatches, `cannot apply update to volume: cannot change the number of structures within volume from 1 to 2`)
@@ -947,7 +948,7 @@ func (u *updateTestSuite) TestUpdateApplyErrorIllegalStructureUpdate(c *C) {
 	// prepare the stage
 	bareStruct := gadget.VolumeStructure{
 		Name: "foo",
-		Size: 5 * gadget.SizeMiB,
+		Size: 5 * quantity.SizeMiB,
 		Content: []gadget.VolumeContent{
 			{Image: "first.img"},
 		},
@@ -955,7 +956,7 @@ func (u *updateTestSuite) TestUpdateApplyErrorIllegalStructureUpdate(c *C) {
 	fsStruct := gadget.VolumeStructure{
 		Name:       "foo",
 		Filesystem: "ext4",
-		Size:       5 * gadget.SizeMiB,
+		Size:       5 * quantity.SizeMiB,
 		Content: []gadget.VolumeContent{
 			{Source: "/", Target: "/"},
 		},
@@ -988,7 +989,7 @@ func (u *updateTestSuite) TestUpdateApplyErrorIllegalStructureUpdate(c *C) {
 
 	rollbackDir := c.MkDir()
 
-	makeSizedFile(c, filepath.Join(oldRootDir, "first.img"), gadget.SizeMiB, nil)
+	makeSizedFile(c, filepath.Join(oldRootDir, "first.img"), quantity.SizeMiB, nil)
 
 	err := gadget.Update(oldData, newData, rollbackDir, nil, nil)
 	c.Assert(err, ErrorMatches, `cannot update volume structure #0 \("foo"\): cannot change a bare structure to filesystem one`)
@@ -998,7 +999,7 @@ func (u *updateTestSuite) TestUpdateApplyErrorDifferentVolume(c *C) {
 	// prepare the stage
 	bareStruct := gadget.VolumeStructure{
 		Name: "foo",
-		Size: 5 * gadget.SizeMiB,
+		Size: 5 * quantity.SizeMiB,
 		Content: []gadget.VolumeContent{
 			{Image: "first.img"},
 		},
@@ -1037,7 +1038,7 @@ func (u *updateTestSuite) TestUpdateApplyUpdatesAreOptInWithDefaultPolicy(c *C)
 	// prepare the stage
 	bareStruct := gadget.VolumeStructure{
 		Name: "foo",
-		Size: 5 * gadget.SizeMiB,
+		Size: 5 * quantity.SizeMiB,
 		Content: []gadget.VolumeContent{
 			{Image: "first.img"},
 		},
@@ -1057,13 +1058,13 @@ func (u *updateTestSuite) TestUpdateApplyUpdatesAreOptInWithDefaultPolicy(c *C)
 
 	oldRootDir := c.MkDir()
 	oldData := gadget.GadgetData{Info: oldInfo, RootDir: oldRootDir}
-	makeSizedFile(c, filepath.Join(oldRootDir, "first.img"), gadget.SizeMiB, nil)
+	makeSizedFile(c, filepath.Join(oldRootDir, "first.img"), quantity.SizeMiB, nil)
 
 	newRootDir := c.MkDir()
 	// same volume description
 	newData := gadget.GadgetData{Info: oldInfo, RootDir: newRootDir}
 	// different content, but updates are opt in
-	makeSizedFile(c, filepath.Join(newRootDir, "first.img"), 900*gadget.SizeKiB, nil)
+	makeSizedFile(c, filepath.Join(newRootDir, "first.img"), 900*quantity.SizeKiB, nil)
 
 	rollbackDir := c.MkDir()
 
@@ -1087,7 +1088,7 @@ func policyDataSet(c *C) (oldData gadget.GadgetData, newData gadget.GadgetData,
 	noPartitionStruct := gadget.VolumeStructure{
 		Name: "no-partition",
 		Type: "bare",
-		Size: 5 * gadget.SizeMiB,
+		Size: 5 * quantity.SizeMiB,
 		Content: []gadget.VolumeContent{
 			{Image: "first.img"},
 		},
@@ -1416,9 +1417,9 @@ func (u *updateTestSuite) TestUpdaterForStructure(c *C) {
 	psBare := &gadget.LaidOutStructure{
 		VolumeStructure: &gadget.VolumeStructure{
 			Filesystem: "none",
-			Size:       10 * gadget.SizeMiB,
+			Size:       10 * quantity.SizeMiB,
 		},
-		StartOffset: 1 * gadget.SizeMiB,
+		StartOffset: 1 * quantity.SizeMiB,
 	}
 	updater, err := gadget.UpdaterForStructure(psBare, gadgetRootDir, rollbackDir, nil)
 	c.Assert(err, IsNil)
@@ -1427,10 +1428,10 @@ func (u *updateTestSuite) TestUpdaterForStructure(c *C) {
 	psFs := &gadget.LaidOutStructure{
 		VolumeStructure: &gadget.VolumeStructure{
 			Filesystem: "ext4",
-			Size:       10 * gadget.SizeMiB,
+			Size:       10 * quantity.SizeMiB,
 			Label:      "writable",
 		},
-		StartOffset: 1 * gadget.SizeMiB,
+		StartOffset: 1 * quantity.SizeMiB,
 	}
 	updater, err = gadget.UpdaterForStructure(psFs, gadgetRootDir, rollbackDir, nil)
 	c.Assert(err, IsNil)
