@@ -65,12 +65,19 @@ type ConfinementOptions struct {
 	Classic bool
 }
 
+// SecurityBackendOptions carries extra flags that affect initialization of the
+// backends.
+type SecurityBackendOptions struct {
+	// Preseed flag is set when snapd runs in preseed mode.
+	Preseed bool
+}
+
 // SecurityBackend abstracts interactions between the interface system and the
 // needs of a particular security system.
 type SecurityBackend interface {
 	// Initialize performs any initialization required by the backend.
 	// It is called during snapd startup process.
-	Initialize() error
+	Initialize(opts *SecurityBackendOptions) error
 
 	// Name returns the name of the backend.
 	// This is intended for diagnostic messages.
@@ -96,3 +103,11 @@ type SecurityBackend interface {
 	// SandboxFeatures returns a list of tags that identify sandbox features.
 	SandboxFeatures() []string
 }
+
+// SecurityBackendSetupMany interface may be implemented by backends that can optimize their operations
+// when setting up multiple snaps at once.
+type SecurityBackendSetupMany interface {
+	// SetupMany creates and loads apparmor profiles of multiple snaps. It tries to process all snaps and doesn't interrupt processing
+	// on errors of individual snaps.
+	SetupMany(snaps []*snap.Info, confinement func(snapName string) ConfinementOptions, repo *Repository, tm timings.Measurer) []error
+}
