@@ -26,8 +26,10 @@ import (
 	"github.com/snapcore/snapd/interfaces"
 	"github.com/snapcore/snapd/interfaces/apparmor"
 	"github.com/snapcore/snapd/interfaces/kmod"
+	"github.com/snapcore/snapd/interfaces/mount"
 	"github.com/snapcore/snapd/interfaces/seccomp"
 	"github.com/snapcore/snapd/interfaces/udev"
+	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/snap"
 )
 
@@ -55,6 +57,9 @@ type commonInterface struct {
 	connectedPlugUDev      []string
 	rejectAutoConnectPairs bool
 
+	connectedPlugUpdateNSAppArmor string
+	connectedPlugMount            []osutil.MountEntry
+
 	connectedPlugKModModules []string
 	connectedSlotKModModules []string
 	permanentPlugKModModules []string
@@ -64,6 +69,8 @@ type commonInterface struct {
 	suppressPtraceTrace  bool
 	suppressHomeIx       bool
 	controlsDeviceCgroup bool
+
+	serviceSnippets []string
 }
 
 // Name returns the interface name.
@@ -83,6 +90,10 @@ func (iface *commonInterface) StaticInfo() interfaces.StaticInfo {
 	}
 }
 
+func (iface *commonInterface) ServicePermanentPlug(plug *snap.PlugInfo) []string {
+	return iface.serviceSnippets
+}
+
 func (iface *commonInterface) AppArmorConnectedPlug(spec *apparmor.Specification, plug *interfaces.ConnectedPlug, slot *interfaces.ConnectedSlot) error {
 	if iface.usesPtraceTrace {
 		spec.SetUsesPtraceTrace()
@@ -92,8 +103,11 @@ func (iface *commonInterface) AppArmorConnectedPlug(spec *apparmor.Specification
 	if iface.suppressHomeIx {
 		spec.SetSuppressHomeIx()
 	}
-	if iface.connectedPlugAppArmor != "" {
-		spec.AddSnippet(iface.connectedPlugAppArmor)
+	if snippet := iface.connectedPlugAppArmor; snippet != "" {
+		spec.AddSnippet(snippet)
+	}
+	if snippet := iface.connectedPlugUpdateNSAppArmor; snippet != "" {
+		spec.AddUpdateNS(snippet)
 	}
 	return nil
 }
@@ -125,6 +139,15 @@ func (iface *commonInterface) KModConnectedSlot(spec *kmod.Specification, plug *
 	return nil
 }
 
+func (iface *commonInterface) MountConnectedPlug(spec *mount.Specification, plug *interfaces.ConnectedPlug, slot *interfaces.ConnectedSlot) error {
+	for _, entry := range iface.connectedPlugMount {
+		if err := spec.AddMountEntry(entry); err != nil {
+			return err
+		}
+	}
+	return nil
+}
+
 func (iface *commonInterface) KModPermanentPlug(spec *kmod.Specification, plug *snap.PlugInfo) error {
 	for _, m := range iface.permanentPlugKModModules {
 		if err := spec.AddModule(m); err != nil {
