@@ -78,11 +78,11 @@ func (s *OpticalDriveInterfaceSuite) SetUpTest(c *C) {
 	consumingSnapInfo := snaptest.MockInfo(c, opticalDriveConsumerYaml, nil)
 
 	s.testPlugDefaultInfo = consumingSnapInfo.Plugs["optical-drive"]
-	s.testPlugDefault = interfaces.NewConnectedPlug(s.testPlugDefaultInfo, nil)
+	s.testPlugDefault = interfaces.NewConnectedPlug(s.testPlugDefaultInfo, nil, nil)
 	s.testPlugReadonlyInfo = consumingSnapInfo.Plugs["plug-for-readonly"]
-	s.testPlugReadonly = interfaces.NewConnectedPlug(s.testPlugReadonlyInfo, nil)
+	s.testPlugReadonly = interfaces.NewConnectedPlug(s.testPlugReadonlyInfo, nil, nil)
 	s.testPlugWritableInfo = consumingSnapInfo.Plugs["plug-for-writable"]
-	s.testPlugWritable = interfaces.NewConnectedPlug(s.testPlugWritableInfo, nil)
+	s.testPlugWritable = interfaces.NewConnectedPlug(s.testPlugWritableInfo, nil, nil)
 
 	s.slot, s.slotInfo = MockConnectedSlot(c, opticalDriveCoreYaml, nil, "optical-drive")
 }
@@ -93,13 +93,6 @@ func (s *OpticalDriveInterfaceSuite) TestName(c *C) {
 
 func (s *OpticalDriveInterfaceSuite) TestSanitizeSlot(c *C) {
 	c.Assert(interfaces.BeforePrepareSlot(s.iface, s.slotInfo), IsNil)
-	slot := &snap.SlotInfo{
-		Snap:      &snap.Info{SuggestedName: "some-snap"},
-		Name:      "optical-drive",
-		Interface: "optical-drive",
-	}
-	c.Assert(interfaces.BeforePrepareSlot(s.iface, slot), ErrorMatches,
-		"optical-drive slots are reserved for the core snap")
 }
 
 func (s *OpticalDriveInterfaceSuite) TestSanitizePlug(c *C) {
@@ -152,7 +145,7 @@ func (s *OpticalDriveInterfaceSuite) TestUDevSpec(c *C) {
 	c.Assert(spec.AddConnectedPlug(s.iface, s.testPlugDefault, s.slot), IsNil)
 	c.Assert(spec.AddConnectedPlug(s.iface, s.testPlugReadonly, s.slot), IsNil)
 	c.Assert(spec.AddConnectedPlug(s.iface, s.testPlugWritable, s.slot), IsNil)
-	c.Assert(spec.Snippets(), HasLen, 9) // three rules multiplied by three apps
+	c.Assert(spec.Snippets(), HasLen, 12) // four rules multiplied by three apps
 	c.Assert(spec.Snippets(), testutil.Contains, `# optical-drive
 KERNEL=="sr[0-9]*", TAG+="snap_consumer_app"`)
 	c.Assert(spec.Snippets(), testutil.Contains, `TAG=="snap_consumer_app", RUN+="/usr/lib/snapd/snap-device-helper $env{ACTION} snap_consumer_app $devpath $major:$minor"`)
