@@ -25,8 +25,10 @@ import (
 	"github.com/snapcore/snapd/interfaces"
 	"github.com/snapcore/snapd/interfaces/apparmor"
 	"github.com/snapcore/snapd/interfaces/builtin"
+	"github.com/snapcore/snapd/interfaces/mount"
 	"github.com/snapcore/snapd/interfaces/seccomp"
 	"github.com/snapcore/snapd/interfaces/udev"
+	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/release"
 	"github.com/snapcore/snapd/snap"
 	"github.com/snapcore/snapd/testutil"
@@ -36,6 +38,8 @@ type X11InterfaceSuite struct {
 	iface           interfaces.Interface
 	coreSlotInfo    *snap.SlotInfo
 	coreSlot        *interfaces.ConnectedSlot
+	corePlugInfo    *snap.PlugInfo
+	corePlug        *interfaces.ConnectedPlug
 	classicSlotInfo *snap.SlotInfo
 	classicSlot     *interfaces.ConnectedSlot
 	plugInfo        *snap.PlugInfo
@@ -57,8 +61,15 @@ apps:
 const x11CoreYaml = `name: x11
 version: 0
 apps:
- app1:
-  slots: [x11]
+ app:
+  slots: [x11-provider]
+  plugs: [x11-consumer]
+plugs:
+  x11-consumer:
+    interface: x11
+slots:
+  x11-provider:
+    interface: x11
 `
 
 // an x11 slot on the core snap (as automatically added on classic)
@@ -72,7 +83,8 @@ slots:
 
 func (s *X11InterfaceSuite) SetUpTest(c *C) {
 	s.plug, s.plugInfo = MockConnectedPlug(c, x11MockPlugSnapInfoYaml, nil, "x11")
-	s.coreSlot, s.coreSlotInfo = MockConnectedSlot(c, x11CoreYaml, nil, "x11")
+	s.coreSlot, s.coreSlotInfo = MockConnectedSlot(c, x11CoreYaml, nil, "x11-provider")
+	s.corePlug, s.corePlugInfo = MockConnectedPlug(c, x11CoreYaml, nil, "x11-consumer")
 	s.classicSlot, s.classicSlotInfo = MockConnectedSlot(c, x11ClassicYaml, nil, "x11")
 }
 
@@ -89,28 +101,80 @@ func (s *X11InterfaceSuite) TestSanitizePlug(c *C) {
 	c.Assert(interfaces.BeforePreparePlug(s.iface, s.plugInfo), IsNil)
 }
 
+func (s *X11InterfaceSuite) TestMountSpec(c *C) {
+	// case A: x11 slot is provided by the system
+	spec := &mount.Specification{}
+	c.Assert(spec.AddConnectedPlug(s.iface, s.plug, s.classicSlot), IsNil)
+	c.Assert(spec.MountEntries(), DeepEquals, []osutil.MountEntry{{
+		Name:    "/var/lib/snapd/hostfs/tmp/.X11-unix",
+		Dir:     "/tmp/.X11-unix",
+		Options: []string{"bind", "ro"},
+	}})
+	c.Assert(spec.UserMountEntries(), HasLen, 0)
+
+	// case B: x11 slot is provided by another snap on the system
+	spec = &mount.Specification{}
+	c.Assert(spec.AddConnectedPlug(s.iface, s.plug, s.coreSlot), IsNil)
+	c.Assert(spec.MountEntries(), DeepEquals, []osutil.MountEntry{{
+		Name:    "/var/lib/snapd/hostfs/tmp/snap.x11/tmp/.X11-unix",
+		Dir:     "/tmp/.X11-unix",
+		Options: []string{"bind", "ro"},
+	}})
+	c.Assert(spec.UserMountEntries(), HasLen, 0)
+
+	// case C: x11 slot is both provided and consumed by a snap on the system.
+	spec = &mount.Specification{}
+	c.Assert(spec.AddConnectedPlug(s.iface, s.corePlug, s.coreSlot), IsNil)
+	c.Assert(spec.MountEntries(), HasLen, 0)
+	c.Assert(spec.UserMountEntries(), HasLen, 0)
+}
+
 func (s *X11InterfaceSuite) TestAppArmorSpec(c *C) {
-	// on a core system with x11 slot coming from a regular app snap.
-	restore := release.MockOnClassic(false)
+	// case A: x11 slot is provided by the classic system
+	restore := release.MockOnClassic(true)
 	defer restore()
 
-	// connected plug to core slot
+	// Plug side connection permissions
 	spec := &apparmor.Specification{}
+	c.Assert(spec.AddConnectedPlug(s.iface, s.plug, s.classicSlot), IsNil)
+	c.Assert(spec.SecurityTags(), DeepEquals, []string{"snap.consumer.app"})
+	c.Assert(spec.SnippetForTag("snap.consumer.app"), testutil.Contains, "fontconfig")
+	c.Assert(spec.UpdateNS(), HasLen, 1)
+	c.Assert(spec.UpdateNS()[0], testutil.Contains, `mount options=(rw, bind) /var/lib/snapd/hostfs/tmp/.X11-unix/ -> /tmp/.X11-unix/,`)
+
+	// case B: x11 slot is provided by another snap on the system
+	restore = release.MockOnClassic(false)
+	defer restore()
+
+	// Plug side connection permissions
+	spec = &apparmor.Specification{}
 	c.Assert(spec.AddConnectedPlug(s.iface, s.plug, s.coreSlot), IsNil)
 	c.Assert(spec.SecurityTags(), DeepEquals, []string{"snap.consumer.app"})
 	c.Assert(spec.SnippetForTag("snap.consumer.app"), testutil.Contains, "fontconfig")
+	c.Assert(spec.UpdateNS(), HasLen, 1)
+	c.Assert(spec.UpdateNS()[0], testutil.Contains, `mount options=(rw, bind) /var/lib/snapd/hostfs/tmp/snap.x11/tmp/.X11-unix/ -> /tmp/.X11-unix/,`)
 
-	// connected core slot to plug
+	// Slot side connection permissions
 	spec = &apparmor.Specification{}
 	c.Assert(spec.AddConnectedSlot(s.iface, s.plug, s.coreSlot), IsNil)
-	c.Assert(spec.SecurityTags(), DeepEquals, []string{"snap.x11.app1"})
-	c.Assert(spec.SnippetForTag("snap.x11.app1"), testutil.Contains, `peer=(label="snap.consumer.app"),`)
+	c.Assert(spec.SecurityTags(), DeepEquals, []string{"snap.x11.app"})
+	c.Assert(spec.SnippetForTag("snap.x11.app"), testutil.Contains, `peer=(label="snap.consumer.app"),`)
+	c.Assert(spec.UpdateNS(), HasLen, 0)
 
-	// permanent core slot
+	// Slot side permantent permissions
 	spec = &apparmor.Specification{}
 	c.Assert(spec.AddPermanentSlot(s.iface, s.coreSlotInfo), IsNil)
-	c.Assert(spec.SecurityTags(), DeepEquals, []string{"snap.x11.app1"})
-	c.Assert(spec.SnippetForTag("snap.x11.app1"), testutil.Contains, "capability sys_tty_config,")
+	c.Assert(spec.SecurityTags(), DeepEquals, []string{"snap.x11.app"})
+	c.Assert(spec.SnippetForTag("snap.x11.app"), testutil.Contains, "capability sys_tty_config,")
+	c.Assert(spec.UpdateNS(), HasLen, 0)
+
+	// case C: x11 slot is both provided and consumed by a snap on the system.
+	spec = &apparmor.Specification{}
+	c.Assert(spec.AddConnectedPlug(s.iface, s.corePlug, s.coreSlot), IsNil)
+	c.Assert(spec.SecurityTags(), DeepEquals, []string{"snap.x11.app"})
+	c.Assert(spec.SnippetForTag("snap.x11.app"), testutil.Contains, "fontconfig")
+	// Self-connection does not need bind mounts, so no additional permissions are provided to snap-update-ns.
+	c.Assert(spec.UpdateNS(), HasLen, 0)
 }
 
 func (s *X11InterfaceSuite) TestAppArmorSpecOnClassic(c *C) {
@@ -163,8 +227,8 @@ func (s *X11InterfaceSuite) TestSecCompOnCore(c *C) {
 	c.Assert(err, IsNil)
 
 	// both app and x11 have secomp rules set
-	c.Assert(seccompSpec.SecurityTags(), DeepEquals, []string{"snap.consumer.app", "snap.x11.app1"})
-	c.Assert(seccompSpec.SnippetForTag("snap.x11.app1"), testutil.Contains, "listen\n")
+	c.Assert(seccompSpec.SecurityTags(), DeepEquals, []string{"snap.consumer.app", "snap.x11.app"})
+	c.Assert(seccompSpec.SnippetForTag("snap.x11.app"), testutil.Contains, "listen\n")
 	c.Assert(seccompSpec.SnippetForTag("snap.consumer.app"), testutil.Contains, "bind\n")
 }
 
@@ -177,16 +241,16 @@ func (s *X11InterfaceSuite) TestUDev(c *C) {
 	c.Assert(spec.AddPermanentSlot(s.iface, s.coreSlotInfo), IsNil)
 	c.Assert(spec.Snippets(), HasLen, 6)
 	c.Assert(spec.Snippets(), testutil.Contains, `# x11
-KERNEL=="event[0-9]*", TAG+="snap_x11_app1"`)
+KERNEL=="event[0-9]*", TAG+="snap_x11_app"`)
 	c.Assert(spec.Snippets(), testutil.Contains, `# x11
-KERNEL=="mice", TAG+="snap_x11_app1"`)
+KERNEL=="mice", TAG+="snap_x11_app"`)
 	c.Assert(spec.Snippets(), testutil.Contains, `# x11
-KERNEL=="mouse[0-9]*", TAG+="snap_x11_app1"`)
+KERNEL=="mouse[0-9]*", TAG+="snap_x11_app"`)
 	c.Assert(spec.Snippets(), testutil.Contains, `# x11
-KERNEL=="ts[0-9]*", TAG+="snap_x11_app1"`)
+KERNEL=="ts[0-9]*", TAG+="snap_x11_app"`)
 	c.Assert(spec.Snippets(), testutil.Contains, `# x11
-KERNEL=="tty[0-9]*", TAG+="snap_x11_app1"`)
-	c.Assert(spec.Snippets(), testutil.Contains, `TAG=="snap_x11_app1", RUN+="/usr/lib/snapd/snap-device-helper $env{ACTION} snap_x11_app1 $devpath $major:$minor"`)
+KERNEL=="tty[0-9]*", TAG+="snap_x11_app"`)
+	c.Assert(spec.Snippets(), testutil.Contains, `TAG=="snap_x11_app", RUN+="/usr/lib/snapd/snap-device-helper $env{ACTION} snap_x11_app $devpath $major:$minor"`)
 	c.Assert(spec.TriggeredSubsystems(), DeepEquals, []string{"input"})
 
 	// on a classic system with x11 slot coming from the core snap.
@@ -194,7 +258,7 @@ KERNEL=="tty[0-9]*", TAG+="snap_x11_app1"`)
 	defer restore()
 
 	spec = &udev.Specification{}
-	c.Assert(spec.AddPermanentSlot(s.iface, s.coreSlotInfo), IsNil)
+	c.Assert(spec.AddPermanentSlot(s.iface, s.classicSlotInfo), IsNil)
 	c.Assert(spec.Snippets(), HasLen, 0)
 	c.Assert(spec.TriggeredSubsystems(), IsNil)
 }
