@@ -27,7 +27,7 @@ import (
 	"github.com/snapcore/snapd/snap"
 )
 
-// Sanitize plug with a given snapd interface.
+// BeforePreparePlug sanitizes a plug with a given snapd interface.
 func BeforePreparePlug(iface Interface, plugInfo *snap.PlugInfo) error {
 	if iface.Name() != plugInfo.Interface {
 		return fmt.Errorf("cannot sanitize plug %q (interface %q) using interface %q",
@@ -40,6 +40,13 @@ func BeforePreparePlug(iface Interface, plugInfo *snap.PlugInfo) error {
 	return err
 }
 
+// ByName returns an Interface for the given interface name. Note that in order for
+// this to work properly, the package "interfaces/builtin" must also eventually be
+// imported to populate the full list of interfaces.
+var ByName = func(name string) (iface Interface, err error) {
+	panic("ByName is unset, import interfaces/builtin to initialize this")
+}
+
 // PlugRef is a reference to a plug.
 type PlugRef struct {
 	Snap string `json:"snap"`
@@ -197,6 +204,27 @@ type StaticInfo struct {
 	BaseDeclarationSlots string
 }
 
+// PermanentPlugServiceSnippets will return the set of snippets for the systemd
+// service unit that should be generated for a snap with the specified plug.
+// The list returned is not unique, callers must de-duplicate themselves.
+// The plug is provided because the snippet may depend on plug attributes for
+// example. The plug is sanitized before the snippets are returned.
+func PermanentPlugServiceSnippets(iface Interface, plug *snap.PlugInfo) (snips []string, err error) {
+	// sanitize the plug first
+	err = BeforePreparePlug(iface, plug)
+	if err != nil {
+		return nil, err
+	}
+
+	type serviceSnippetPlugger interface {
+		ServicePermanentPlug(plug *snap.PlugInfo) []string
+	}
+	if iface, ok := iface.(serviceSnippetPlugger); ok {
+		snips = iface.ServicePermanentPlug(plug)
+	}
+	return snips, nil
+}
+
 // StaticInfoOf returns the static-info of the given interface.
 func StaticInfoOf(iface Interface) (si StaticInfo) {
 	type metaDataProvider interface {
