@@ -52,6 +52,16 @@ func MockIsHomeUsingNFS(new func() (bool, error)) (restore func()) {
 	}
 }
 
+// MockIsRootWritableOverlay mocks the real implementation of
+// osutil.IsRootWritableOverlay
+func MockIsRootWritableOverlay(new func() (string, error)) (restore func()) {
+	old := isRootWritableOverlay
+	isRootWritableOverlay = new
+	return func() {
+		isRootWritableOverlay = old
+	}
+}
+
 func MockReadBuildID(mock func(p string) (string, error)) (restore func()) {
 	old := readBuildID
 	readBuildID = mock
