@@ -32,7 +32,6 @@ import (
 
 	. "gopkg.in/check.v1"
 
-	"github.com/snapcore/snapd/cmd"
 	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/interfaces"
 	"github.com/snapcore/snapd/interfaces/ifacetest"
@@ -42,6 +41,7 @@ import (
 	seccomp_sandbox "github.com/snapcore/snapd/sandbox/seccomp"
 	"github.com/snapcore/snapd/snap"
 	"github.com/snapcore/snapd/snap/snaptest"
+	"github.com/snapcore/snapd/snapdtool"
 	"github.com/snapcore/snapd/strutil"
 	"github.com/snapcore/snapd/testutil"
 	"github.com/snapcore/snapd/timings"
@@ -76,7 +76,7 @@ func (s *backendSuite) SetUpTest(c *C) {
 	err := os.MkdirAll(dirs.SnapSeccompDir, 0700)
 	c.Assert(err, IsNil)
 
-	s.restoreReadlink = cmd.MockOsReadlink(func(string) (string, error) {
+	s.restoreReadlink = snapdtool.MockOsReadlink(func(string) (string, error) {
 		// pretend that snapd is run from distro libexecdir
 		return filepath.Join(dirs.DistroLibExecDir, "snapd"), nil
 	})
@@ -149,7 +149,7 @@ func (s *backendSuite) TestInstallingSnapWritesHookProfiles(c *C) {
 }
 
 func (s *backendSuite) TestInstallingSnapWritesProfilesWithReexec(c *C) {
-	restore := cmd.MockOsReadlink(func(string) (string, error) {
+	restore := snapdtool.MockOsReadlink(func(string) (string, error) {
 		// simulate that we run snapd from core
 		return filepath.Join(dirs.SnapMountDir, "core/42/usr/lib/snapd/snapd"), nil
 	})
@@ -700,7 +700,7 @@ func (s *backendSuite) TestInitializationDuringBootstrap(c *C) {
 	// during bootstrap, before seeding, snapd/core snap is mounted at some
 	// random location under /tmp
 	tmpDir := c.MkDir()
-	restore := cmd.MockOsReadlink(func(string) (string, error) {
+	restore := snapdtool.MockOsReadlink(func(string) (string, error) {
 		return filepath.Join(tmpDir, "usr/lib/snapd/snapd"), nil
 	})
 	defer restore()
