@@ -22,6 +22,8 @@ package interfaces
 import (
 	"encoding/json"
 	"errors"
+	"fmt"
+	"io"
 	"io/ioutil"
 	"os"
 	"path/filepath"
@@ -29,13 +31,13 @@ import (
 	"strconv"
 	"strings"
 
-	"github.com/snapcore/snapd/cmd"
 	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/sandbox/apparmor"
 	"github.com/snapcore/snapd/sandbox/cgroup"
 	"github.com/snapcore/snapd/sandbox/seccomp"
+	"github.com/snapcore/snapd/snapdtool"
 )
 
 // ErrSystemKeyIncomparableVersions indicates that the system-key
@@ -102,7 +104,7 @@ func generateSystemKey() (*systemKey, error) {
 	sk := &systemKey{
 		Version: systemKeyVersion,
 	}
-	snapdPath, err := cmd.InternalToolPath("snapd")
+	snapdPath, err := snapdtool.InternalToolPath("snapd")
 	if err != nil {
 		return nil, err
 	}
@@ -157,6 +159,17 @@ func generateSystemKey() (*systemKey, error) {
 	return sk, nil
 }
 
+// UnmarshalJSONSystemKey unmarshalls the data from the reader as JSON into a
+// system key usable with SystemKeysMatch.
+func UnmarshalJSONSystemKey(r io.Reader) (interface{}, error) {
+	sk := &systemKey{}
+	err := json.NewDecoder(r).Decode(sk)
+	if err != nil {
+		return nil, err
+	}
+	return sk, nil
+}
+
 // WriteSystemKey will write the current system-key to disk
 func WriteSystemKey() error {
 	sk, err := generateSystemKey()
@@ -164,10 +177,15 @@ func WriteSystemKey() error {
 		return err
 	}
 
-	// We only want to calculate this when the mtime of the parser changes.
-	// Since we calculate the mtime() as part of generateSystemKey, we can
-	// simply unconditionally write this out here.
-	sk.AppArmorParserFeatures, _ = apparmor.ParserFeatures()
+	// only fix AppArmorParserFeatures if we didn't already mock a system-key
+	// if we mocked a system-key we are running a test and don't want to use
+	// the real host system's parser features
+	if mockedSystemKey == nil {
+		// We only want to calculate this when the mtime of the parser changes.
+		// Since we calculate the mtime() as part of generateSystemKey, we can
+		// simply unconditionally write this out here.
+		sk.AppArmorParserFeatures, _ = apparmor.ParserFeatures()
+	}
 
 	sks, err := json.Marshal(sk)
 	if err != nil {
@@ -213,17 +231,11 @@ func SystemKeyMismatch() (bool, error) {
 		return false, err
 	}
 
-	raw, err := ioutil.ReadFile(dirs.SnapSystemKeyFile)
-	if err != nil && os.IsNotExist(err) {
-		return false, ErrSystemKeyMissing
-	}
+	diskSystemKey, err := readSystemKey()
 	if err != nil {
 		return false, err
 	}
-	var diskSystemKey systemKey
-	if err := json.Unmarshal(raw, &diskSystemKey); err != nil {
-		return false, err
-	}
+
 	// deal with the race that "snap run" may start, then snapd
 	// is upgraded and generates a new system-key with different
 	// inputs than the "snap run" in memory. In this case we
@@ -251,8 +263,51 @@ func SystemKeyMismatch() (bool, error) {
 	diskSystemKey.AppArmorParserFeatures = nil
 	mySystemKey.AppArmorParserFeatures = nil
 
+	ok, err := SystemKeysMatch(mySystemKey, diskSystemKey)
+	return !ok, err
+}
+
+func readSystemKey() (*systemKey, error) {
+	raw, err := ioutil.ReadFile(dirs.SnapSystemKeyFile)
+	if err != nil && os.IsNotExist(err) {
+		return nil, ErrSystemKeyMissing
+	}
+	if err != nil {
+		return nil, err
+	}
+	var diskSystemKey systemKey
+	if err := json.Unmarshal(raw, &diskSystemKey); err != nil {
+		return nil, err
+	}
+	return &diskSystemKey, nil
+}
+
+// RecordedSystemKey returns the system key read from the disk as opaque interface{}.
+func RecordedSystemKey() (interface{}, error) {
+	diskSystemKey, err := readSystemKey()
+	if err != nil {
+		return nil, err
+	}
+	return diskSystemKey, nil
+}
+
+// CurrentSystemKey calculates and returns the current system key as opaque interface{}.
+func CurrentSystemKey() (interface{}, error) {
+	currentSystemKey, err := generateSystemKey()
+	return currentSystemKey, err
+}
+
+// SystemKeysMatch returns whether the given system keys match.
+func SystemKeysMatch(systemKey1, systemKey2 interface{}) (bool, error) {
+	// sanity check
+	_, ok1 := systemKey1.(*systemKey)
+	_, ok2 := systemKey2.(*systemKey)
+	if !(ok1 && ok2) {
+		return false, fmt.Errorf("SystemKeysMatch: arguments are not system keys")
+	}
+
 	// TODO: write custom struct compare
-	return !reflect.DeepEqual(mySystemKey, &diskSystemKey), nil
+	return reflect.DeepEqual(systemKey1, systemKey2), nil
 }
 
 func MockSystemKey(s string) func() {
