@@ -41,11 +41,3 @@ func MockUdevPropertiesForDevice(new func(string) (map[string]string, error)) (r
 		udevadmProperties = old
 	}
 }
-
-func MockDevBlockDir(new string) (restore func()) {
-	old := devBlockDir
-	devBlockDir = new
-	return func() {
-		devBlockDir = old
-	}
-}
