@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2016-2019 Canonical Ltd
+ * Copyright (C) 2016-2020 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -21,10 +21,14 @@ package assertstate_test
 
 import (
 	"bytes"
+	"context"
 	"crypto"
+	"errors"
 	"fmt"
 	"io/ioutil"
 	"path/filepath"
+	"sort"
+	"strings"
 	"testing"
 	"time"
 
@@ -36,6 +40,7 @@ import (
 	"github.com/snapcore/snapd/asserts/assertstest"
 	"github.com/snapcore/snapd/asserts/sysdb"
 	"github.com/snapcore/snapd/dirs"
+	"github.com/snapcore/snapd/httputil"
 	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/overlord"
 	"github.com/snapcore/snapd/overlord/assertstate"
@@ -45,6 +50,7 @@ import (
 	"github.com/snapcore/snapd/overlord/state"
 	"github.com/snapcore/snapd/snap"
 	"github.com/snapcore/snapd/snap/snaptest"
+	"github.com/snapcore/snapd/store"
 	"github.com/snapcore/snapd/store/storetest"
 	"github.com/snapcore/snapd/testutil"
 )
@@ -74,6 +80,11 @@ type fakeStore struct {
 	state                  *state.State
 	db                     asserts.RODatabase
 	maxDeclSupportedFormat int
+
+	requestedTypes [][]string
+
+	snapActionErr         error
+	downloadAssertionsErr error
 }
 
 func (sto *fakeStore) pokeStateLock() {
@@ -93,6 +104,97 @@ func (sto *fakeStore) Assertion(assertType *asserts.AssertionType, key []string,
 	return ref.Resolve(sto.db.Find)
 }
 
+func (sto *fakeStore) SnapAction(_ context.Context, currentSnaps []*store.CurrentSnap, actions []*store.SnapAction, assertQuery store.AssertionQuery, user *auth.UserState, opts *store.RefreshOptions) ([]store.SnapActionResult, []store.AssertionResult, error) {
+	sto.pokeStateLock()
+
+	if len(currentSnaps) != 0 || len(actions) != 0 {
+		panic("only assertion query supported")
+	}
+
+	toResolve, err := assertQuery.ToResolve()
+	if err != nil {
+		return nil, nil, err
+	}
+
+	if sto.snapActionErr != nil {
+		return nil, nil, sto.snapActionErr
+	}
+
+	restore := asserts.MockMaxSupportedFormat(asserts.SnapDeclarationType, sto.maxDeclSupportedFormat)
+	defer restore()
+
+	reqTypes := make(map[string]bool)
+	ares := make([]store.AssertionResult, 0, len(toResolve))
+	for g, ats := range toResolve {
+		urls := make([]string, 0, len(ats))
+		for _, at := range ats {
+			reqTypes[at.Ref.Type.Name] = true
+			a, err := at.Ref.Resolve(sto.db.Find)
+			if err != nil {
+				assertQuery.AddError(err, &at.Ref)
+				continue
+			}
+			if a.Revision() > at.Revision {
+				urls = append(urls, fmt.Sprintf("/assertions/%s", at.Unique()))
+			}
+		}
+		ares = append(ares, store.AssertionResult{
+			Grouping:   asserts.Grouping(g),
+			StreamURLs: urls,
+		})
+	}
+	// behave like the actual SnapAction if there are no results
+	if len(ares) == 0 {
+		return nil, ares, &store.SnapActionError{
+			NoResults: true,
+		}
+	}
+
+	typeNames := make([]string, 0, len(reqTypes))
+	for k := range reqTypes {
+		typeNames = append(typeNames, k)
+	}
+	sort.Strings(typeNames)
+	sto.requestedTypes = append(sto.requestedTypes, typeNames)
+
+	return nil, ares, nil
+}
+
+func (sto *fakeStore) DownloadAssertions(urls []string, b *asserts.Batch, user *auth.UserState) error {
+	sto.pokeStateLock()
+
+	if sto.downloadAssertionsErr != nil {
+		return sto.downloadAssertionsErr
+	}
+
+	resolve := func(ref *asserts.Ref) (asserts.Assertion, error) {
+		restore := asserts.MockMaxSupportedFormat(asserts.SnapDeclarationType, sto.maxDeclSupportedFormat)
+		defer restore()
+		return ref.Resolve(sto.db.Find)
+	}
+
+	for _, u := range urls {
+		comps := strings.Split(u, "/")
+
+		if len(comps) < 4 {
+			return fmt.Errorf("cannot use URL: %s", u)
+		}
+
+		assertType := asserts.Type(comps[2])
+		key := comps[3:]
+		ref := &asserts.Ref{Type: assertType, PrimaryKey: key}
+		a, err := resolve(ref)
+		if err != nil {
+			return err
+		}
+		if err := b.Add(a); err != nil {
+			return err
+		}
+	}
+
+	return nil
+}
+
 var (
 	dev1PrivKey, _ = assertstest.GenerateKey(752)
 )
@@ -788,6 +890,16 @@ func (s *assertMgrSuite) TestRefreshSnapDeclarationsTooEarly(c *C) {
 	c.Check(err, FitsTypeOf, &snapstate.ChangeConflictError{})
 }
 
+func (s *assertMgrSuite) TestRefreshSnapDeclarationsNop(c *C) {
+	s.state.Lock()
+	defer s.state.Unlock()
+
+	s.setModel(sysdb.GenericClassicModel())
+
+	err := assertstate.RefreshSnapDeclarations(s.state, 0)
+	c.Assert(err, IsNil)
+}
+
 func (s *assertMgrSuite) TestRefreshSnapDeclarationsNoStore(c *C) {
 	s.state.Lock()
 	defer s.state.Unlock()
@@ -861,7 +973,7 @@ func (s *assertMgrSuite) TestRefreshSnapDeclarationsNoStore(c *C) {
 	c.Check(a.(*asserts.Account).DisplayName(), Equals, "Dev 1 edited display-name")
 
 	// change snap decl to something that has a too new format
-
+	s.fakeStore.(*fakeStore).maxDeclSupportedFormat = 999
 	(func() {
 		restore := asserts.MockMaxSupportedFormat(asserts.SnapDeclarationType, 999)
 		defer restore()
@@ -895,6 +1007,68 @@ func (s *assertMgrSuite) TestRefreshSnapDeclarationsNoStore(c *C) {
 	c.Check(a.(*asserts.SnapDeclaration).Revision(), Equals, 1)
 }
 
+func (s *assertMgrSuite) TestRefreshSnapDeclarationsChangingKey(c *C) {
+	s.state.Lock()
+	defer s.state.Unlock()
+
+	s.setModel(sysdb.GenericClassicModel())
+
+	snapDeclFoo := s.snapDecl(c, "foo", nil)
+
+	s.stateFromDecl(c, snapDeclFoo, "", snap.R(7))
+
+	// previous state
+	err := assertstate.Add(s.state, s.storeSigning.StoreAccountKey(""))
+	c.Assert(err, IsNil)
+	err = assertstate.Add(s.state, s.dev1Acct)
+	c.Assert(err, IsNil)
+	err = assertstate.Add(s.state, snapDeclFoo)
+	c.Assert(err, IsNil)
+
+	storePrivKey2, _ := assertstest.GenerateKey(752)
+	err = s.storeSigning.ImportKey(storePrivKey2)
+	c.Assert(err, IsNil)
+	storeKey2 := assertstest.NewAccountKey(s.storeSigning.RootSigning, s.storeSigning.TrustedAccount, map[string]interface{}{
+		"name": "store2",
+	}, storePrivKey2.PublicKey(), "")
+	err = s.storeSigning.Add(storeKey2)
+	c.Assert(err, IsNil)
+
+	// one changed assertion signed with different key
+	headers := map[string]interface{}{
+		"series":       "16",
+		"snap-id":      "foo-id",
+		"snap-name":    "foo",
+		"publisher-id": s.dev1Acct.AccountID(),
+		"timestamp":    time.Now().Format(time.RFC3339),
+		"revision":     "1",
+	}
+	storeKey2ID := storePrivKey2.PublicKey().ID()
+	snapDeclFoo1, err := s.storeSigning.Sign(asserts.SnapDeclarationType, headers, nil, storeKey2ID)
+	c.Assert(err, IsNil)
+	c.Check(snapDeclFoo1.SignKeyID(), Not(Equals), snapDeclFoo.SignKeyID())
+	err = s.storeSigning.Add(snapDeclFoo1)
+	c.Assert(err, IsNil)
+
+	_, err = storeKey2.Ref().Resolve(assertstate.DB(s.state).Find)
+	c.Check(asserts.IsNotFound(err), Equals, true)
+
+	err = assertstate.RefreshSnapDeclarations(s.state, 0)
+	c.Assert(err, IsNil)
+
+	a, err := assertstate.DB(s.state).Find(asserts.SnapDeclarationType, map[string]string{
+		"series":  "16",
+		"snap-id": "foo-id",
+	})
+	c.Assert(err, IsNil)
+	c.Check(a.Revision(), Equals, 1)
+	c.Check(a.SignKeyID(), Equals, storeKey2ID)
+
+	// key was fetched as well
+	_, err = storeKey2.Ref().Resolve(assertstate.DB(s.state).Find)
+	c.Check(err, IsNil)
+}
+
 func (s *assertMgrSuite) TestRefreshSnapDeclarationsWithStore(c *C) {
 	s.state.Lock()
 	defer s.state.Unlock()
@@ -994,6 +1168,326 @@ func (s *assertMgrSuite) TestRefreshSnapDeclarationsWithStore(c *C) {
 	c.Check(a.(*asserts.Store).Location(), Equals, "the-cloud")
 }
 
+func (s *assertMgrSuite) TestRefreshSnapDeclarationsDownloadError(c *C) {
+	s.state.Lock()
+	defer s.state.Unlock()
+
+	s.setModel(sysdb.GenericClassicModel())
+
+	snapDeclFoo := s.snapDecl(c, "foo", nil)
+
+	s.stateFromDecl(c, snapDeclFoo, "", snap.R(7))
+
+	// previous state
+	err := assertstate.Add(s.state, s.storeSigning.StoreAccountKey(""))
+	c.Assert(err, IsNil)
+	err = assertstate.Add(s.state, s.dev1Acct)
+	c.Assert(err, IsNil)
+	err = assertstate.Add(s.state, snapDeclFoo)
+	c.Assert(err, IsNil)
+
+	// one changed assertion
+	headers := map[string]interface{}{
+		"series":       "16",
+		"snap-id":      "foo-id",
+		"snap-name":    "fo-o",
+		"publisher-id": s.dev1Acct.AccountID(),
+		"timestamp":    time.Now().Format(time.RFC3339),
+		"revision":     "1",
+	}
+	snapDeclFoo1, err := s.storeSigning.Sign(asserts.SnapDeclarationType, headers, nil, "")
+	c.Assert(err, IsNil)
+	err = s.storeSigning.Add(snapDeclFoo1)
+	c.Assert(err, IsNil)
+
+	s.fakeStore.(*fakeStore).downloadAssertionsErr = errors.New("download error")
+
+	err = assertstate.RefreshSnapDeclarations(s.state, 0)
+	c.Assert(err, ErrorMatches, `cannot refresh snap-declarations for snaps:
+ - foo: download error`)
+}
+
+func (s *assertMgrSuite) TestRefreshSnapDeclarationsPersistentNetworkError(c *C) {
+	s.state.Lock()
+	defer s.state.Unlock()
+
+	s.setModel(sysdb.GenericClassicModel())
+
+	snapDeclFoo := s.snapDecl(c, "foo", nil)
+
+	s.stateFromDecl(c, snapDeclFoo, "", snap.R(7))
+
+	// previous state
+	err := assertstate.Add(s.state, s.storeSigning.StoreAccountKey(""))
+	c.Assert(err, IsNil)
+	err = assertstate.Add(s.state, s.dev1Acct)
+	c.Assert(err, IsNil)
+	err = assertstate.Add(s.state, snapDeclFoo)
+	c.Assert(err, IsNil)
+
+	// one changed assertion
+	headers := map[string]interface{}{
+		"series":       "16",
+		"snap-id":      "foo-id",
+		"snap-name":    "fo-o",
+		"publisher-id": s.dev1Acct.AccountID(),
+		"timestamp":    time.Now().Format(time.RFC3339),
+		"revision":     "1",
+	}
+	snapDeclFoo1, err := s.storeSigning.Sign(asserts.SnapDeclarationType, headers, nil, "")
+	c.Assert(err, IsNil)
+	err = s.storeSigning.Add(snapDeclFoo1)
+	c.Assert(err, IsNil)
+
+	pne := new(httputil.PersistentNetworkError)
+	s.fakeStore.(*fakeStore).snapActionErr = pne
+
+	err = assertstate.RefreshSnapDeclarations(s.state, 0)
+	c.Assert(err, Equals, pne)
+}
+
+func (s *assertMgrSuite) TestRefreshSnapDeclarationsNoStoreFallback(c *C) {
+	// test that if we get a 4xx or 500 error from the store trying bulk
+	// assertion refresh we fall back to the old logic
+	s.fakeStore.(*fakeStore).snapActionErr = &store.UnexpectedHTTPStatusError{StatusCode: 400}
+
+	logbuf, restore := logger.MockLogger()
+	defer restore()
+
+	s.TestRefreshSnapDeclarationsNoStore(c)
+
+	c.Check(logbuf.String(), Matches, "(?m).*bulk refresh of snap-declarations failed, falling back to one-by-one assertion fetching:.*HTTP status code 400.*")
+}
+
+func (s *assertMgrSuite) TestRefreshSnapDeclarationsNoStoreFallbackUnexpectedSnapActionError(c *C) {
+	// test that if we get an unexpected SnapAction error from the
+	// store trying bulk assertion refresh we fall back to the old
+	// logic
+	s.fakeStore.(*fakeStore).snapActionErr = &store.SnapActionError{
+		NoResults: true,
+		Other:     []error{errors.New("unexpected error")},
+	}
+
+	logbuf, restore := logger.MockLogger()
+	defer restore()
+
+	s.TestRefreshSnapDeclarationsNoStore(c)
+
+	c.Check(logbuf.String(), Matches, "(?m).*bulk refresh of snap-declarations failed, falling back to one-by-one assertion fetching:.*unexpected error.*")
+}
+
+func (s *assertMgrSuite) TestRefreshSnapDeclarationsWithStoreFallback(c *C) {
+	// test that if we get a 4xx or 500 error from the store trying bulk
+	// assertion refresh we fall back to the old logic
+	s.fakeStore.(*fakeStore).snapActionErr = &store.UnexpectedHTTPStatusError{StatusCode: 500}
+
+	logbuf, restore := logger.MockLogger()
+	defer restore()
+
+	s.TestRefreshSnapDeclarationsWithStore(c)
+
+	c.Check(logbuf.String(), Matches, "(?m).*bulk refresh of snap-declarations failed, falling back to one-by-one assertion fetching:.*HTTP status code 500.*")
+}
+
+// the following tests cover what happens when refreshing snap-declarations
+// need to support overflowing the chosen asserts.Pool maximum groups
+
+func (s *assertMgrSuite) testRefreshSnapDeclarationsMany(c *C, n int) error {
+	// reduce maxGroups to test and stress the logic that deals
+	// with overflowing it
+	s.AddCleanup(assertstate.MockMaxGroups(16))
+
+	// previous state
+	err := assertstate.Add(s.state, s.storeSigning.StoreAccountKey(""))
+	c.Assert(err, IsNil)
+	err = assertstate.Add(s.state, s.dev1Acct)
+	c.Assert(err, IsNil)
+
+	for i := 1; i <= n; i++ {
+		name := fmt.Sprintf("foo%d", i)
+		snapDeclFooX := s.snapDecl(c, name, nil)
+
+		s.stateFromDecl(c, snapDeclFooX, "", snap.R(7+i))
+
+		// previous state
+		err = assertstate.Add(s.state, snapDeclFooX)
+		c.Assert(err, IsNil)
+
+		// make an update on top
+		headers := map[string]interface{}{
+			"series":       "16",
+			"snap-id":      name + "-id",
+			"snap-name":    fmt.Sprintf("fo-o-%d", i),
+			"publisher-id": s.dev1Acct.AccountID(),
+			"timestamp":    time.Now().Format(time.RFC3339),
+			"revision":     "1",
+		}
+		snapDeclFooX1, err := s.storeSigning.Sign(asserts.SnapDeclarationType, headers, nil, "")
+		c.Assert(err, IsNil)
+		err = s.storeSigning.Add(snapDeclFooX1)
+		c.Assert(err, IsNil)
+	}
+
+	err = assertstate.RefreshSnapDeclarations(s.state, 0)
+	if err != nil {
+		// fot the caller to check
+		return err
+	}
+
+	// check we got the updates
+	for i := 1; i <= n; i++ {
+		name := fmt.Sprintf("foo%d", i)
+		a, err := assertstate.DB(s.state).Find(asserts.SnapDeclarationType, map[string]string{
+			"series":  "16",
+			"snap-id": name + "-id",
+		})
+		c.Assert(err, IsNil)
+		c.Check(a.(*asserts.SnapDeclaration).SnapName(), Equals, fmt.Sprintf("fo-o-%d", i))
+	}
+
+	return nil
+}
+
+func (s *assertMgrSuite) TestRefreshSnapDeclarationsMany14NoStore(c *C) {
+	s.state.Lock()
+	defer s.state.Unlock()
+	s.setModel(sysdb.GenericClassicModel())
+
+	err := s.testRefreshSnapDeclarationsMany(c, 14)
+	c.Assert(err, IsNil)
+
+	c.Check(s.fakeStore.(*fakeStore).requestedTypes, DeepEquals, [][]string{
+		{"account", "account-key", "snap-declaration"},
+	})
+}
+
+func (s *assertMgrSuite) TestRefreshSnapDeclarationsMany16NoStore(c *C) {
+	s.state.Lock()
+	defer s.state.Unlock()
+	s.setModel(sysdb.GenericClassicModel())
+
+	err := s.testRefreshSnapDeclarationsMany(c, 16)
+	c.Assert(err, IsNil)
+
+	c.Check(s.fakeStore.(*fakeStore).requestedTypes, DeepEquals, [][]string{
+		{"account", "account-key", "snap-declaration"},
+	})
+}
+
+func (s *assertMgrSuite) TestRefreshSnapDeclarationsMany16WithStore(c *C) {
+	s.state.Lock()
+	defer s.state.Unlock()
+	// have a model and the store assertion available
+	storeAs := s.setupModelAndStore(c)
+	err := s.storeSigning.Add(storeAs)
+	c.Assert(err, IsNil)
+
+	err = s.testRefreshSnapDeclarationsMany(c, 16)
+	c.Assert(err, IsNil)
+
+	c.Check(s.fakeStore.(*fakeStore).requestedTypes, DeepEquals, [][]string{
+		// first 16 groups request
+		{"account", "account-key", "snap-declaration"},
+		// final separate request covering store only
+		{"store"},
+	})
+
+	// store assertion was also fetched
+	_, err = assertstate.DB(s.state).Find(asserts.StoreType, map[string]string{
+		"store": "my-brand-store",
+	})
+	c.Assert(err, IsNil)
+}
+
+func (s *assertMgrSuite) TestRefreshSnapDeclarationsMany17NoStore(c *C) {
+	s.state.Lock()
+	defer s.state.Unlock()
+	s.setModel(sysdb.GenericClassicModel())
+
+	err := s.testRefreshSnapDeclarationsMany(c, 17)
+	c.Assert(err, IsNil)
+
+	c.Check(s.fakeStore.(*fakeStore).requestedTypes, DeepEquals, [][]string{
+		// first 16 groups request
+		{"account", "account-key", "snap-declaration"},
+		// final separate request for the rest
+		{"snap-declaration"},
+	})
+}
+
+func (s *assertMgrSuite) TestRefreshSnapDeclarationsMany17NoStoreMergeErrors(c *C) {
+	s.state.Lock()
+	defer s.state.Unlock()
+	s.setModel(sysdb.GenericClassicModel())
+
+	s.fakeStore.(*fakeStore).downloadAssertionsErr = errors.New("download error")
+
+	err := s.testRefreshSnapDeclarationsMany(c, 17)
+	c.Check(err, ErrorMatches, `(?s)cannot refresh snap-declarations for snaps:
+ - foo1: download error.* - foo9: download error`)
+	// all foo* snaps accounted for
+	c.Check(strings.Count(err.Error(), "foo"), Equals, 17)
+
+	c.Check(s.fakeStore.(*fakeStore).requestedTypes, DeepEquals, [][]string{
+		// first 16 groups request
+		{"account", "account-key", "snap-declaration"},
+		// final separate request for the rest
+		{"snap-declaration"},
+	})
+}
+
+func (s *assertMgrSuite) TestRefreshSnapDeclarationsMany31WithStore(c *C) {
+	s.state.Lock()
+	defer s.state.Unlock()
+	// have a model and the store assertion available
+	storeAs := s.setupModelAndStore(c)
+	err := s.storeSigning.Add(storeAs)
+	c.Assert(err, IsNil)
+
+	err = s.testRefreshSnapDeclarationsMany(c, 31)
+	c.Assert(err, IsNil)
+
+	c.Check(s.fakeStore.(*fakeStore).requestedTypes, DeepEquals, [][]string{
+		// first 16 groups request
+		{"account", "account-key", "snap-declaration"},
+		// final separate request for the rest and store
+		{"snap-declaration", "store"},
+	})
+
+	// store assertion was also fetched
+	_, err = assertstate.DB(s.state).Find(asserts.StoreType, map[string]string{
+		"store": "my-brand-store",
+	})
+	c.Assert(err, IsNil)
+}
+
+func (s *assertMgrSuite) TestRefreshSnapDeclarationsMany32WithStore(c *C) {
+	s.state.Lock()
+	defer s.state.Unlock()
+	// have a model and the store assertion available
+	storeAs := s.setupModelAndStore(c)
+	err := s.storeSigning.Add(storeAs)
+	c.Assert(err, IsNil)
+
+	err = s.testRefreshSnapDeclarationsMany(c, 32)
+	c.Assert(err, IsNil)
+
+	c.Check(s.fakeStore.(*fakeStore).requestedTypes, DeepEquals, [][]string{
+		// first 16 groups request
+		{"account", "account-key", "snap-declaration"},
+		// 2nd round request
+		{"snap-declaration"},
+		// final separate request covering store
+		{"store"},
+	})
+
+	// store assertion was also fetched
+	_, err = assertstate.DB(s.state).Find(asserts.StoreType, map[string]string{
+		"store": "my-brand-store",
+	})
+	c.Assert(err, IsNil)
+}
+
 func (s *assertMgrSuite) TestValidateRefreshesNothing(c *C) {
 	s.state.Lock()
 	defer s.state.Unlock()
