@@ -26,7 +26,6 @@ import (
 	"regexp"
 	"strings"
 
-	"github.com/snapcore/snapd/features"
 	"github.com/snapcore/snapd/jsonutil"
 	"github.com/snapcore/snapd/overlord/state"
 	"github.com/snapcore/snapd/snap"
@@ -274,25 +273,15 @@ func DeleteSnapConfig(st *state.State, snapName string) error {
 // Conf is an interface describing both state and transaction.
 type Conf interface {
 	Get(snapName, key string, result interface{}) error
+	GetMaybe(snapName, key string, result interface{}) error
+	GetPristine(snapName, key string, result interface{}) error
 	Set(snapName, key string, value interface{}) error
 	Changes() []string
 	State() *state.State
 }
 
-// GetFeatureFlag returns the value of a given feature flag.
-func GetFeatureFlag(tr Conf, feature features.SnapdFeature) (bool, error) {
-	var isEnabled interface{}
-	snapName, confName := feature.ConfigOption()
-	if err := tr.Get(snapName, confName, &isEnabled); err != nil && !IsNoOption(err) {
-		return false, err
-	}
-	switch isEnabled {
-	case true, "true":
-		return true, nil
-	case false, "false":
-		return false, nil
-	case nil, "":
-		return feature.IsEnabledWhenUnset(), nil
-	}
-	return false, fmt.Errorf("%s can only be set to 'true' or 'false', got %q", feature, isEnabled)
+// ConfGetter is an interface for reading of config values.
+type ConfGetter interface {
+	Get(snapName, key string, result interface{}) error
+	GetMaybe(snapName, key string, result interface{}) error
 }
