@@ -37,15 +37,6 @@ type cloudSuite struct {
 
 var _ = Suite(&cloudSuite{})
 
-func (s *cloudSuite) SetUpTest(c *C) {
-	s.configcoreSuite.SetUpTest(c)
-	dirs.SetRootDir(c.MkDir())
-}
-
-func (s *cloudSuite) TearDownTest(c *C) {
-	dirs.SetRootDir("/")
-}
-
 func (s *cloudSuite) TestHandleCloud(c *C) {
 	tests := []struct {
 		instData         string
