@@ -631,7 +631,8 @@ func (s *preseedDoneSuite) TestDoMarkPreseededAfterFirstboot(c *C) {
 	c.Assert(st.Get("seed-restart-time", &seedRestartTime), IsNil)
 	c.Check(seedRestartTime.Equal(devicestate.StartTime()), Equals, true)
 
+	// this runs on first boot
 	c.Check(s.cmdSystemctl.Calls(), DeepEquals, [][]string{
-		{"systemctl", "--root", dirs.GlobalRootDir, "enable", "snap.test-snap.srv.service"},
+		{"systemctl", "enable", "snap.test-snap.srv.service"},
 	})
 }
