@@ -52,6 +52,14 @@ func MockPruneInterval(prunei, prunew, abortw time.Duration) (restore func()) {
 	}
 }
 
+func MockPruneTicker(f func(t *time.Ticker) <-chan time.Time) (restore func()) {
+	old := pruneTickerC
+	pruneTickerC = f
+	return func() {
+		pruneTickerC = old
+	}
+}
+
 // MockEnsureNext sets o.ensureNext for tests.
 func MockEnsureNext(o *Overlord, t time.Time) {
 	o.ensureNext = t
@@ -81,3 +89,11 @@ func MockConfigstateInit(new func(*state.State, *hookstate.HookManager) error) (
 		configstateInit = configstate.Init
 	}
 }
+
+func MockPreseedExitWithError(f func(err error)) (restore func()) {
+	old := preseedExitWithError
+	preseedExitWithError = f
+	return func() {
+		preseedExitWithError = old
+	}
+}
