@@ -25,7 +25,6 @@ import (
 	"time"
 
 	"github.com/snapcore/snapd/client"
-	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/overlord/cmdstate"
 	"github.com/snapcore/snapd/overlord/hookstate"
 	"github.com/snapcore/snapd/overlord/snapstate"
@@ -133,7 +132,7 @@ func NewStatusDecorator(rep interface {
 	Notify(string)
 }) *StatusDecorator {
 	return &StatusDecorator{
-		sysd: systemd.New(dirs.GlobalRootDir, systemd.SystemMode, rep),
+		sysd: systemd.New(systemd.SystemMode, rep),
 	}
 }
 
