@@ -20,9 +20,12 @@
 package backend_test
 
 import (
+	"archive/tar"
 	"archive/zip"
 	"bytes"
 	"context"
+	"crypto"
+	"encoding/json"
 	"errors"
 	"fmt"
 	"io"
@@ -30,6 +33,7 @@ import (
 	"os"
 	"os/exec"
 	"os/user"
+	"path"
 	"path/filepath"
 	"sort"
 	"strings"
@@ -41,6 +45,7 @@ import (
 	"github.com/snapcore/snapd/client"
 	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/logger"
+	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/osutil/sys"
 	"github.com/snapcore/snapd/overlord/snapshotstate/backend"
 	"github.com/snapcore/snapd/snap"
@@ -138,6 +143,59 @@ func hashkeys(snapshot *client.Snapshot) (keys []string) {
 	return keys
 }
 
+func (s *snapshotSuite) TestLastSnapshotID(c *check.C) {
+	// LastSnapshotSetID is happy without any snapshots
+	setID, err := backend.LastSnapshotSetID()
+	c.Assert(err, check.IsNil)
+	c.Check(setID, check.Equals, uint64(0))
+
+	// create snapshots dir and dummy snapshots
+	os.MkdirAll(dirs.SnapshotsDir, os.ModePerm)
+	for _, name := range []string{
+		"9_some-snap-1.zip", "1234_not-a-snapshot", "12_other-snap.zip", "3_foo.zip",
+	} {
+		c.Assert(ioutil.WriteFile(filepath.Join(dirs.SnapshotsDir, name), []byte{}, 0644), check.IsNil)
+	}
+	setID, err = backend.LastSnapshotSetID()
+	c.Assert(err, check.IsNil)
+	c.Check(setID, check.Equals, uint64(12))
+}
+
+func (s *snapshotSuite) TestLastSnapshotIDErrorOnDirNames(c *check.C) {
+	// we need snapshots dir, otherwise LastSnapshotSetID exits early.
+	c.Assert(os.MkdirAll(dirs.SnapshotsDir, os.ModePerm), check.IsNil)
+
+	defer backend.MockDirNames(func(*os.File, int) ([]string, error) {
+		return nil, fmt.Errorf("fail")
+	})()
+	setID, err := backend.LastSnapshotSetID()
+	c.Assert(err, check.ErrorMatches, "fail")
+	c.Check(setID, check.Equals, uint64(0))
+}
+
+func (s *snapshotSuite) TestIsSnapshotFilename(c *check.C) {
+	tests := []struct {
+		name  string
+		valid bool
+		setID uint64
+	}{
+		{"1_foo.zip", true, 1},
+		{"14_hello-world_6.4_29.zip", true, 14},
+		{"1_.zip", false, 0},
+		{"1_foo.zip.bak", false, 0},
+		{"foo_1_foo.zip", false, 0},
+		{"foo_bar_baz.zip", false, 0},
+		{"", false, 0},
+		{"1_", false, 0},
+	}
+
+	for _, t := range tests {
+		ok, setID := backend.IsSnapshotFilename(t.name)
+		c.Check(ok, check.Equals, t.valid, check.Commentf("fail: %s", t.name))
+		c.Check(setID, check.Equals, t.setID, check.Commentf("fail: %s", t.name))
+	}
+}
+
 func (s *snapshotSuite) TestIterBailsIfContextDone(c *check.C) {
 	ctx, cancel := context.WithCancel(context.Background())
 	cancel()
@@ -166,7 +224,7 @@ func (s *snapshotSuite) TestIterBailsIfContextDoneMidway(c *check.C) {
 		return []string{"hello"}, nil
 	})()
 	triedToOpenSnapshot := false
-	defer backend.MockOpen(func(string) (*backend.Reader, error) {
+	defer backend.MockOpen(func(string, uint64) (*backend.Reader, error) {
 		triedToOpenSnapshot = true
 		return nil, nil
 	})()
@@ -217,10 +275,10 @@ func (s *snapshotSuite) TestIterWarnsOnOpenErrorIfSnapshotNil(c *check.C) {
 		if readNames > 1 {
 			return nil, io.EOF
 		}
-		return []string{"hello"}, nil
+		return []string{"1_hello.zip"}, nil
 	})()
 	triedToOpenSnapshot := false
-	defer backend.MockOpen(func(string) (*backend.Reader, error) {
+	defer backend.MockOpen(func(string, uint64) (*backend.Reader, error) {
 		triedToOpenSnapshot = true
 		return nil, os.ErrInvalid
 	})()
@@ -237,7 +295,7 @@ func (s *snapshotSuite) TestIterWarnsOnOpenErrorIfSnapshotNil(c *check.C) {
 	c.Check(triedToOpenDir, check.Equals, true)
 	c.Check(readNames, check.Equals, 2)
 	c.Check(triedToOpenSnapshot, check.Equals, true)
-	c.Check(logbuf.String(), check.Matches, `(?m).* Cannot open snapshot "hello": invalid argument.`)
+	c.Check(logbuf.String(), check.Matches, `(?m).* Cannot open snapshot "1_hello.zip": invalid argument.`)
 	c.Check(calledF, check.Equals, false)
 }
 
@@ -255,13 +313,14 @@ func (s *snapshotSuite) TestIterCallsFuncIfSnapshotNotNil(c *check.C) {
 		if readNames > 1 {
 			return nil, io.EOF
 		}
-		return []string{"hello"}, nil
+		return []string{"1_hello.zip"}, nil
 	})()
 	triedToOpenSnapshot := false
-	defer backend.MockOpen(func(string) (*backend.Reader, error) {
+	defer backend.MockOpen(func(string, uint64) (*backend.Reader, error) {
 		triedToOpenSnapshot = true
 		// NOTE non-nil reader, and error, returned
 		r := backend.Reader{}
+		r.SetID = 1
 		r.Broken = "xyzzy"
 		return &r, os.ErrInvalid
 	})()
@@ -297,10 +356,10 @@ func (s *snapshotSuite) TestIterReportsCloseError(c *check.C) {
 		if readNames > 1 {
 			return nil, io.EOF
 		}
-		return []string{"hello"}, nil
+		return []string{"42_hello.zip"}, nil
 	})()
 	triedToOpenSnapshot := false
-	defer backend.MockOpen(func(string) (*backend.Reader, error) {
+	defer backend.MockOpen(func(string, uint64) (*backend.Reader, error) {
 		triedToOpenSnapshot = true
 		r := backend.Reader{}
 		r.SetID = 42
@@ -324,10 +383,98 @@ func (s *snapshotSuite) TestIterReportsCloseError(c *check.C) {
 	c.Check(calledF, check.Equals, true)
 }
 
+func readerForFilename(fname string, c *check.C) *backend.Reader {
+	var snapname string
+	var id uint64
+	fn := strings.TrimSuffix(filepath.Base(fname), ".zip")
+	_, err := fmt.Sscanf(fn, "%d_%s", &id, &snapname)
+	c.Assert(err, check.IsNil, check.Commentf(fn))
+	f, err := os.Open(os.DevNull)
+	c.Assert(err, check.IsNil, check.Commentf(fn))
+	return &backend.Reader{
+		File: f,
+		Snapshot: client.Snapshot{
+			SetID: id,
+			Snap:  snapname,
+		},
+	}
+}
+
+func (s *snapshotSuite) TestIterIgnoresSnapshotsWithInvalidNames(c *check.C) {
+	logbuf, restore := logger.MockLogger()
+	defer restore()
+
+	defer backend.MockOsOpen(func(string) (*os.File, error) {
+		return new(os.File), nil
+	})()
+	readNames := 0
+	defer backend.MockDirNames(func(*os.File, int) ([]string, error) {
+		readNames++
+		if readNames > 1 {
+			return nil, io.EOF
+		}
+		return []string{
+			"_foo.zip",
+			"43_bar.zip",
+			"foo_bar.zip",
+			"bar.",
+		}, nil
+	})()
+	defer backend.MockOpen(func(fname string, setID uint64) (*backend.Reader, error) {
+		return readerForFilename(fname, c), nil
+	})()
+
+	var calledF int
+	f := func(snapshot *backend.Reader) error {
+		calledF++
+		c.Check(snapshot.SetID, check.Equals, uint64(43))
+		return nil
+	}
+
+	err := backend.Iter(context.Background(), f)
+	c.Check(err, check.IsNil)
+	c.Check(logbuf.String(), check.Equals, "")
+	c.Check(calledF, check.Equals, 1)
+}
+
+func (s *snapshotSuite) TestIterSetIDoverride(c *check.C) {
+	if os.Geteuid() == 0 {
+		c.Skip("this test cannot run as root (runuser will fail)")
+	}
+	logger.SimpleSetup()
+
+	epoch := snap.E("42*")
+	info := &snap.Info{SideInfo: snap.SideInfo{RealName: "hello-snap", Revision: snap.R(42), SnapID: "hello-id"}, Version: "v1.33", Epoch: epoch}
+	cfg := map[string]interface{}{"some-setting": false}
+
+	shw, err := backend.Save(context.TODO(), 12, info, cfg, []string{"snapuser"}, &backend.Flags{})
+	c.Assert(err, check.IsNil)
+	c.Check(shw.SetID, check.Equals, uint64(12))
+
+	snapshotPath := filepath.Join(dirs.SnapshotsDir, "12_hello-snap_v1.33_42.zip")
+	c.Check(backend.Filename(shw), check.Equals, snapshotPath)
+	c.Check(hashkeys(shw), check.DeepEquals, []string{"archive.tgz", "user/snapuser.tgz"})
+
+	// rename the snapshot, verify that set id from the filename is used by the reader.
+	c.Assert(os.Rename(snapshotPath, filepath.Join(dirs.SnapshotsDir, "33_hello.zip")), check.IsNil)
+
+	var calledF int
+	f := func(snapshot *backend.Reader) error {
+		calledF++
+		c.Check(snapshot.SetID, check.Equals, uint64(uint(33)))
+		c.Check(snapshot.Snap, check.Equals, "hello-snap")
+		return nil
+	}
+
+	c.Assert(backend.Iter(context.Background(), f), check.IsNil)
+	c.Check(calledF, check.Equals, 1)
+}
+
 func (s *snapshotSuite) TestList(c *check.C) {
 	logbuf, restore := logger.MockLogger()
 	defer restore()
 	defer backend.MockOsOpen(func(string) (*os.File, error) { return new(os.File), nil })()
+
 	readNames := 0
 	defer backend.MockDirNames(func(*os.File, int) ([]string, error) {
 		readNames++
@@ -335,15 +482,16 @@ func (s *snapshotSuite) TestList(c *check.C) {
 			return nil, io.EOF
 		}
 		return []string{
-			fmt.Sprintf("%d_foo", readNames),
-			fmt.Sprintf("%d_bar", readNames),
-			fmt.Sprintf("%d_baz", readNames),
+			fmt.Sprintf("%d_foo.zip", readNames),
+			fmt.Sprintf("%d_bar.zip", readNames),
+			fmt.Sprintf("%d_baz.zip", readNames),
 		}, nil
 	})()
-	defer backend.MockOpen(func(fn string) (*backend.Reader, error) {
+	defer backend.MockOpen(func(fn string, setID uint64) (*backend.Reader, error) {
 		var id uint64
 		var snapname string
-		fn = filepath.Base(fn)
+		c.Assert(strings.HasSuffix(fn, ".zip"), check.Equals, true)
+		fn = strings.TrimSuffix(filepath.Base(fn), ".zip")
 		_, err := fmt.Sscanf(fn, "%d_%s", &id, &snapname)
 		c.Assert(err, check.IsNil, check.Commentf(fn))
 		f, err := os.Open(os.DevNull)
@@ -509,7 +657,7 @@ func (s *snapshotSuite) testHappyRoundtrip(c *check.C, marker string, auto bool)
 	c.Assert(shs, check.HasLen, 1)
 	c.Assert(shs[0].Snapshots, check.HasLen, 1)
 
-	shr, err := backend.Open(backend.Filename(shw))
+	shr, err := backend.Open(backend.Filename(shw), backend.ExtractFnameSetID)
 	c.Assert(err, check.IsNil)
 	defer shr.Close()
 
@@ -555,6 +703,30 @@ func (s *snapshotSuite) testHappyRoundtrip(c *check.C, marker string, auto bool)
 	}
 }
 
+func (s *snapshotSuite) TestOpenSetIDoverride(c *check.C) {
+	if os.Geteuid() == 0 {
+		c.Skip("this test cannot run as root (runuser will fail)")
+	}
+	logger.SimpleSetup()
+
+	epoch := snap.E("42*")
+	info := &snap.Info{SideInfo: snap.SideInfo{RealName: "hello-snap", Revision: snap.R(42), SnapID: "hello-id"}, Version: "v1.33", Epoch: epoch}
+	cfg := map[string]interface{}{"some-setting": false}
+
+	shw, err := backend.Save(context.TODO(), 12, info, cfg, []string{"snapuser"}, &backend.Flags{})
+	c.Assert(err, check.IsNil)
+	c.Check(shw.SetID, check.Equals, uint64(12))
+
+	c.Check(backend.Filename(shw), check.Equals, filepath.Join(dirs.SnapshotsDir, "12_hello-snap_v1.33_42.zip"))
+	c.Check(hashkeys(shw), check.DeepEquals, []string{"archive.tgz", "user/snapuser.tgz"})
+
+	shr, err := backend.Open(backend.Filename(shw), 99)
+	c.Assert(err, check.IsNil)
+	defer shr.Close()
+
+	c.Check(shr.SetID, check.Equals, uint64(99))
+}
+
 func (s *snapshotSuite) TestRestoreRoundtripDifferentRevision(c *check.C) {
 	if os.Geteuid() == 0 {
 		c.Skip("this test cannot run as root (runuser will fail)")
@@ -569,7 +741,7 @@ func (s *snapshotSuite) TestRestoreRoundtripDifferentRevision(c *check.C) {
 	c.Assert(err, check.IsNil)
 	c.Check(shw.Revision, check.Equals, info.Revision)
 
-	shr, err := backend.Open(backend.Filename(shw))
+	shr, err := backend.Open(backend.Filename(shw), backend.ExtractFnameSetID)
 	c.Assert(err, check.IsNil)
 	defer shr.Close()
 
@@ -720,6 +892,152 @@ func (s *snapshotSuite) TestMaybeRunuserNoHappy(c *check.C) {
 	c.Check(strings.TrimSpace(logbuf.String()), check.Matches, ".* No user wrapper found.*")
 }
 
+func (s *snapshotSuite) TestImport(c *check.C) {
+	tempdir := c.MkDir()
+
+	// create snapshot export file
+	tarFile1 := path.Join(tempdir, "exported1.snapshot")
+	err := createTestExportFile(tarFile1, &createTestExportFlags{exportJSON: true})
+	c.Check(err, check.IsNil)
+
+	// create an exported snapshot with missing export.json
+	tarFile2 := path.Join(tempdir, "exported2.snapshot")
+	err = createTestExportFile(tarFile2, &createTestExportFlags{})
+	c.Check(err, check.IsNil)
+
+	// create invalid exported file
+	tarFile3 := path.Join(tempdir, "exported3.snapshot")
+	err = ioutil.WriteFile(tarFile3, []byte("invalid"), 0755)
+	c.Check(err, check.IsNil)
+
+	// create an exported snapshot with a directory
+	tarFile4 := path.Join(tempdir, "exported4.snapshot")
+	flags := &createTestExportFlags{
+		exportJSON: true,
+		withDir:    true,
+	}
+	err = createTestExportFile(tarFile4, flags)
+	c.Check(err, check.IsNil)
+
+	type tableT struct {
+		setID      uint64
+		filename   string
+		inProgress bool
+		error      string
+	}
+
+	table := []tableT{
+		{14, tarFile1, false, ""},
+		{14, tarFile2, false, "cannot import snapshot 14: no export.json file in uploaded data"},
+		{14, tarFile3, false, "cannot import snapshot 14: cannot read snapshot import: unexpected EOF"},
+		{14, tarFile4, false, "cannot import snapshot 14: unexpected directory in import file"},
+		{14, tarFile1, true, "cannot import snapshot 14: already in progress for this set id"},
+	}
+
+	for i, t := range table {
+		comm := check.Commentf("%d: %d %s", i, t.setID, t.filename)
+
+		// reset
+		err = os.RemoveAll(dirs.SnapshotsDir)
+		c.Assert(err, check.IsNil, comm)
+		err := os.MkdirAll(dirs.SnapshotsDir, 0700)
+		c.Assert(err, check.IsNil, comm)
+		importingFile := filepath.Join(dirs.SnapshotsDir, fmt.Sprintf("%d_importing", t.setID))
+		if t.inProgress {
+			err = ioutil.WriteFile(importingFile, nil, 0644)
+			c.Assert(err, check.IsNil, comm)
+		} else {
+			err = os.RemoveAll(importingFile)
+			c.Assert(err, check.IsNil, comm)
+		}
+
+		f, err := os.Open(t.filename)
+		c.Assert(err, check.IsNil, comm)
+		defer f.Close()
+
+		snapNames, err := backend.Import(context.Background(), t.setID, f)
+		if t.error != "" {
+			c.Check(err, check.ErrorMatches, t.error, comm)
+			continue
+		}
+		c.Check(err, check.IsNil, comm)
+		sort.Strings(snapNames)
+		c.Check(snapNames, check.DeepEquals, []string{"bar", "baz", "foo"})
+
+		dir, err := os.Open(dirs.SnapshotsDir)
+		c.Assert(err, check.IsNil, comm)
+		defer dir.Close()
+		names, err := dir.Readdirnames(100)
+		c.Assert(err, check.IsNil, comm)
+		c.Check(len(names), check.Equals, 3, comm)
+	}
+}
+
+func (s *snapshotSuite) TestImportCheckErorr(c *check.C) {
+	err := os.MkdirAll(dirs.SnapshotsDir, 0755)
+	c.Assert(err, check.IsNil)
+
+	// create snapshot export file
+	tarFile1 := path.Join(c.MkDir(), "exported1.snapshot")
+	flags := &createTestExportFlags{
+		exportJSON:      true,
+		corruptChecksum: true,
+	}
+	err = createTestExportFile(tarFile1, flags)
+	c.Assert(err, check.IsNil)
+
+	f, err := os.Open(tarFile1)
+	c.Assert(err, check.IsNil)
+	_, err = backend.Import(context.Background(), 14, f)
+	c.Assert(err, check.ErrorMatches, `cannot import snapshot 14: validation failed for .+/14_foo_1.0_199.zip": snapshot entry "archive.tgz" expected hash \(d5ef563…\) does not match actual \(6655519…\)`)
+}
+
+func (s *snapshotSuite) TestImportExportRoundtrip(c *check.C) {
+	err := os.MkdirAll(dirs.SnapshotsDir, 0755)
+	c.Assert(err, check.IsNil)
+
+	ctx := context.TODO()
+
+	epoch := snap.E("42*")
+	info := &snap.Info{SideInfo: snap.SideInfo{RealName: "hello-snap", Revision: snap.R(42), SnapID: "hello-id"}, Version: "v1.33", Epoch: epoch}
+	cfg := map[string]interface{}{"some-setting": false}
+	shID := uint64(12)
+
+	shw, err := backend.Save(ctx, shID, info, cfg, []string{"snapuser"}, &backend.Flags{})
+	c.Assert(err, check.IsNil)
+	c.Check(shw.SetID, check.Equals, shID)
+
+	c.Check(backend.Filename(shw), check.Equals, filepath.Join(dirs.SnapshotsDir, "12_hello-snap_v1.33_42.zip"))
+	c.Check(hashkeys(shw), check.DeepEquals, []string{"archive.tgz", "user/snapuser.tgz"})
+
+	export, err := backend.NewSnapshotExport(ctx, shw.SetID)
+	c.Assert(err, check.IsNil)
+	c.Assert(export.Init(), check.IsNil)
+
+	buf := bytes.NewBuffer(nil)
+	c.Assert(export.StreamTo(buf), check.IsNil)
+	c.Check(buf.Len(), check.Equals, int(export.Size()))
+
+	// now import it
+	c.Assert(os.Remove(filepath.Join(dirs.SnapshotsDir, "12_hello-snap_v1.33_42.zip")), check.IsNil)
+
+	names, err := backend.Import(ctx, 123, buf)
+	c.Assert(err, check.IsNil)
+	c.Check(names, check.DeepEquals, []string{"hello-snap"})
+
+	sets, err := backend.List(ctx, 0, nil)
+	c.Assert(err, check.IsNil)
+	c.Assert(sets, check.HasLen, 1)
+	c.Check(sets[0].ID, check.Equals, uint64(123))
+
+	rdr, err := backend.Open(filepath.Join(dirs.SnapshotsDir, "123_hello-snap_v1.33_42.zip"), backend.ExtractFnameSetID)
+	defer rdr.Close()
+	c.Check(err, check.IsNil)
+	c.Check(rdr.SetID, check.Equals, uint64(123))
+	c.Check(rdr.Snap, check.Equals, "hello-snap")
+	c.Check(rdr.IsValid(), check.Equals, true)
+}
+
 func (s *snapshotSuite) TestEstimateSnapshotSize(c *check.C) {
 	restore := backend.MockUsersForUsernames(func(usernames []string) ([]*user.User, error) {
 		return []*user.User{{HomeDir: filepath.Join(s.root, "home/user1")}}, nil
@@ -876,3 +1194,161 @@ func (s *snapshotSuite) TestExportUnhappy(c *check.C) {
 	c.Assert(err, check.ErrorMatches, "no snapshot data found for 5")
 	c.Assert(se, check.IsNil)
 }
+
+type createTestExportFlags struct {
+	exportJSON      bool
+	withDir         bool
+	corruptChecksum bool
+}
+
+func createTestExportFile(filename string, flags *createTestExportFlags) error {
+	tf, err := os.Create(filename)
+	if err != nil {
+		return err
+	}
+	defer tf.Close()
+	tw := tar.NewWriter(tf)
+	defer tw.Close()
+
+	for _, s := range []string{"foo", "bar", "baz"} {
+		fname := fmt.Sprintf("5_%s_1.0_199.zip", s)
+
+		buf := bytes.NewBuffer(nil)
+		zipW := zip.NewWriter(buf)
+		defer zipW.Close()
+
+		sha := map[string]string{}
+
+		// create dummy archive.tgz
+		archiveWriter, err := zipW.CreateHeader(&zip.FileHeader{Name: "archive.tgz"})
+		if err != nil {
+			return err
+		}
+		var sz osutil.Sizer
+		hasher := crypto.SHA3_384.New()
+		out := io.MultiWriter(archiveWriter, hasher, &sz)
+		if _, err := out.Write([]byte(s)); err != nil {
+			return err
+		}
+
+		if flags.corruptChecksum {
+			hasher.Write([]byte{0})
+		}
+		sha["archive.tgz"] = fmt.Sprintf("%x", hasher.Sum(nil))
+
+		snapshot := backend.MockSnapshot(5, s, snap.Revision{N: 199}, sz.Size(), sha)
+
+		// create meta.json
+		metaWriter, err := zipW.Create("meta.json")
+		if err != nil {
+			return err
+		}
+		hasher = crypto.SHA3_384.New()
+		enc := json.NewEncoder(io.MultiWriter(metaWriter, hasher))
+		if err := enc.Encode(snapshot); err != nil {
+			return err
+		}
+
+		// write meta.sha3_384
+		metaSha3Writer, err := zipW.Create("meta.sha3_384")
+		if err != nil {
+			return err
+		}
+		fmt.Fprintf(metaSha3Writer, "%x\n", hasher.Sum(nil))
+		zipW.Close()
+
+		hdr := &tar.Header{
+			Name: fname,
+			Mode: 0644,
+			Size: int64(buf.Len()),
+		}
+		if err := tw.WriteHeader(hdr); err != nil {
+			return err
+		}
+		if _, err := tw.Write(buf.Bytes()); err != nil {
+			return err
+		}
+	}
+
+	if flags.withDir {
+		hdr := &tar.Header{
+			Name:     dirs.SnapshotsDir,
+			Mode:     0700,
+			Size:     int64(0),
+			Typeflag: tar.TypeDir,
+		}
+		if err := tw.WriteHeader(hdr); err != nil {
+			return err
+		}
+		if _, err = tw.Write([]byte("")); err != nil {
+			return nil
+		}
+	}
+
+	if flags.exportJSON {
+		exp := fmt.Sprintf(`{"format":1, "date":"%s"}`, time.Now().Format(time.RFC3339))
+		hdr := &tar.Header{
+			Name: "export.json",
+			Mode: 0644,
+			Size: int64(len(exp)),
+		}
+		if err := tw.WriteHeader(hdr); err != nil {
+			return err
+		}
+		if _, err = tw.Write([]byte(exp)); err != nil {
+			return nil
+		}
+	}
+
+	return nil
+}
+
+func makeMockSnapshotZipContent(c *check.C) []byte {
+	buf := bytes.NewBuffer(nil)
+	zipW := zip.NewWriter(buf)
+
+	// create dummy archive.tgz
+	archiveWriter, err := zipW.CreateHeader(&zip.FileHeader{Name: "archive.tgz"})
+	c.Assert(err, check.IsNil)
+	_, err = archiveWriter.Write([]byte("mock archive.tgz content"))
+	c.Assert(err, check.IsNil)
+
+	// create dummy meta.json
+	archiveWriter, err = zipW.CreateHeader(&zip.FileHeader{Name: "meta.json"})
+	c.Assert(err, check.IsNil)
+	_, err = archiveWriter.Write([]byte("{}"))
+	c.Assert(err, check.IsNil)
+
+	zipW.Close()
+	return buf.Bytes()
+}
+
+func (s *snapshotSuite) TestIterWithMockedSnapshotFiles(c *check.C) {
+	err := os.MkdirAll(dirs.SnapshotsDir, 0755)
+	c.Assert(err, check.IsNil)
+
+	fn := "1_hello_1.0_x1.zip"
+	err = ioutil.WriteFile(filepath.Join(dirs.SnapshotsDir, fn), makeMockSnapshotZipContent(c), 0644)
+	c.Assert(err, check.IsNil)
+
+	callbackCalled := 0
+	f := func(snapshot *backend.Reader) error {
+		callbackCalled++
+		return nil
+	}
+
+	err = backend.Iter(context.Background(), f)
+	c.Check(err, check.IsNil)
+	c.Check(callbackCalled, check.Equals, 1)
+
+	// now pretend we are importing snapshot id 1
+	callbackCalled = 0
+	fn = "1_importing"
+	err = ioutil.WriteFile(filepath.Join(dirs.SnapshotsDir, fn), nil, 0644)
+	c.Assert(err, check.IsNil)
+
+	// and while importing Iter() does not call the callback
+	err = backend.Iter(context.Background(), f)
+	c.Check(err, check.IsNil)
+	c.Check(callbackCalled, check.Equals, 0)
+}
