@@ -42,6 +42,10 @@ import (
 	"github.com/snapcore/snapd/strutil"
 )
 
+// ExtractFnameSetID can be passed to Open() to have set ID inferred from
+// snapshot filename.
+const ExtractFnameSetID = 0
+
 // A Reader is a snapshot that's been opened for reading.
 type Reader struct {
 	*os.File
@@ -50,13 +54,17 @@ type Reader struct {
 
 // Open a Snapshot given its full filename.
 //
+// The returned reader will have its setID set to the value of the argument,
+// or inferred from the snapshot filename if ExtractFnameSetID constant is
+// passed.
+//
 // If the returned error is nil, the caller must close the reader (or
 // its file) when done with it.
 //
 // If the returned error is non-nil, the returned Reader will be nil,
 // *or* have a non-empty Broken; in the latter case its file will be
 // closed.
-func Open(fn string) (reader *Reader, e error) {
+func Open(fn string, setID uint64) (reader *Reader, e error) {
 	f, err := os.Open(fn)
 	if err != nil {
 		return nil, err
@@ -84,6 +92,18 @@ func Open(fn string) (reader *Reader, e error) {
 		return nil, err
 	}
 
+	if setID == ExtractFnameSetID {
+		// set id from the filename has the authority and overrides the one from
+		// meta file.
+		var ok bool
+		ok, setID = isSnapshotFilename(fn)
+		if !ok {
+			return nil, fmt.Errorf("not a snapshot filename: %q", fn)
+		}
+	}
+
+	reader.SetID = setID
+
 	// OK, from here on we have a Snapshot
 
 	if !reader.IsValid() {
