@@ -40,6 +40,8 @@ type mountunitSuite struct {
 	umount *testutil.MockCmd
 
 	systemctlRestorer func()
+
+	testutil.BaseTest
 }
 
 var _ = Suite(&mountunitSuite{})
@@ -47,6 +49,9 @@ var _ = Suite(&mountunitSuite{})
 func (s *mountunitSuite) SetUpTest(c *C) {
 	dirs.SetRootDir(c.MkDir())
 
+	// needed for system key generation
+	s.AddCleanup(osutil.MockMountInfo(""))
+
 	err := os.MkdirAll(filepath.Join(dirs.GlobalRootDir, "etc", "systemd", "system", "multi-user.target.wants"), 0755)
 	c.Assert(err, IsNil)
 
