@@ -27,6 +27,7 @@ import (
 	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/overlord/configstate/config"
 	"github.com/snapcore/snapd/overlord/snapstate"
+	"github.com/snapcore/snapd/overlord/snapstate/snapstatetest"
 	"github.com/snapcore/snapd/overlord/state"
 	"github.com/snapcore/snapd/snap"
 	"github.com/snapcore/snapd/store"
@@ -51,9 +52,11 @@ func (s *downloadSnapSuite) SetUpTest(c *C) {
 	defer s.state.Unlock()
 	snapstate.ReplaceStore(s.state, s.fakeStore)
 	s.state.Set("refresh-privacy-key", "privacy-key")
+
+	s.AddCleanup(snapstatetest.UseFallbackDeviceModel())
 }
 
-func (s *downloadSnapSuite) TestDoDownloadSnapCompatbility(c *C) {
+func (s *downloadSnapSuite) TestDoDownloadSnapCompatibility(c *C) {
 	s.state.Lock()
 	t := s.state.NewTask("download-snap", "test")
 	t.Set("snap-setup", &snapstate.SnapSetup{
@@ -128,13 +131,19 @@ func (s *downloadSnapSuite) TestDoDownloadSnapNormal(c *C) {
 			DownloadURL: "http://some-url.com/snap",
 		},
 	})
-	s.state.NewChange("dummy", "...").AddTask(t)
+	chg := s.state.NewChange("dummy", "...")
+	chg.AddTask(t)
 
 	s.state.Unlock()
 
 	s.se.Ensure()
 	s.se.Wait()
 
+	s.state.Lock()
+	defer s.state.Unlock()
+
+	c.Assert(chg.Err(), IsNil)
+
 	// only the download endpoint of the store was hit
 	c.Assert(s.fakeBackend.ops, DeepEquals, fakeOps{
 		{
@@ -143,9 +152,6 @@ func (s *downloadSnapSuite) TestDoDownloadSnapNormal(c *C) {
 		},
 	})
 
-	s.state.Lock()
-	defer s.state.Unlock()
-
 	var snapsup snapstate.SnapSetup
 	t.Get("snap-setup", &snapsup)
 	c.Check(snapsup.SideInfo, DeepEquals, si)
@@ -161,6 +167,50 @@ func (s *downloadSnapSuite) TestDoDownloadSnapNormal(c *C) {
 	})
 }
 
+func (s *downloadSnapSuite) TestDoDownloadSnapWithDeviceContext(c *C) {
+	s.state.Lock()
+
+	// unset the global store, it will need to come via the device context
+	// CtxStore
+	snapstate.ReplaceStore(s.state, nil)
+
+	r := snapstatetest.MockDeviceContext(&snapstatetest.TrivialDeviceContext{
+		CtxStore: s.fakeStore,
+	})
+	defer r()
+
+	si := &snap.SideInfo{
+		RealName: "foo",
+		SnapID:   "mySnapID",
+		Revision: snap.R(11),
+		Channel:  "my-channel",
+	}
+
+	// download, ensure the store does not query
+	t := s.state.NewTask("download-snap", "test")
+	t.Set("snap-setup", &snapstate.SnapSetup{
+		Channel:  "some-channel",
+		SideInfo: si,
+		DownloadInfo: &snap.DownloadInfo{
+			DownloadURL: "http://some-url.com/snap",
+		},
+	})
+	s.state.NewChange("dummy", "...").AddTask(t)
+
+	s.state.Unlock()
+
+	s.se.Ensure()
+	s.se.Wait()
+
+	// only the download endpoint of the store was hit
+	c.Assert(s.fakeBackend.ops, DeepEquals, fakeOps{
+		{
+			op:   "storesvc-download",
+			name: "foo",
+		},
+	})
+}
+
 func (s *downloadSnapSuite) TestDoUndoDownloadSnap(c *C) {
 	s.state.Lock()
 	si := &snap.SideInfo{
