@@ -28,10 +28,10 @@ func CheckKeySealingSupported() error {
 	return fmt.Errorf("build without secboot support")
 }
 
-func SealKey(key EncryptionKey, params *SealKeyParams) error {
+func SealKeys(keys []SealKeyRequest, params *SealKeysParams) error {
 	return fmt.Errorf("build without secboot support")
 }
 
-func ResealKey(params *ResealKeyParams) error {
+func ResealKeys(params *ResealKeysParams) error {
 	return fmt.Errorf("build without secboot support")
 }
