@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2019 Canonical Ltd
+ * Copyright (C) 2019-2020 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -224,8 +224,7 @@ func (s *seed20Suite) TestLoadAssertionsModelTempDBHappy(c *C) {
 	err = seed20.LoadAssertions(nil, nil)
 	c.Assert(err, IsNil)
 
-	model, err := seed20.Model()
-	c.Assert(err, IsNil)
+	model := seed20.Model()
 	c.Check(model.Model(), Equals, "my-model")
 	c.Check(model.Base(), Equals, "core20")
 
@@ -700,11 +699,15 @@ Hiding:
 }
 
 func (s *seed20Suite) TestLoadEssentialMetaCore20(c *C) {
+	r := seed.MockTrusted(s.StoreSigning.Trusted)
+	defer r()
+
 	s.makeSnap(c, "snapd", "")
 	s.makeSnap(c, "core20", "")
 	s.makeSnap(c, "pc-kernel=20", "")
 	s.makeSnap(c, "pc=20", "")
-	s.makeSnap(c, "required20", "developerid")
+	s.makeSnap(c, "core18", "")
+	s.makeSnap(c, "required18", "developerid")
 
 	sysLabel := "20191018"
 	s.MakeSeed(c, sysLabel, "my-brand", "my-model", map[string]interface{}{
@@ -725,8 +728,13 @@ func (s *seed20Suite) TestLoadEssentialMetaCore20(c *C) {
 				"default-channel": "20",
 			},
 			map[string]interface{}{
-				"name": "required20",
-				"id":   s.AssertedSnapID("required20"),
+				"name": "core18",
+				"id":   s.AssertedSnapID("core18"),
+				"type": "base",
+			},
+			map[string]interface{}{
+				"name": "required18",
+				"id":   s.AssertedSnapID("required18"),
 			}},
 	}, nil)
 
@@ -761,11 +769,16 @@ func (s *seed20Suite) TestLoadEssentialMetaCore20(c *C) {
 		Required:      true,
 		Channel:       "20",
 	}
-	required20Snap := &seed.Snap{
-		Path: s.expectedPath("required20"),
+	core18Snap := &seed.Snap{
+		// no EssentialType, so it's always hidden, shouldn't matter
+		// because we should not look at it
+		Path: s.expectedPath("core18"),
+	}
+	required18Snap := &seed.Snap{
+		Path: s.expectedPath("required18"),
 	}
 
-	all := []*seed.Snap{snapdSnap, pcKernelSnap, core20Snap, pcSnap, required20Snap}
+	all := []*seed.Snap{snapdSnap, pcKernelSnap, core20Snap, pcSnap, core18Snap, required18Snap}
 
 	tests := []struct {
 		onlyTypes []snap.Type
@@ -793,7 +806,7 @@ func (s *seed20Suite) TestLoadEssentialMetaCore20(c *C) {
 		essSeed20, ok := seed20.(seed.EssentialMetaLoaderSeed)
 		c.Assert(ok, Equals, true)
 
-		err = essSeed20.LoadAssertions(s.db, s.commitTo)
+		err = essSeed20.LoadAssertions(nil, nil)
 		c.Assert(err, IsNil)
 
 		err = essSeed20.LoadEssentialMeta(t.onlyTypes, s.perfTimings)
@@ -811,6 +824,14 @@ func (s *seed20Suite) TestLoadEssentialMetaCore20(c *C) {
 		c.Check(runSnaps, HasLen, 0)
 
 		unhide()
+
+		// test short-cut helper as well
+		mod, essSnaps, err := seed.ReadSystemEssential(s.SeedDir, sysLabel, t.onlyTypes, s.perfTimings)
+		c.Assert(err, IsNil)
+		c.Check(mod.BrandID(), Equals, "my-brand")
+		c.Check(mod.Model(), Equals, "my-model")
+		c.Check(essSnaps, HasLen, len(t.expected))
+		c.Check(essSnaps, DeepEquals, t.expected)
 	}
 }
 
