@@ -33,6 +33,8 @@ import (
 	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/seed/seedwriter"
 	"github.com/snapcore/snapd/snap"
+	"github.com/snapcore/snapd/snap/naming"
+	"github.com/snapcore/snapd/snap/snapfile"
 	"github.com/snapcore/snapd/snap/snaptest"
 )
 
@@ -54,6 +56,10 @@ func (ss *SeedSnaps) SetupAssertSigning(storeBrandID string) {
 }
 
 func (ss *SeedSnaps) AssertedSnapID(snapName string) string {
+	snapID := naming.WellKnownSnapID(snapName)
+	if snapID != "" {
+		return snapID
+	}
 	return snaptest.AssertedSnapID(snapName)
 }
 
@@ -213,7 +219,7 @@ type TestingSeed20 struct {
 	SeedDir string
 }
 
-func (s *TestingSeed20) MakeSeed(c *C, label, brandID, modelID string, modelHeaders map[string]interface{}, optSnaps []*seedwriter.OptionsSnap) {
+func (s *TestingSeed20) MakeSeed(c *C, label, brandID, modelID string, modelHeaders map[string]interface{}, optSnaps []*seedwriter.OptionsSnap) *asserts.Model {
 	model := s.Brands.Model(brandID, modelID, modelHeaders)
 
 	db, err := asserts.OpenDatabase(&asserts.DatabaseConfig{
@@ -239,7 +245,7 @@ func (s *TestingSeed20) MakeSeed(c *C, label, brandID, modelID string, modelHead
 		}
 		return asserts.NewFetcher(db, retrieve, save2)
 	}
-	assertstest.AddMany(s.StoreSigning, s.Brands.AccountsAndKeys("my-brand")...)
+	assertstest.AddMany(s.StoreSigning, s.Brands.AccountsAndKeys(brandID)...)
 
 	opts := seedwriter.Options{
 		SeedDir: s.SeedDir,
@@ -262,7 +268,7 @@ func (s *TestingSeed20) MakeSeed(c *C, label, brandID, modelID string, modelHead
 		if !asserts.IsNotFound(err) {
 			c.Assert(err, IsNil)
 		}
-		f, err := snap.Open(sn.Path)
+		f, err := snapfile.Open(sn.Path)
 		c.Assert(err, IsNil)
 		info, err := snap.ReadInfoFromSnapFile(f, si)
 		c.Assert(err, IsNil)
@@ -281,7 +287,7 @@ func (s *TestingSeed20) MakeSeed(c *C, label, brandID, modelID string, modelHead
 			name := sn.SnapName()
 
 			info := s.AssertedSnapInfo(name)
-			c.Assert(info, NotNil, Commentf("%s", name))
+			c.Assert(info, NotNil, Commentf("no snap info for %q", name))
 			err := w.SetInfo(sn, info)
 			c.Assert(err, IsNil)
 
@@ -290,6 +296,11 @@ func (s *TestingSeed20) MakeSeed(c *C, label, brandID, modelID string, modelHead
 			c.Assert(err, IsNil)
 			sn.ARefs = rf.Refs()[prev:]
 
+			if _, err := os.Stat(sn.Path); err == nil {
+				// snap is already present
+				continue
+			}
+
 			err = os.Rename(s.AssertedSnap(name), sn.Path)
 			c.Assert(err, IsNil)
 		}
@@ -310,4 +321,6 @@ func (s *TestingSeed20) MakeSeed(c *C, label, brandID, modelID string, modelHead
 
 	err = w.WriteMeta()
 	c.Assert(err, IsNil)
+
+	return model
 }
