@@ -25,7 +25,6 @@ import (
 	"fmt"
 	"os"
 	"path/filepath"
-	"regexp"
 
 	"github.com/snapcore/snapd/asserts"
 	"github.com/snapcore/snapd/seed/internal"
@@ -34,15 +33,6 @@ import (
 	"github.com/snapcore/snapd/snap/naming"
 )
 
-var validSystemLabel = regexp.MustCompile("^[a-zA-Z0-9]+(?:-[a-zA-Z0-9]+)*$")
-
-func validateSystemLabel(label string) error {
-	if !validSystemLabel.MatchString(label) {
-		return fmt.Errorf("system label contains invalid characters: %s", label)
-	}
-	return nil
-}
-
 type policy20 struct {
 	model *asserts.Model
 	opts  *Options
@@ -50,7 +40,7 @@ type policy20 struct {
 	warningf func(format string, a ...interface{})
 }
 
-var errNotAllowedExceptForDangerous = errors.New("cannot override channels, add local snaps or extra snaps with a model of grade higher than dangerous")
+var errNotAllowedExceptForDangerous = errors.New("cannot override channels, add devmode snaps, local snaps, or extra snaps with a model of grade higher than dangerous")
 
 func (pol *policy20) checkAllowedDangerous() error {
 	if pol.model.Grade() != asserts.ModelDangerous {
