@@ -72,7 +72,8 @@ type appYaml struct {
 	Command      string   `yaml:"command"`
 	CommandChain []string `yaml:"command-chain,omitempty"`
 
-	Daemon string `yaml:"daemon"`
+	Daemon      string      `yaml:"daemon"`
+	DaemonScope DaemonScope `yaml:"daemon-scope"`
 
 	StopCommand     string          `yaml:"stop-command,omitempty"`
 	ReloadCommand   string          `yaml:"reload-command,omitempty"`
@@ -89,8 +90,9 @@ type appYaml struct {
 	SlotNames    []string         `yaml:"slots,omitempty"`
 	PlugNames    []string         `yaml:"plugs,omitempty"`
 
-	BusName  string `yaml:"bus-name,omitempty"`
-	CommonID string `yaml:"common-id,omitempty"`
+	BusName     string   `yaml:"bus-name,omitempty"`
+	ActivatesOn []string `yaml:"activates-on,omitempty"`
+	CommonID    string   `yaml:"common-id,omitempty"`
 
 	Environment strutil.OrderedMap `yaml:"environment,omitempty"`
 
@@ -349,6 +351,7 @@ func setAppsFromSnapYaml(y snapYaml, snap *Info, strk *scopedTracker) error {
 			CommandChain:    yApp.CommandChain,
 			StartTimeout:    yApp.StartTimeout,
 			Daemon:          yApp.Daemon,
+			DaemonScope:     yApp.DaemonScope,
 			StopTimeout:     yApp.StopTimeout,
 			StopCommand:     yApp.StopCommand,
 			ReloadCommand:   yApp.ReloadCommand,
@@ -375,6 +378,13 @@ func setAppsFromSnapYaml(y snapYaml, snap *Info, strk *scopedTracker) error {
 		if len(yApp.Sockets) > 0 {
 			app.Sockets = make(map[string]*SocketInfo, len(yApp.Sockets))
 		}
+		if len(yApp.ActivatesOn) > 0 {
+			app.ActivatesOn = make([]*SlotInfo, 0, len(yApp.ActivatesOn))
+		}
+		// Daemons default to being system daemons
+		if app.Daemon != "" && app.DaemonScope == "" {
+			app.DaemonScope = SystemDaemon
+		}
 
 		snap.Apps[appName] = app
 		for _, alias := range app.LegacyAliases {
@@ -417,6 +427,17 @@ func setAppsFromSnapYaml(y snapYaml, snap *Info, strk *scopedTracker) error {
 			app.Slots[slotName] = slot
 			slot.Apps[appName] = app
 		}
+		for _, slotName := range yApp.ActivatesOn {
+			slot, ok := snap.Slots[slotName]
+			if !ok {
+				return fmt.Errorf("invalid activates-on value %q on app %q: slot not found", slotName, appName)
+			}
+			app.ActivatesOn = append(app.ActivatesOn, slot)
+			// Implicitly add the slot to the app
+			strk.markSlot(slot)
+			app.Slots[slotName] = slot
+			slot.Apps[appName] = app
+		}
 		for name, data := range yApp.Sockets {
 			app.Sockets[name] = &SocketInfo{
 				App:          app,
