@@ -61,7 +61,7 @@ func (s *InfoSnapYamlTestSuite) TestSimple(c *C) {
 	c.Assert(err, IsNil)
 	c.Assert(info.InstanceName(), Equals, "foo")
 	c.Assert(info.Version, Equals, "1.0")
-	c.Assert(info.GetType(), Equals, snap.TypeApp)
+	c.Assert(info.Type(), Equals, snap.TypeApp)
 	c.Assert(info.Epoch, DeepEquals, snap.E("0"))
 }
 
@@ -71,7 +71,7 @@ version: 1.0`))
 	c.Assert(err, IsNil)
 	c.Assert(info.InstanceName(), Equals, "snapd")
 	c.Assert(info.Version, Equals, "1.0")
-	c.Assert(info.GetType(), Equals, snap.TypeSnapd)
+	c.Assert(info.Type(), Equals, snap.TypeSnapd)
 }
 
 func (s *InfoSnapYamlTestSuite) TestFail(c *C) {
@@ -1225,7 +1225,7 @@ slots:
 	c.Assert(err, IsNil)
 	c.Check(info.InstanceName(), Equals, "foo")
 	c.Check(info.Version, Equals, "1.2")
-	c.Check(info.GetType(), Equals, snap.TypeApp)
+	c.Check(info.Type(), Equals, snap.TypeApp)
 	c.Check(info.Epoch, DeepEquals, snap.E("1*"))
 	c.Check(info.Confinement, Equals, snap.DevModeConfinement)
 	c.Check(info.Title(), Equals, "Foo")
@@ -1350,6 +1350,64 @@ slots:
 		app1.Name: app1, app2.Name: app2})
 }
 
+func (s *YamlSuite) TestUnmarshalActivatesOn(c *C) {
+	info, err := snap.InfoFromSnapYaml([]byte(`
+name: snap
+slots:
+    test-slot1:
+    test-slot2:
+apps:
+    daemon:
+        activates-on: ["test-slot1", "test-slot2"]
+    foo:
+`))
+	c.Assert(err, IsNil)
+	c.Check(info.InstanceName(), Equals, "snap")
+	c.Check(info.Plugs, HasLen, 0)
+	c.Check(info.Slots, HasLen, 2)
+	c.Check(info.Apps, HasLen, 2)
+	c.Check(info.Hooks, HasLen, 0)
+
+	app1 := info.Apps["daemon"]
+	app2 := info.Apps["foo"]
+	slot1 := info.Slots["test-slot1"]
+	slot2 := info.Slots["test-slot2"]
+
+	c.Assert(app1, Not(IsNil))
+	c.Check(app1.Name, Equals, "daemon")
+	c.Check(app1.ActivatesOn, DeepEquals, []*snap.SlotInfo{slot1, slot2})
+	// activates-on slots are implicitly added to the app
+	c.Check(app1.Slots, DeepEquals, map[string]*snap.SlotInfo{
+		slot1.Name: slot1, slot2.Name: slot2})
+
+	c.Assert(app2, Not(IsNil))
+	c.Check(app2.Name, Equals, "foo")
+	c.Check(app2.ActivatesOn, HasLen, 0)
+	// As slot has been bound to app1, it isn't implicitly applied here
+	c.Check(app2.Slots, HasLen, 0)
+
+	c.Assert(slot1, Not(IsNil))
+	c.Check(slot1.Name, Equals, "test-slot1")
+	c.Check(slot1.Apps, DeepEquals, map[string]*snap.AppInfo{
+		app1.Name: app1})
+
+	c.Assert(slot2, Not(IsNil))
+	c.Check(slot2.Name, Equals, "test-slot2")
+	c.Check(slot2.Apps, DeepEquals, map[string]*snap.AppInfo{
+		app1.Name: app1})
+}
+
+func (s *YamlSuite) TestUnmarshalActivatesOnUnknownSlot(c *C) {
+	info, err := snap.InfoFromSnapYaml([]byte(`
+name: snap
+apps:
+    daemon:
+        activates-on: ["test-slot"]
+`))
+	c.Check(info, IsNil)
+	c.Check(err, ErrorMatches, `invalid activates-on value "test-slot" on app "daemon": slot not found`)
+}
+
 // type and architectures
 
 func (s *YamlSuite) TestSnapYamlTypeDefault(c *C) {
@@ -1358,7 +1416,7 @@ version: 1.0
 `)
 	info, err := snap.InfoFromSnapYaml(y)
 	c.Assert(err, IsNil)
-	c.Assert(info.GetType(), Equals, snap.TypeApp)
+	c.Assert(info.Type(), Equals, snap.TypeApp)
 }
 
 func (s *YamlSuite) TestSnapYamlEpochDefault(c *C) {
@@ -1459,6 +1517,7 @@ apps:
    stop-timeout: 25s
    start-timeout: 42m
    daemon: forking
+   daemon-scope: system
    stop-command: stop-cmd
    post-stop-command: post-stop-cmd
    restart-condition: on-abnormal
@@ -1476,6 +1535,7 @@ apps:
 		Name:            "svc",
 		Command:         "svc1",
 		Daemon:          "forking",
+		DaemonScope:     snap.SystemDaemon,
 		RestartCond:     snap.RestartOnAbnormal,
 		StopTimeout:     timeout.Timeout(25 * time.Second),
 		StartTimeout:    timeout.Timeout(42 * time.Minute),
@@ -1495,6 +1555,35 @@ apps:
 	c.Check(info.Apps, DeepEquals, map[string]*snap.AppInfo{"svc": &app})
 }
 
+func (s *YamlSuite) TestDaemonUserDaemon(c *C) {
+	y := []byte(`name: wat
+version: 42
+apps:
+ svc:
+   command: svc1
+   daemon: simple
+   daemon-scope: user
+`)
+	info, err := snap.InfoFromSnapYaml(y)
+	c.Assert(err, IsNil)
+	c.Check(info.Apps["svc"].DaemonScope, Equals, snap.UserDaemon)
+}
+
+func (s *YamlSuite) TestDaemonNoDaemonScope(c *C) {
+	y := []byte(`name: wat
+version: 42
+apps:
+ svc:
+   command: svc1
+   daemon: simple
+`)
+	info, err := snap.InfoFromSnapYaml(y)
+	c.Assert(err, IsNil)
+
+	// If daemon-scope is unset, default to system scope
+	c.Check(info.Apps["svc"].DaemonScope, Equals, snap.SystemDaemon)
+}
+
 func (s *YamlSuite) TestDaemonListenStreamAsInteger(c *C) {
 	y := []byte(`name: wat
 version: 42
@@ -1542,6 +1631,18 @@ apps:
 		"  line 9: cannot unmarshal !!str `asdfasdf` into os.FileMode")
 }
 
+func (s *YamlSuite) TestDaemonInvalidDaemonScope(c *C) {
+	y := []byte(`name: wat
+version: 42
+apps:
+ svc:
+   command: svc
+   daemon-scope: invalid
+`)
+	_, err := snap.InfoFromSnapYaml(y)
+	c.Check(err.Error(), Equals, "cannot parse snap.yaml: invalid daemon scope: \"invalid\"")
+}
+
 func (s *YamlSuite) TestSnapYamlGlobalEnvironment(c *C) {
 	y := []byte(`
 name: foo
