@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2014-2016 Canonical Ltd
+ * Copyright (C) 2014-2020 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -33,7 +33,9 @@ import (
 	"gopkg.in/yaml.v2"
 
 	"github.com/snapcore/snapd/dirs"
+	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/snap"
+	"github.com/snapcore/snapd/snap/snapfile"
 	"github.com/snapcore/snapd/snap/snaptest"
 	"github.com/snapcore/snapd/snap/squashfs"
 	"github.com/snapcore/snapd/testutil"
@@ -394,14 +396,14 @@ epoch: 1*
 confinement: devmode`
 	snapPath := snaptest.MakeTestSnapWithFiles(c, yaml, nil)
 
-	snapf, err := snap.Open(snapPath)
+	snapf, err := snapfile.Open(snapPath)
 	c.Assert(err, IsNil)
 
 	info, err := snap.ReadInfoFromSnapFile(snapf, nil)
 	c.Assert(err, IsNil)
 	c.Check(info.InstanceName(), Equals, "foo")
 	c.Check(info.Version, Equals, "1.0")
-	c.Check(info.GetType(), Equals, snap.TypeApp)
+	c.Check(info.Type(), Equals, snap.TypeApp)
 	c.Check(info.Revision, Equals, snap.R(0))
 	c.Check(info.Epoch.String(), Equals, "1*")
 	c.Check(info.Confinement, Equals, snap.DevModeConfinement)
@@ -416,14 +418,14 @@ type: app
 confinement: classic`
 	snapPath := snaptest.MakeTestSnapWithFiles(c, yaml, nil)
 
-	snapf, err := snap.Open(snapPath)
+	snapf, err := snapfile.Open(snapPath)
 	c.Assert(err, IsNil)
 
 	info, err := snap.ReadInfoFromSnapFile(snapf, nil)
 	c.Assert(err, IsNil)
 	c.Check(info.InstanceName(), Equals, "foo")
 	c.Check(info.Version, Equals, "1.0")
-	c.Check(info.GetType(), Equals, snap.TypeApp)
+	c.Check(info.Type(), Equals, snap.TypeApp)
 	c.Check(info.Revision, Equals, snap.R(0))
 	c.Check(info.Confinement, Equals, snap.ClassicConfinement)
 	c.Check(info.NeedsDevMode(), Equals, false)
@@ -436,14 +438,14 @@ version: 1.0
 type: app`
 	snapPath := snaptest.MakeTestSnapWithFiles(c, yaml, nil)
 
-	snapf, err := snap.Open(snapPath)
+	snapf, err := snapfile.Open(snapPath)
 	c.Assert(err, IsNil)
 
 	info, err := snap.ReadInfoFromSnapFile(snapf, nil)
 	c.Assert(err, IsNil)
 	c.Check(info.InstanceName(), Equals, "foo")
 	c.Check(info.Version, Equals, "1.0")
-	c.Check(info.GetType(), Equals, snap.TypeApp)
+	c.Check(info.Type(), Equals, snap.TypeApp)
 	c.Check(info.Revision, Equals, snap.R(0))
 	c.Check(info.Epoch.String(), Equals, "0") // Defaults to 0
 	c.Check(info.Confinement, Equals, snap.StrictConfinement)
@@ -456,7 +458,7 @@ version: 1.0
 type: app`
 	snapPath := snaptest.MakeTestSnapWithFiles(c, yaml, nil)
 
-	snapf, err := snap.Open(snapPath)
+	snapf, err := snapfile.Open(snapPath)
 	c.Assert(err, IsNil)
 
 	info, err := snap.ReadInfoFromSnapFile(snapf, &snap.SideInfo{
@@ -466,7 +468,7 @@ type: app`
 	c.Assert(err, IsNil)
 	c.Check(info.InstanceName(), Equals, "baz")
 	c.Check(info.Version, Equals, "1.0")
-	c.Check(info.GetType(), Equals, snap.TypeApp)
+	c.Check(info.Type(), Equals, snap.TypeApp)
 	c.Check(info.Revision, Equals, snap.R(42))
 }
 
@@ -476,7 +478,7 @@ version: 1.0
 type: app`
 	snapPath := makeTestSnap(c, yaml)
 
-	snapf, err := snap.Open(snapPath)
+	snapf, err := snapfile.Open(snapPath)
 	c.Assert(err, IsNil)
 
 	_, err = snap.ReadInfoFromSnapFile(snapf, nil)
@@ -489,7 +491,7 @@ version: 1.0
 type: foo`
 	snapPath := makeTestSnap(c, yaml)
 
-	snapf, err := snap.Open(snapPath)
+	snapf, err := snapfile.Open(snapPath)
 	c.Assert(err, IsNil)
 
 	_, err = snap.ReadInfoFromSnapFile(snapf, nil)
@@ -502,7 +504,7 @@ version: 1.0
 confinement: foo`
 	snapPath := makeTestSnap(c, yaml)
 
-	snapf, err := snap.Open(snapPath)
+	snapf, err := snapfile.Open(snapPath)
 	c.Assert(err, IsNil)
 
 	_, err = snap.ReadInfoFromSnapFile(snapf, nil)
@@ -523,11 +525,9 @@ apps:
 	info, err := snap.InfoFromSnapYaml([]byte(yaml))
 	c.Assert(err, IsNil)
 
-	env := info.Apps["foo"].Env()
-	sort.Strings(env)
-	c.Check(env, DeepEquals, []string{
-		"app-k=app-v",
-		"global-k=global-v",
+	c.Check(info.Apps["foo"].EnvChain(), DeepEquals, []osutil.ExpandableEnv{
+		osutil.NewExpandableEnv("global-k", "global-v"),
+		osutil.NewExpandableEnv("app-k", "app-v"),
 	})
 }
 
@@ -547,12 +547,9 @@ apps:
 	info, err := snap.InfoFromSnapYaml([]byte(yaml))
 	c.Assert(err, IsNil)
 
-	env := info.Apps["foo"].Env()
-	sort.Strings(env)
-	c.Check(env, DeepEquals, []string{
-		"app-k=app-v",
-		"global-and-local=local-v",
-		"global-k=global-v",
+	c.Check(info.Apps["foo"].EnvChain(), DeepEquals, []osutil.ExpandableEnv{
+		osutil.NewExpandableEnv("global-k", "global-v", "global-and-local", "global-v"),
+		osutil.NewExpandableEnv("app-k", "app-v", "global-and-local", "local-v"),
 	})
 }
 
@@ -570,11 +567,9 @@ hooks:
 	info, err := snap.InfoFromSnapYaml([]byte(yaml))
 	c.Assert(err, IsNil)
 
-	env := info.Hooks["foo"].Env()
-	sort.Strings(env)
-	c.Check(env, DeepEquals, []string{
-		"app-k=app-v",
-		"global-k=global-v",
+	c.Check(info.Hooks["foo"].EnvChain(), DeepEquals, []osutil.ExpandableEnv{
+		osutil.NewExpandableEnv("global-k", "global-v"),
+		osutil.NewExpandableEnv("app-k", "app-v"),
 	})
 }
 
@@ -594,12 +589,9 @@ hooks:
 	info, err := snap.InfoFromSnapYaml([]byte(yaml))
 	c.Assert(err, IsNil)
 
-	env := info.Hooks["foo"].Env()
-	sort.Strings(env)
-	c.Check(env, DeepEquals, []string{
-		"app-k=app-v",
-		"global-and-local=local-v",
-		"global-k=global-v",
+	c.Check(info.Hooks["foo"].EnvChain(), DeepEquals, []osutil.ExpandableEnv{
+		osutil.NewExpandableEnv("global-k", "global-v", "global-and-local", "global-v"),
+		osutil.NewExpandableEnv("app-k", "app-v", "global-and-local", "local-v"),
 	})
 }
 
@@ -660,7 +652,7 @@ hooks:
   123abc:`
 	snapPath := makeTestSnap(c, yaml)
 
-	snapf, err := snap.Open(snapPath)
+	snapf, err := snapfile.Open(snapPath)
 	c.Assert(err, IsNil)
 
 	_, err = snap.ReadInfoFromSnapFile(snapf, nil)
@@ -672,7 +664,7 @@ func (s *infoSuite) TestReadInfoFromSnapFileCatchesInvalidImplicitHook(c *C) {
 version: 1.0`
 	snapPath := snaptest.MakeTestSnapWithFiles(c, yaml, emptyHooks("123abc"))
 
-	snapf, err := snap.Open(snapPath)
+	snapf, err := snapfile.Open(snapPath)
 	c.Assert(err, IsNil)
 
 	_, err = snap.ReadInfoFromSnapFile(snapf, nil)
@@ -690,7 +682,7 @@ func (s *infoSuite) checkInstalledSnapAndSnapFile(c *C, instanceName, yaml strin
 
 	// Now check snap file
 	snapPath := snaptest.MakeTestSnapWithFiles(c, yaml, emptyHooks(hooks...))
-	snapf, err := snap.Open(snapPath)
+	snapf, err := snapfile.Open(snapPath)
 	c.Assert(err, IsNil)
 	info, err = snap.ReadInfoFromSnapFile(snapf, nil)
 	c.Check(err, IsNil)
@@ -1113,12 +1105,14 @@ func (s *infoSuite) TestAppDesktopFile(c *C) {
 	c.Check(snapInfo.InstanceName(), Equals, "sample")
 	c.Check(snapInfo.Apps["app"].DesktopFile(), Matches, `.*/var/lib/snapd/desktop/applications/sample_app.desktop`)
 	c.Check(snapInfo.Apps["sample"].DesktopFile(), Matches, `.*/var/lib/snapd/desktop/applications/sample_sample.desktop`)
+	c.Check(snapInfo.DesktopPrefix(), Equals, "sample")
 
 	// snap with instance key
 	snapInfo.InstanceKey = "instance"
 	c.Check(snapInfo.InstanceName(), Equals, "sample_instance")
-	c.Check(snapInfo.Apps["app"].DesktopFile(), Matches, `.*/var/lib/snapd/desktop/applications/sample_instance_app.desktop`)
-	c.Check(snapInfo.Apps["sample"].DesktopFile(), Matches, `.*/var/lib/snapd/desktop/applications/sample_instance_sample.desktop`)
+	c.Check(snapInfo.Apps["app"].DesktopFile(), Matches, `.*/var/lib/snapd/desktop/applications/sample\+instance_app.desktop`)
+	c.Check(snapInfo.Apps["sample"].DesktopFile(), Matches, `.*/var/lib/snapd/desktop/applications/sample\+instance_sample.desktop`)
+	c.Check(snapInfo.DesktopPrefix(), Equals, "sample+instance")
 }
 
 const coreSnapYaml = `name: core
@@ -1133,7 +1127,7 @@ plugs:
 func (s *infoSuite) TestReadInfoFromSnapFileRenamesCorePlus(c *C) {
 	snapPath := snaptest.MakeTestSnapWithFiles(c, coreSnapYaml, nil)
 
-	snapf, err := snap.Open(snapPath)
+	snapf, err := snapfile.Open(snapPath)
 	c.Assert(err, IsNil)
 
 	info, err := snap.ReadInfoFromSnapFile(snapf, nil)
@@ -1208,6 +1202,9 @@ apps:
     daemon: potato
   svc2:
     daemon: no
+  svc3:
+    daemon: simple
+    daemon-scope: user
   app1:
   app2:
 `))
@@ -1215,10 +1212,16 @@ apps:
 
 	svc := info.Apps["svc1"]
 	c.Check(svc.IsService(), Equals, true)
+	c.Check(svc.DaemonScope, Equals, snap.SystemDaemon)
 	c.Check(svc.ServiceName(), Equals, "snap.pans.svc1.service")
 	c.Check(svc.ServiceFile(), Equals, dirs.GlobalRootDir+"/etc/systemd/system/snap.pans.svc1.service")
 
 	c.Check(info.Apps["svc2"].IsService(), Equals, true)
+	userSvc := info.Apps["svc3"]
+	c.Check(userSvc.IsService(), Equals, true)
+	c.Check(userSvc.DaemonScope, Equals, snap.UserDaemon)
+	c.Check(userSvc.ServiceName(), Equals, "snap.pans.svc3.service")
+	c.Check(userSvc.ServiceFile(), Equals, dirs.GlobalRootDir+"/etc/systemd/user/snap.pans.svc3.service")
 	c.Check(info.Apps["app1"].IsService(), Equals, false)
 	c.Check(info.Apps["app1"].IsService(), Equals, false)
 
@@ -1226,6 +1229,8 @@ apps:
 	info.InstanceKey = "instance"
 	c.Check(svc.ServiceName(), Equals, "snap.pans_instance.svc1.service")
 	c.Check(svc.ServiceFile(), Equals, dirs.GlobalRootDir+"/etc/systemd/system/snap.pans_instance.svc1.service")
+	c.Check(userSvc.ServiceName(), Equals, "snap.pans_instance.svc3.service")
+	c.Check(userSvc.ServiceFile(), Equals, dirs.GlobalRootDir+"/etc/systemd/user/snap.pans_instance.svc3.service")
 }
 
 func (s *infoSuite) TestAppInfoStringer(c *C) {
@@ -1563,17 +1568,14 @@ func (s *infoSuite) TestIsActive(c *C) {
 	c.Check(info2.IsActive(), Equals, false)
 }
 
-func (s *infoSuite) TestGetTypeSnapdBackwardCompatibility(c *C) {
-	restore := snap.MockSnapdSnapID("snapd-id")
-	defer restore()
-
+func (s *infoSuite) TestInfoTypeSnapdBackwardCompatibility(c *C) {
 	const snapdYaml = `
 name: snapd
 type: app
 version: 1
 `
-	snapInfo := snaptest.MockSnap(c, sampleYaml, &snap.SideInfo{Revision: snap.R(1), SnapID: "snapd-id"})
-	c.Check(snapInfo.GetType(), Equals, snap.TypeSnapd)
+	snapInfo := snaptest.MockSnap(c, sampleYaml, &snap.SideInfo{Revision: snap.R(1), SnapID: "PMrrV4ml8uWuEUDBT8dSGnKUYbevVhc4"})
+	c.Check(snapInfo.Type(), Equals, snap.TypeSnapd)
 }
 
 func (s *infoSuite) TestDirAndFileHelpers(c *C) {
@@ -1740,3 +1742,22 @@ func (s *infoSuite) TestSortApps(c *C) {
 		}
 	}
 }
+
+func (s *infoSuite) TestSortAppInfoBySnapApp(c *C) {
+	snap1 := &snap.Info{SuggestedName: "snapa"}
+	snap2 := &snap.Info{SuggestedName: "snapb"}
+	infos := []*snap.AppInfo{
+		{Snap: snap1, Name: "b"},
+		{Snap: snap2, Name: "b"},
+		{Snap: snap1, Name: "a"},
+		{Snap: snap2, Name: "a"},
+	}
+	sort.Stable(snap.AppInfoBySnapApp(infos))
+
+	c.Check(infos, DeepEquals, []*snap.AppInfo{
+		{Snap: snap1, Name: "a"},
+		{Snap: snap1, Name: "b"},
+		{Snap: snap2, Name: "a"},
+		{Snap: snap2, Name: "b"},
+	})
+}
