@@ -20,6 +20,7 @@
 package pack_test
 
 import (
+	"bytes"
 	"fmt"
 	"io/ioutil"
 	"os"
@@ -27,15 +28,18 @@ import (
 	"path/filepath"
 	"regexp"
 	"strings"
-	"syscall"
 	"testing"
 
+	. "gopkg.in/check.v1"
+
 	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/snap"
 	"github.com/snapcore/snapd/snap/pack"
+	"github.com/snapcore/snapd/snap/squashfs"
 	"github.com/snapcore/snapd/testutil"
 
-	. "gopkg.in/check.v1"
+	// for SanitizePlugsSlots
+	_ "github.com/snapcore/snapd/interfaces/builtin"
 )
 
 func Test(t *testing.T) { TestingT(t) }
@@ -48,7 +52,6 @@ var _ = Suite(&packSuite{})
 
 func (s *packSuite) SetUpTest(c *C) {
 	s.BaseTest.SetUpTest(c)
-	s.BaseTest.AddCleanup(snap.MockSanitizePlugsSlots(func(snapInfo *snap.Info) {}))
 
 	// chdir into a tempdir
 	pwd, err := os.Getwd()
@@ -112,7 +115,7 @@ printf "hello world"
 func (s *packSuite) TestPackNoManifestFails(c *C) {
 	sourceDir := makeExampleSnapSourceDir(c, "{name: hello, version: 0}")
 	c.Assert(os.Remove(filepath.Join(sourceDir, "meta", "snap.yaml")), IsNil)
-	_, err := pack.Snap(sourceDir, "")
+	_, err := pack.Snap(sourceDir, pack.Defaults)
 	c.Assert(err, ErrorMatches, `.*/meta/snap\.yaml: no such file or directory`)
 }
 
@@ -124,56 +127,40 @@ apps:
   command: bin/hello-world
 `)
 	c.Assert(os.Remove(filepath.Join(sourceDir, "bin", "hello-world")), IsNil)
-	_, err := pack.Snap(sourceDir, "")
+	_, err := pack.Snap(sourceDir, pack.Defaults)
 	c.Assert(err, Equals, snap.ErrMissingPaths)
 }
 
 func (s *packSuite) TestValidateMissingAppFailsWithErrMissingPaths(c *C) {
+	var buf bytes.Buffer
 	sourceDir := makeExampleSnapSourceDir(c, `name: hello
 version: 0
 apps:
  foo:
   command: bin/hello-world
+  plugs: [potato]
 `)
-	c.Assert(os.Remove(filepath.Join(sourceDir, "bin", "hello-world")), IsNil)
-	err := pack.CheckSkeleton(sourceDir)
-	c.Assert(err, Equals, snap.ErrMissingPaths)
-}
-
-func (s *packSuite) TestCopyCopies(c *C) {
-	sourceDir := makeExampleSnapSourceDir(c, "{name: hello, version: 0}")
-	// actually this'll be on /tmp so it'll be a link
-	target := c.MkDir()
-	c.Assert(pack.CopyToBuildDir(sourceDir, target), IsNil)
-	out, err := exec.Command("diff", "-qrN", sourceDir, target).Output()
-	c.Check(err, IsNil)
-	c.Check(out, DeepEquals, []byte{})
-}
-
-func (s *packSuite) TestCopyActuallyCopies(c *C) {
-	sourceDir := makeExampleSnapSourceDir(c, "{name: hello, version: 0}")
-
-	// hoping to get the non-linking behaviour via /dev/shm
-	target, err := ioutil.TempDir("/dev/shm", "copy")
-	// sbuild environments won't allow writing to /dev/shm, so its
-	// ok to skip there
-	if os.IsPermission(err) {
-		c.Skip("/dev/shm is not writable for us")
-	}
+	err := pack.CheckSkeleton(&buf, sourceDir)
 	c.Assert(err, IsNil)
+	c.Check(buf.String(), Equals, "snap \"hello\" has bad plugs or slots: potato (unknown interface \"potato\")\n")
 
-	c.Assert(pack.CopyToBuildDir(sourceDir, target), IsNil)
-	out, err := exec.Command("diff", "-qrN", sourceDir, target).Output()
-	c.Check(err, IsNil)
-	c.Check(out, DeepEquals, []byte{})
+	buf.Reset()
+	c.Assert(os.Remove(filepath.Join(sourceDir, "bin", "hello-world")), IsNil)
+
+	err = pack.CheckSkeleton(&buf, sourceDir)
+	c.Assert(err, Equals, snap.ErrMissingPaths)
+	c.Check(buf.String(), Equals, "")
 }
 
-func (s *packSuite) TestCopyExcludesBackups(c *C) {
+func (s *packSuite) TestPackExcludesBackups(c *C) {
 	sourceDir := makeExampleSnapSourceDir(c, "{name: hello, version: 0}")
 	target := c.MkDir()
 	// add a backup file
 	c.Assert(ioutil.WriteFile(filepath.Join(sourceDir, "foo~"), []byte("hi"), 0755), IsNil)
-	c.Assert(pack.CopyToBuildDir(sourceDir, target), IsNil)
+	snapfile, err := pack.Snap(sourceDir, &pack.Options{TargetDir: c.MkDir()})
+	c.Assert(err, IsNil)
+	c.Assert(squashfs.New(snapfile).Unpack("*", target), IsNil)
+
 	cmd := exec.Command("diff", "-qr", sourceDir, target)
 	cmd.Env = append(cmd.Env, "LANG=C")
 	out, err := cmd.Output()
@@ -181,14 +168,16 @@ func (s *packSuite) TestCopyExcludesBackups(c *C) {
 	c.Check(string(out), Matches, `(?m)Only in \S+: foo~`)
 }
 
-func (s *packSuite) TestCopyExcludesTopLevelDEBIAN(c *C) {
+func (s *packSuite) TestPackExcludesTopLevelDEBIAN(c *C) {
 	sourceDir := makeExampleSnapSourceDir(c, "{name: hello, version: 0}")
 	target := c.MkDir()
 	// add a toplevel DEBIAN
 	c.Assert(os.MkdirAll(filepath.Join(sourceDir, "DEBIAN", "foo"), 0755), IsNil)
 	// and a non-toplevel DEBIAN
 	c.Assert(os.MkdirAll(filepath.Join(sourceDir, "bar", "DEBIAN", "baz"), 0755), IsNil)
-	c.Assert(pack.CopyToBuildDir(sourceDir, target), IsNil)
+	snapfile, err := pack.Snap(sourceDir, &pack.Options{TargetDir: c.MkDir()})
+	c.Assert(err, IsNil)
+	c.Assert(squashfs.New(snapfile).Unpack("*", target), IsNil)
 	cmd := exec.Command("diff", "-qr", sourceDir, target)
 	cmd.Env = append(cmd.Env, "LANG=C")
 	out, err := cmd.Output()
@@ -198,61 +187,30 @@ func (s *packSuite) TestCopyExcludesTopLevelDEBIAN(c *C) {
 	c.Check(strings.Count(string(out), "Only in"), Equals, 1)
 }
 
-func (s *packSuite) TestCopyExcludesWholeDirs(c *C) {
+func (s *packSuite) TestPackExcludesWholeDirs(c *C) {
 	sourceDir := makeExampleSnapSourceDir(c, "{name: hello, version: 0}")
 	target := c.MkDir()
 	// add a file inside a skipped dir
 	c.Assert(os.Mkdir(filepath.Join(sourceDir, ".bzr"), 0755), IsNil)
 	c.Assert(ioutil.WriteFile(filepath.Join(sourceDir, ".bzr", "foo"), []byte("hi"), 0755), IsNil)
-	c.Assert(pack.CopyToBuildDir(sourceDir, target), IsNil)
+	snapfile, err := pack.Snap(sourceDir, &pack.Options{TargetDir: c.MkDir()})
+	c.Assert(err, IsNil)
+	c.Assert(squashfs.New(snapfile).Unpack("*", target), IsNil)
 	out, _ := exec.Command("find", sourceDir).Output()
 	c.Check(string(out), Not(Equals), "")
 	cmd := exec.Command("diff", "-qr", sourceDir, target)
 	cmd.Env = append(cmd.Env, "LANG=C")
-	out, err := cmd.Output()
+	out, err = cmd.Output()
 	c.Check(err, NotNil)
 	c.Check(string(out), Matches, `(?m)Only in \S+: \.bzr`)
 }
 
-func (s *packSuite) TestExcludeDynamicFalseIfNoSnapignore(c *C) {
-	basedir := c.MkDir()
-	c.Check(pack.ShouldExcludeDynamic(basedir, "foo"), Equals, false)
-}
-
-func (s *packSuite) TestExcludeDynamicWorksIfSnapignore(c *C) {
-	basedir := c.MkDir()
-	c.Assert(ioutil.WriteFile(filepath.Join(basedir, ".snapignore"), []byte("foo\nb.r\n"), 0644), IsNil)
-	c.Check(pack.ShouldExcludeDynamic(basedir, "foo"), Equals, true)
-	c.Check(pack.ShouldExcludeDynamic(basedir, "bar"), Equals, true)
-	c.Check(pack.ShouldExcludeDynamic(basedir, "bzr"), Equals, true)
-	c.Check(pack.ShouldExcludeDynamic(basedir, "baz"), Equals, false)
-}
-
-func (s *packSuite) TestExcludeDynamicWeirdRegexps(c *C) {
-	basedir := c.MkDir()
-	c.Assert(ioutil.WriteFile(filepath.Join(basedir, ".snapignore"), []byte("*hello\n"), 0644), IsNil)
-	// note "*hello" is not a valid regexp, so will be taken literally (not globbed!)
-	c.Check(pack.ShouldExcludeDynamic(basedir, "ahello"), Equals, false)
-	c.Check(pack.ShouldExcludeDynamic(basedir, "*hello"), Equals, true)
-}
-
 func (s *packSuite) TestDebArchitecture(c *C) {
 	c.Check(pack.DebArchitecture(&snap.Info{Architectures: []string{"foo"}}), Equals, "foo")
 	c.Check(pack.DebArchitecture(&snap.Info{Architectures: []string{"foo", "bar"}}), Equals, "multi")
 	c.Check(pack.DebArchitecture(&snap.Info{Architectures: nil}), Equals, "all")
 }
 
-func (s *packSuite) TestPackFailsForUnknownType(c *C) {
-	sourceDir := makeExampleSnapSourceDir(c, `name: hello
-version: 1.0.1
-`)
-	err := syscall.Mkfifo(filepath.Join(sourceDir, "fifo"), 0644)
-	c.Assert(err, IsNil)
-
-	_, err = pack.Snap(sourceDir, "")
-	c.Assert(err, ErrorMatches, "cannot handle type of file .*")
-}
-
 func (s *packSuite) TestPackSimple(c *C) {
 	sourceDir := makeExampleSnapSourceDir(c, `name: hello
 version: 1.0.1
@@ -262,55 +220,136 @@ integration:
   apparmor-profile: meta/hello.apparmor
 `)
 
-	resultSnap, err := pack.Snap(sourceDir, "")
-	c.Assert(err, IsNil)
+	outputDir := filepath.Join(c.MkDir(), "output")
+	absSnapFile := filepath.Join(c.MkDir(), "foo.snap")
 
-	// check that there is result
-	_, err = os.Stat(resultSnap)
-	c.Assert(err, IsNil)
-	c.Assert(resultSnap, Equals, "hello_1.0.1_multi.snap")
-
-	// check that the content looks sane
-	output, err := exec.Command("unsquashfs", "-ll", "hello_1.0.1_multi.snap").CombinedOutput()
-	c.Assert(err, IsNil)
-	for _, needle := range []string{
-		"meta/snap.yaml",
-		"bin/hello-world",
-		"symlink -> bin/hello-world",
-	} {
-		expr := fmt.Sprintf(`(?ms).*%s.*`, regexp.QuoteMeta(needle))
-		c.Assert(string(output), Matches, expr)
+	type T struct {
+		outputDir, filename, expected string
 	}
+
+	table := []T{
+		// no output dir, no filename -> default in .
+		{"", "", "hello_1.0.1_multi.snap"},
+		// no output dir, relative filename -> filename in .
+		{"", "foo.snap", "foo.snap"},
+		// no putput dir, absolute filename -> absolute filename
+		{"", absSnapFile, absSnapFile},
+		// output dir, no filename -> default in outputdir
+		{outputDir, "", filepath.Join(outputDir, "hello_1.0.1_multi.snap")},
+		// output dir, relative filename -> filename in outputDir
+		{filepath.Join(outputDir, "inner"), "../foo.snap", filepath.Join(outputDir, "foo.snap")},
+		// output dir, absolute filename -> absolute filename
+		{outputDir, absSnapFile, absSnapFile},
+	}
+
+	for i, t := range table {
+		comm := Commentf("%d", i)
+		resultSnap, err := pack.Snap(sourceDir, &pack.Options{
+			TargetDir: t.outputDir,
+			SnapName:  t.filename,
+		})
+		c.Assert(err, IsNil, comm)
+
+		// check that there is result
+		_, err = os.Stat(resultSnap)
+		c.Assert(err, IsNil, comm)
+		c.Assert(resultSnap, Equals, t.expected, comm)
+
+		// check that the content looks sane
+		output, err := exec.Command("unsquashfs", "-ll", resultSnap).CombinedOutput()
+		c.Assert(err, IsNil, comm)
+		for _, needle := range []string{
+			"meta/snap.yaml",
+			"bin/hello-world",
+			"symlink -> bin/hello-world",
+		} {
+			expr := fmt.Sprintf(`(?ms).*%s.*`, regexp.QuoteMeta(needle))
+			c.Assert(string(output), Matches, expr, comm)
+		}
+	}
+
 }
 
-func (s *packSuite) TestPackSimpleOutputDir(c *C) {
-	sourceDir := makeExampleSnapSourceDir(c, `name: hello
+func (s *packSuite) TestPackGadgetValidate(c *C) {
+	sourceDir := makeExampleSnapSourceDir(c, `name: funky-gadget
 version: 1.0.1
-architectures: ["i386", "amd64"]
-integration:
- app:
-  apparmor-profile: meta/hello.apparmor
+type: gadget
 `)
 
+	var gadgetYamlContent = `
+volumes:
+  bad:
+    bootloader: grub
+    structure:
+      - name: fs-struct
+        type: DA,21686148-6449-6E6F-744E-656564454649
+        size: 1M
+        filesystem: ext4
+        content:
+          - source: foo/
+            target: /
+      - name: bare-struct
+        type: DA,21686148-6449-6E6F-744E-656564454649
+        size: 1M
+        content:
+          - image: bare.img
+
+`
+	err := ioutil.WriteFile(filepath.Join(sourceDir, "meta/gadget.yaml"), []byte(gadgetYamlContent), 0644)
+	c.Assert(err, IsNil)
+
 	outputDir := filepath.Join(c.MkDir(), "output")
-	snapOutput := filepath.Join(outputDir, "hello_1.0.1_multi.snap")
-	resultSnap, err := pack.Snap(sourceDir, outputDir)
+	absSnapFile := filepath.Join(c.MkDir(), "foo.snap")
+
+	// gadget validation fails during layout
+	_, err = pack.Snap(sourceDir, &pack.Options{
+		TargetDir: outputDir,
+		SnapName:  absSnapFile,
+	})
+	c.Assert(err, ErrorMatches, `invalid layout of volume "bad": cannot lay out structure #1 \("bare-struct"\): content "bare.img": stat .*/bare.img: no such file or directory`)
+
+	err = ioutil.WriteFile(filepath.Join(sourceDir, "bare.img"), []byte("foo"), 0644)
 	c.Assert(err, IsNil)
 
-	// check that there is result
-	_, err = os.Stat(resultSnap)
-	c.Assert(err, IsNil)
-	c.Assert(resultSnap, Equals, snapOutput)
+	// gadget validation fails during content presence checks
+	_, err = pack.Snap(sourceDir, &pack.Options{
+		TargetDir: outputDir,
+		SnapName:  absSnapFile,
+	})
+	c.Assert(err, ErrorMatches, `invalid volume "bad": structure #0 \("fs-struct"\), content source:foo/: source path does not exist`)
 
-	// check that the content looks sane
-	output, err := exec.Command("unsquashfs", "-ll", resultSnap).CombinedOutput()
+	err = os.Mkdir(filepath.Join(sourceDir, "foo"), 0644)
 	c.Assert(err, IsNil)
-	for _, needle := range []string{
-		"meta/snap.yaml",
-		"bin/hello-world",
-		"symlink -> bin/hello-world",
-	} {
-		expr := fmt.Sprintf(`(?ms).*%s.*`, regexp.QuoteMeta(needle))
-		c.Assert(string(output), Matches, expr)
+	// all good now
+	_, err = pack.Snap(sourceDir, &pack.Options{
+		TargetDir: outputDir,
+		SnapName:  absSnapFile,
+	})
+	c.Assert(err, IsNil)
+}
+
+func (s *packSuite) TestPackWithCompressionHappy(c *C) {
+	sourceDir := makeExampleSnapSourceDir(c, "{name: hello, version: 0}")
+
+	for _, comp := range []string{"", "xz", "lzo"} {
+		snapfile, err := pack.Snap(sourceDir, &pack.Options{
+			TargetDir:   c.MkDir(),
+			Compression: comp,
+		})
+		c.Assert(err, IsNil)
+		c.Assert(snapfile, testutil.FilePresent)
+	}
+}
+
+func (s *packSuite) TestPackWithCompressionUnhappy(c *C) {
+	sourceDir := makeExampleSnapSourceDir(c, "{name: hello, version: 0}")
+
+	for _, comp := range []string{"gzip", "zstd", "silly"} {
+		snapfile, err := pack.Snap(sourceDir, &pack.Options{
+			TargetDir:   c.MkDir(),
+			Compression: comp,
+		})
+		c.Assert(err, ErrorMatches, fmt.Sprintf("cannot use compression %q", comp))
+		c.Assert(snapfile, Equals, "")
 	}
 }
