@@ -48,6 +48,25 @@ func ValidateName(name string) error {
 	return naming.ValidateSnap(name)
 }
 
+// ValidateDesktopPrefix checks if a string can be used as a desktop file
+// prefix. A desktop prefix should be of the form 'snapname' or
+// 'snapname+instance'.
+func ValidateDesktopPrefix(prefix string) bool {
+	tokens := strings.Split(prefix, "+")
+	if len(tokens) == 0 || len(tokens) > 2 {
+		return false
+	}
+	if err := ValidateName(tokens[0]); err != nil {
+		return false
+	}
+	if len(tokens) == 2 {
+		if err := ValidateInstanceName(tokens[1]); err != nil {
+			return false
+		}
+	}
+	return true
+}
+
 // ValidatePlugName checks if a string can be used as a slot name.
 //
 // Slot names and plug names within one snap must have unique names.
@@ -198,9 +217,19 @@ func validateSocketAddrPath(socket *SocketInfo, fieldName string, path string) e
 		return fmt.Errorf("invalid %q: %q should be written as %q", fieldName, path, clean)
 	}
 
-	if !(strings.HasPrefix(path, "$SNAP_DATA/") || strings.HasPrefix(path, "$SNAP_COMMON/") || strings.HasPrefix(path, "$XDG_RUNTIME_DIR/")) {
-		return fmt.Errorf(
-			"invalid %q: must have a prefix of $SNAP_DATA, $SNAP_COMMON or $XDG_RUNTIME_DIR", fieldName)
+	switch socket.App.DaemonScope {
+	case SystemDaemon:
+		if !(strings.HasPrefix(path, "$SNAP_DATA/") || strings.HasPrefix(path, "$SNAP_COMMON/") || strings.HasPrefix(path, "$XDG_RUNTIME_DIR/")) {
+			return fmt.Errorf(
+				"invalid %q: system daemon sockets must have a prefix of $SNAP_DATA, $SNAP_COMMON or $XDG_RUNTIME_DIR", fieldName)
+		}
+	case UserDaemon:
+		if !(strings.HasPrefix(path, "$SNAP_USER_DATA/") || strings.HasPrefix(path, "$SNAP_USER_COMMON/") || strings.HasPrefix(path, "$XDG_RUNTIME_DIR/")) {
+			return fmt.Errorf(
+				"invalid %q: user daemon sockets must have a prefix of $SNAP_USER_DATA, $SNAP_USER_COMMON, or $XDG_RUNTIME_DIR", fieldName)
+		}
+	default:
+		return fmt.Errorf("invalid %q: cannot validate sockets for daemon-scope %q", fieldName, socket.App.DaemonScope)
 	}
 
 	return nil
@@ -360,9 +389,9 @@ func Validate(info *Info) error {
 // ValidateBase validates the base field.
 func ValidateBase(info *Info) error {
 	// validate that bases do not have base fields
-	if info.GetType() == TypeOS || info.GetType() == TypeBase {
+	if info.Type() == TypeOS || info.Type() == TypeBase {
 		if info.Base != "" && info.Base != "none" {
-			return fmt.Errorf(`cannot have "base" field on %q snap %q`, info.GetType(), info.InstanceName())
+			return fmt.Errorf(`cannot have "base" field on %q snap %q`, info.Type(), info.InstanceName())
 		}
 	}
 
@@ -390,6 +419,38 @@ func ValidateLayoutAll(info *Info) error {
 	}
 	sort.Strings(paths)
 
+	// Validate that each source path is not a new top-level directory
+	for _, layout := range info.Layout {
+		cleanPathSrc := info.ExpandSnapVariables(filepath.Clean(layout.Path))
+		elems := strings.SplitN(cleanPathSrc, string(os.PathSeparator), 3)
+		switch len(elems) {
+		// len(1) is either relative path or empty string, will be validated
+		// elsewhere
+		case 2, 3:
+			// if the first string is the empty string, then we have a top-level
+			// directory to check
+			if elems[0] != "" {
+				// not the empty string which means this was a relative
+				// specification, i.e. usr/src/doc
+				return fmt.Errorf("layout %q is a relative filename", layout.Path)
+			}
+			if elems[1] != "" {
+				// verify that the top-level directory is a supported one
+				// we can't create new top-level directories because that would
+				// require creating a mimic on top of "/" which we don't
+				// currently support
+				switch elems[1] {
+				// this list was produced by taking all of the top level
+				// directories in the core snap and removing the explicitly
+				// denied top-level directories
+				case "bin", "etc", "lib", "lib64", "meta", "mnt", "opt", "root", "sbin", "snap", "srv", "usr", "var", "writable":
+				default:
+					return fmt.Errorf("layout %q defines a new top-level directory %q", layout.Path, "/"+elems[1])
+				}
+			}
+		}
+	}
+
 	// Validate that each source path is used consistently as a file or as a directory.
 	sourceKindMap := make(map[string]string)
 	for _, path := range paths {
@@ -510,6 +571,10 @@ func validateAppOrderNames(app *AppInfo, dependencies []string) error {
 		if !other.IsService() {
 			return fmt.Errorf("before/after references a non-service application %q", dep)
 		}
+
+		if app.DaemonScope != other.DaemonScope {
+			return fmt.Errorf("before/after references service with different daemon-scope %q", dep)
+		}
 	}
 	return nil
 }
@@ -578,6 +643,43 @@ func validateAppRestart(app *AppInfo) error {
 	return nil
 }
 
+func validateAppActivatesOn(app *AppInfo) error {
+	if len(app.ActivatesOn) == 0 {
+		return nil
+	}
+
+	if !app.IsService() {
+		return errors.New("activates-on is only applicable to services")
+	}
+
+	for _, slot := range app.ActivatesOn {
+		// ActivatesOn slots must use the "dbus" interface
+		if slot.Interface != "dbus" {
+			return fmt.Errorf("invalid activates-on value %q: slot does not use dbus interface", slot.Name)
+		}
+
+		// D-Bus slots must match the daemon scope
+		bus := slot.Attrs["bus"]
+		if app.DaemonScope == SystemDaemon && bus != "system" || app.DaemonScope == UserDaemon && bus != "session" {
+			return fmt.Errorf("invalid activates-on value %q: bus %q does not match daemon-scope %q", slot.Name, bus, app.DaemonScope)
+		}
+
+		// Slots must only be activatable on a single app
+		for _, otherApp := range slot.Apps {
+			if otherApp == app {
+				continue
+			}
+			for _, otherSlot := range otherApp.ActivatesOn {
+				if otherSlot == slot {
+					return fmt.Errorf("invalid activates-on value %q: slot is also activatable on app %q", slot.Name, otherApp.Name)
+				}
+			}
+		}
+	}
+
+	return nil
+}
+
 // appContentWhitelist is the whitelist of legal chars in the "apps"
 // section of snap.yaml. Do not allow any of [',",`] here or snap-exec
 // will get confused. chainContentWhitelist is the same, but for the
@@ -599,6 +701,19 @@ func ValidateApp(app *AppInfo) error {
 		return fmt.Errorf(`"daemon" field contains invalid value %q`, app.Daemon)
 	}
 
+	switch app.DaemonScope {
+	case "":
+		if app.Daemon != "" {
+			return fmt.Errorf(`"daemon-scope" must be set for daemons`)
+		}
+	case SystemDaemon, UserDaemon:
+		if app.Daemon == "" {
+			return fmt.Errorf(`"daemon-scope" can only be set for daemons`)
+		}
+	default:
+		return fmt.Errorf(`invalid "daemon-scope": %q`, app.DaemonScope)
+	}
+
 	// Validate app name
 	if !ValidAppName(app.Name) {
 		return fmt.Errorf("cannot have %q as app name - use letters, digits, and dash as separator", app.Name)
@@ -639,6 +754,10 @@ func ValidateApp(app *AppInfo) error {
 		}
 	}
 
+	if err := validateAppActivatesOn(app); err != nil {
+		return err
+	}
+
 	if err := validateAppRestart(app); err != nil {
 		return err
 	}
