@@ -136,7 +136,7 @@ func (s *downloadSuite) TestActualDownloadFullCloudInfoFromAuthContext(c *C) {
 	defer mockServer.Close()
 
 	device := createTestDevice()
-	theStore := store.New(&store.Config{}, &testAuthContext{c: c, device: device, cloudInfo: &auth.CloudInfo{Name: "aws", Region: "us-east-1", AvailabilityZone: "us-east-1c"}})
+	theStore := store.New(&store.Config{}, &testDauthContext{c: c, device: device, cloudInfo: &auth.CloudInfo{Name: "aws", Region: "us-east-1", AvailabilityZone: "us-east-1c"}})
 
 	var buf SillyBuffer
 	// keep tests happy
@@ -156,7 +156,7 @@ func (s *downloadSuite) TestActualDownloadLessDetailedCloudInfoFromAuthContext(c
 	defer mockServer.Close()
 
 	device := createTestDevice()
-	theStore := store.New(&store.Config{}, &testAuthContext{c: c, device: device, cloudInfo: &auth.CloudInfo{Name: "openstack", Region: "", AvailabilityZone: "nova"}})
+	theStore := store.New(&store.Config{}, &testDauthContext{c: c, device: device, cloudInfo: &auth.CloudInfo{Name: "openstack", Region: "", AvailabilityZone: "nova"}})
 
 	var buf SillyBuffer
 	// keep tests happy
@@ -336,6 +336,7 @@ func (s *downloadSuite) TestActualDownloadResume(c *C) {
 	n := 0
 	mockServer := httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
 		n++
+		w.WriteHeader(206)
 		io.WriteString(w, "data")
 	}))
 	c.Assert(mockServer, NotNil)
@@ -353,6 +354,36 @@ func (s *downloadSuite) TestActualDownloadResume(c *C) {
 	c.Check(n, Equals, 1)
 }
 
+func (s *downloadSuite) TestActualDownloadServerNoResumeHandeled(c *C) {
+	n := 0
+	mockServer := httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
+		n++
+
+		switch n {
+		case 1:
+			c.Check(r.Header["Range"], HasLen, 1)
+		default:
+			c.Fatal("only one request expected")
+		}
+		// server does not do partial content and sends full data instead
+		w.WriteHeader(200)
+		io.WriteString(w, "some data")
+	}))
+	c.Assert(mockServer, NotNil)
+	defer mockServer.Close()
+
+	theStore := store.New(&store.Config{}, nil)
+	buf := NewSillyBufferString("some ")
+	// calc the expected hash
+	h := crypto.SHA3_384.New()
+	h.Write([]byte("some data"))
+	sha3 := fmt.Sprintf("%x", h.Sum(nil))
+	err := store.Download(context.TODO(), "foo", sha3, mockServer.URL, nil, theStore, buf, int64(len("some ")), nil, nil)
+	c.Check(err, IsNil)
+	c.Check(buf.String(), Equals, "some data")
+	c.Check(n, Equals, 1)
+}
+
 func (s *downloadSuite) TestUseDeltas(c *C) {
 	origPath := os.Getenv("PATH")
 	defer os.Setenv("PATH", origPath)
