@@ -4,7 +4,7 @@
 gadget_name=$(snap list | grep 'gadget$' | awk '{ print $1 }')
 kernel_name=$(snap list | grep 'kernel$' | awk '{ print $1 }')
 
-core_name="$(snap known model | awk '/^base: / { print $2 }' || true)"
+core_name="$(snap model --verbose | grep -Po "^base:\\s+\\K.*" || true)"
 if [ -z "$core_name" ]; then
     core_name="core"
 fi
