@@ -3,9 +3,6 @@
 # shellcheck source=tests/lib/systemd.sh
 . "$TESTSLIB"/systemd.sh
 
-# shellcheck source=tests/lib/systems.sh
-. "$TESTSLIB"/systems.sh
-
 # shellcheck source=tests/lib/store.sh
 . "$TESTSLIB"/store.sh
 
@@ -45,11 +42,10 @@ nested_get_boot_id() {
 
 nested_wait_for_reboot() {
     local initial_boot_id="$1"
-    local retry wait last_boot_id
-    retry=150
-    wait=5
+    local last_boot_id="$initial_boot_id"
+    local retry=150
+    local wait=5
 
-    last_boot_id=""
     while [ $retry -ge 0 ]; do
         retry=$(( retry - 1 ))
         # The get_boot_id could fail because the connection is broken due to the reboot
@@ -63,6 +59,20 @@ nested_wait_for_reboot() {
     [ "$last_boot_id" != "$initial_boot_id" ]
 }
 
+nested_uc20_transition_to_system_mode() {
+    local recovery_system="$1"
+    local mode="$2"
+    local current_boot_id
+    current_boot_id=$(nested_get_boot_id)
+    nested_exec "sudo snap reboot --$mode $recovery_system"
+    nested_wait_for_reboot "$current_boot_id"
+
+    # verify we are now in the requested mode
+    if ! nested_exec "cat /proc/cmdline" | MATCH "snapd_recovery_mode=$mode"; then
+        return 1
+    fi
+}
+
 nested_retry_while_success() {
     local retry="$1"
     local wait="$2"
@@ -133,8 +143,14 @@ nested_create_assertions_disk() {
     local AUTO_IMPORT_ASSERT
     if [ -n "$NESTED_CUSTOM_AUTO_IMPORT_ASSERTION" ]; then
         AUTO_IMPORT_ASSERT=$NESTED_CUSTOM_AUTO_IMPORT_ASSERTION
-    else 
-        AUTO_IMPORT_ASSERT="$TESTSLIB/assertions/auto-import.assert"
+    else
+        local per_model_auto
+        per_model_auto="$(nested_model_authority).auto-import.assert"
+        if [ -e "$TESTSLIB/assertions/${per_model_auto}" ]; then
+            AUTO_IMPORT_ASSERT="$TESTSLIB/assertions/${per_model_auto}"
+        else
+            AUTO_IMPORT_ASSERT="$TESTSLIB/assertions/auto-import.assert"
+        fi
     fi
     cp "$AUTO_IMPORT_ASSERT" "$NESTED_ASSETS_DIR/sys-user-partition/auto-import.assert"
 
@@ -254,15 +270,15 @@ nested_is_classic_system() {
 }
 
 nested_is_core_20_system() {
-    is_focal_system
+    os.query is-focal
 }
 
 nested_is_core_18_system() {
-    is_bionic_system
+    os.query is-bionic
 }
 
 nested_is_core_16_system() {
-    is_xenial_system
+    os.query is-xenial
 }
 
 nested_refresh_to_new_core() {
@@ -302,19 +318,18 @@ nested_get_snakeoil_key() {
 }
 
 nested_secboot_sign_file() {
-    local DIR="$1"
+    local FILE="$1"
     local KEY="$2"
     local CERT="$3"
-    local FILE="$4"
-    sbattach --remove "$DIR"/"$FILE"
-    sbsign --key "$KEY" --cert "$CERT" --output "$DIR"/"$FILE" "$DIR"/"$FILE"
+    sbattach --remove "$FILE"
+    sbsign --key "$KEY" --cert "$CERT" --output "$FILE" "$FILE"
 }
 
 nested_secboot_sign_gadget() {
     local GADGET_DIR="$1"
     local KEY="$2"
     local CERT="$3"
-    nested_secboot_sign_file "$GADGET_DIR" "$KEY" "$CERT" "shim.efi.signed"
+    nested_secboot_sign_file "$GADGET_DIR/shim.efi.signed" "$KEY" "$CERT"
 }
 
 nested_prepare_env() {
@@ -413,6 +428,22 @@ nested_get_model() {
     esac
 }
 
+nested_model_authority() {
+    local model
+    model="$(nested_get_model)"
+    grep "authority-id:" "$model"|cut -d ' ' -f2
+}
+
+nested_ensure_ubuntu_save() {
+    local GADGET_DIR="$1"
+    "$TESTSLIB"/ensure_ubuntu_save.py "$GADGET_DIR"/meta/gadget.yaml > /tmp/gadget-with-save.yaml
+    if [ "$(cat /tmp/gadget-with-save.yaml)" != "" ]; then
+        mv /tmp/gadget-with-save.yaml "$GADGET_DIR"/meta/gadget.yaml
+    else
+        rm -f /tmp/gadget-with-save.yaml
+    fi
+}
+
 nested_create_core_vm() {
     # shellcheck source=tests/lib/prepare.sh
     . "$TESTSLIB"/prepare.sh
@@ -449,6 +480,16 @@ nested_create_core_vm() {
                     repack_snapd_deb_into_snapd_snap "$NESTED_ASSETS_DIR"
                     EXTRA_FUNDAMENTAL="$EXTRA_FUNDAMENTAL --snap $NESTED_ASSETS_DIR/snapd-from-deb.snap"
 
+                    snap download --channel="$CORE_CHANNEL" --basename=core18 core18
+                    repack_core_snap_with_tweaks "core18.snap" "new-core18.snap"
+                    EXTRA_FUNDAMENTAL="$EXTRA_FUNDAMENTAL --snap $PWD/new-core18.snap"
+
+                    repack_core_snap_with_tweaks "core18.snap" "new-core18.snap"
+
+                    if [ "$NESTED_SIGN_SNAPS_FAKESTORE" = "true" ]; then
+                        make_snap_installable_with_id "$NESTED_FAKESTORE_BLOB_DIR" "$PWD/new-core18.snap" "CSO04Jhav2yK0uz97cr0ipQRyqg0qQL6"
+                    fi
+
                 elif nested_is_core_20_system; then
                     snap download --basename=pc-kernel --channel="20/edge" pc-kernel
                     uc20_build_initramfs_kernel_snap "$PWD/pc-kernel.snap" "$NESTED_ASSETS_DIR"
@@ -482,6 +523,10 @@ nested_create_core_vm() {
                         snap download --basename=pc --channel="20/edge" pc
                         unsquashfs -d pc-gadget pc.snap
                         nested_secboot_sign_gadget pc-gadget "$SNAKEOIL_KEY" "$SNAKEOIL_CERT"
+                        # TODO:UC20: until https://github.com/snapcore/pc-amd64-gadget/pull/51/
+                        # lands there is no ubuntu-save in the gadget, make sure we have one
+                        nested_ensure_ubuntu_save pc-gadget
+
                         # also make logging persistent for easier debugging of 
                         # test failures, otherwise we have no way to see what 
                         # happened during a failed nested VM boot where we 
@@ -515,7 +560,7 @@ EOF
 
                     # which channel?
                     snap download --channel="$CORE_CHANNEL" --basename=core20 core20
-                    repack_core20_snap_with_tweaks "core20.snap" "new-core20.snap"
+                    repack_core_snap_with_tweaks "core20.snap" "new-core20.snap"
                     EXTRA_FUNDAMENTAL="$EXTRA_FUNDAMENTAL --snap $PWD/new-core20.snap"
 
                     # sign the snapd snap with fakestore if requested
@@ -683,11 +728,44 @@ nested_configure_cloud_init_on_core20_vm() {
     kpartx -d "$IMAGE"
 }
 
+nested_save_serial_log() {
+    if [ -f "${NESTED_LOGS_DIR}/serial.log" ]; then
+        for i in $(seq 1 9); do
+            if [ ! -f "${NESTED_LOGS_DIR}/serial.log.${i}" ]; then
+                cp "${NESTED_LOGS_DIR}/serial.log" "${NESTED_LOGS_DIR}/serial.log.${i}"
+                break
+            fi
+        done
+        # make sure we start with clean log file
+        echo > "${NESTED_LOGS_DIR}/serial.log"
+    fi
+}
+
+nested_print_serial_log() {
+    if [ -f "${NESTED_LOGS_DIR}/serial.log.1" ]; then
+        # here we disable SC2045 because previously it is checked there is at least
+        # 1 file which matches. In this case ls command is needed because it is important
+        # to get the list in reverse order.
+        # shellcheck disable=SC2045
+        for logfile in $(ls "${NESTED_LOGS_DIR}"/serial.log.*); do
+            cat "$logfile"
+        done
+    fi
+    if [ -f "${NESTED_LOGS_DIR}/serial.log" ]; then
+        cat "${NESTED_LOGS_DIR}/serial.log"
+    fi
+}
+
 nested_force_stop_vm() {
     systemctl stop nested-vm
 }
 
 nested_force_start_vm() {
+    # if the nested-vm is using a swtpm, we need to wait until the file exists
+    # because the file disappears temporarily after qemu exits
+    if systemctl show nested-vm -p ExecStart | grep -q swtpm-mvo; then
+        retry -n 10 --wait 1 test -S /var/snap/swtpm-mvo/current/swtpm-sock
+    fi
     systemctl start nested-vm
 }
 
@@ -735,6 +813,9 @@ nested_start_core_vm_unit() {
         PARAM_SERIAL="-chardev socket,telnet,host=localhost,server,port=7777,nowait,id=char0,logfile=${NESTED_LOGS_DIR}/serial.log,logappend=on -serial chardev:char0"
     fi
 
+    # save logs from previous runs
+    nested_save_serial_log
+
     # Set kvm attribute
     local ATTR_KVM
     ATTR_KVM=""
@@ -742,9 +823,6 @@ nested_start_core_vm_unit() {
         ATTR_KVM=",accel=kvm"
         # CPU can be defined just when kvm is enabled
         PARAM_CPU="-cpu host"
-        # Increase the number of cpus used once the issue related to kvm and ovmf is fixed
-        # https://bugs.launchpad.net/ubuntu/+source/kvm/+bug/1872803
-        PARAM_SMP="-smp 1"
     fi
 
     local PARAM_MACHINE
@@ -791,6 +869,10 @@ nested_start_core_vm_unit() {
             OVMF_VARS="snakeoil"
         fi
 
+        if [ "${NESTED_ENABLE_OVMF:-}" = "true" ]; then
+            PARAM_BIOS="-bios /usr/share/OVMF/OVMF_CODE.fd"
+        fi
+        
         if [ "$NESTED_ENABLE_SECURE_BOOT" = "true" ]; then
             cp -f "/usr/share/OVMF/OVMF_VARS.$OVMF_VARS.fd" "$NESTED_ASSETS_DIR/OVMF_VARS.$OVMF_VARS.fd"
             PARAM_BIOS="-drive file=/usr/share/OVMF/OVMF_CODE.$OVMF_CODE.fd,if=pflash,format=raw,unit=0,readonly -drive file=$NESTED_ASSETS_DIR/OVMF_VARS.$OVMF_VARS.fd,if=pflash,format=raw"
@@ -908,17 +990,24 @@ nested_start_core_vm() {
     else
         # Start the nested core vm
         nested_start_core_vm_unit "$CURRENT_IMAGE"
-    fi    
+    fi
 }
 
 nested_shutdown() {
+    # we sometimes have bugs in nested vm's where files that were successfully
+    # written become empty all of a sudden, so doing a sync here in the VM, and
+    # another one in the host when done probably helps to avoid that, and at
+    # least can't hurt anything
+    nested_exec "sync"
     nested_exec "sudo shutdown now" || true
     nested_wait_for_no_ssh
     nested_force_stop_vm
     wait_for_service "$NESTED_VM" inactive
+    sync
 }
 
 nested_start() {
+    nested_save_serial_log
     nested_force_start_vm
     wait_for_service "$NESTED_VM" active
     nested_wait_for_ssh
@@ -971,19 +1060,20 @@ nested_start_classic_vm() {
     PARAM_SMP="-smp 1"
     # use only 2G of RAM for qemu-nested
     if [ "$SPREAD_BACKEND" = "google-nested" ]; then
-        PARAM_MEM="-m 4096"
+        PARAM_MEM="${NESTED_PARAM_MEM:--m 4096}"
     elif [ "$SPREAD_BACKEND" = "qemu-nested" ]; then
-        PARAM_MEM="-m 2048"
+        PARAM_MEM="${NESTED_PARAM_MEM:--m 2048}"
     else
         echo "unknown spread backend $SPREAD_BACKEND"
         exit 1
     fi
-    local PARAM_DISPLAY PARAM_NETWORK PARAM_MONITOR PARAM_USB PARAM_CPU PARAM_RANDOM PARAM_SNAPSHOT
+    local PARAM_DISPLAY PARAM_NETWORK PARAM_MONITOR PARAM_USB PARAM_CPU PARAM_CD PARAM_RANDOM PARAM_SNAPSHOT
     PARAM_DISPLAY="-nographic"
     PARAM_NETWORK="-net nic,model=virtio -net user,hostfwd=tcp::$NESTED_SSH_PORT-:22"
     PARAM_MONITOR="-monitor tcp:127.0.0.1:$NESTED_MON_PORT,server,nowait"
     PARAM_USB="-usb"
     PARAM_CPU=""
+    PARAM_CD="${NESTED_PARAM_CD:-}"
     PARAM_RANDOM="-object rng-random,id=rng0,filename=/dev/urandom -device virtio-rng-pci,rng=rng0"
     PARAM_SNAPSHOT="-snapshot"
 
@@ -1025,8 +1115,9 @@ nested_start_classic_vm() {
 
     # ensure we have a log dir
     mkdir -p "$NESTED_LOGS_DIR"
-    # make sure we start with clean log file
-    echo > "${NESTED_LOGS_DIR}/serial.log"
+    # save logs from previous runs
+    nested_save_serial_log
+
     # Systemd unit is created, it is important to respect the qemu parameters order
     systemd_create_and_start_unit "$NESTED_VM" "${QEMU}  \
         ${PARAM_SMP} \
@@ -1043,13 +1134,14 @@ nested_start_classic_vm() {
         ${PARAM_SEED} \
         ${PARAM_SERIAL} \
         ${PARAM_MONITOR} \
-        ${PARAM_USB} "
+        ${PARAM_USB} \
+        ${PARAM_CD} "
 
     nested_wait_for_ssh
 }
 
 nested_destroy_vm() {
-    systemd_stop_and_destroy_unit "$NESTED_VM"
+    systemd_stop_and_remove_unit "$NESTED_VM"
 
     local CURRENT_IMAGE
     CURRENT_IMAGE="$NESTED_IMAGES_DIR/$(nested_get_current_image_name)" 
@@ -1120,3 +1212,17 @@ nested_fetch_spread() {
         echo "$NESTED_WORK_DIR/spread"
     fi
 }
+
+nested_build_seed_cdrom() {
+    local SEED_DIR="$1"
+    local SEED_NAME="$2"
+    local LABEL="$3"
+
+    shift 3
+
+    local ORIG_DIR=$PWD
+
+    pushd "$SEED_DIR" || return 1 
+    genisoimage -output "$ORIG_DIR/$SEED_NAME" -volid "$LABEL" -joliet -rock "$@"
+    popd || return 1 
+}
