@@ -446,3 +446,14 @@ func (s *clientSuite) TestServicesStopFailure(c *C) {
 		Error:   "failed to stop",
 	})
 }
+
+func (s *clientSuite) TestPendingRefreshNotification(c *C) {
+	s.handler = http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
+		c.Assert(r.URL.Path, Equals, "/v1/notifications/pending-refresh")
+		w.Header().Set("Content-Type", "application/json")
+		w.WriteHeader(200)
+		w.Write([]byte(`{"type": "sync"}`))
+	})
+	err := s.cli.PendingRefreshNotification(context.Background(), &client.PendingSnapRefreshInfo{})
+	c.Assert(err, IsNil)
+}
