@@ -21,12 +21,6 @@ package userd
 
 import (
 	"github.com/godbus/dbus"
-
-	"github.com/snapcore/snapd/sandbox/cgroup"
-)
-
-var (
-	SnapFromPid = snapFromPid
 )
 
 func MockSnapFromSender(f func(*dbus.Conn, dbus.Sender) (string, error)) func() {
@@ -36,11 +30,3 @@ func MockSnapFromSender(f func(*dbus.Conn, dbus.Sender) (string, error)) func()
 		snapFromSender = origSnapFromSender
 	}
 }
-
-func MockProcGroup(f func(pid int, match cgroup.GroupMatcher) (string, error)) (restore func()) {
-	old := cgroupProcGroup
-	cgroupProcGroup = f
-	return func() {
-		cgroupProcGroup = old
-	}
-}
