@@ -26,6 +26,7 @@ import (
 	"os"
 	"path/filepath"
 	"regexp"
+	"syscall"
 	"time"
 
 	"github.com/snapcore/snapd/dirs"
@@ -137,7 +138,7 @@ func AddSnapdSnapServices(s *snap.Info, inter interacter) error {
 		return nil
 	}
 
-	sysd := systemd.New(dirs.GlobalRootDir, systemd.SystemMode, inter)
+	sysd := systemd.New(systemd.SystemMode, inter)
 
 	if err := writeSnapdToolingMountUnit(sysd, s.MountDir()); err != nil {
 		return err
@@ -383,7 +384,7 @@ func writeSnapdUserServicesOnCore(s *snap.Info, inter interacter) error {
 		return err
 	}
 
-	sysd := systemd.New(dirs.GlobalRootDir, systemd.GlobalUserMode, inter)
+	sysd := systemd.New(systemd.GlobalUserMode, inter)
 
 	serviceUnits, err := filepath.Glob(filepath.Join(s.MountDir(), "usr/lib/systemd/user/*.service"))
 	if err != nil {
@@ -451,7 +452,7 @@ func writeSnapdUserServicesOnCore(s *snap.Info, inter interacter) error {
 // deployed in the filesystem as part of snapd snap installation. This should
 // only be executed as part of a controlled undo path.
 func undoSnapdUserServicesOnCore(s *snap.Info, inter interacter) error {
-	sysd := systemd.New(dirs.GlobalRootDir, systemd.GlobalUserMode, inter)
+	sysd := systemd.NewUnderRoot(dirs.GlobalRootDir, systemd.GlobalUserMode, inter)
 
 	// list user service and socket units present in the snapd snap
 	serviceUnits, err := filepath.Glob(filepath.Join(s.MountDir(), "usr/lib/systemd/user/*.service"))
@@ -516,15 +517,40 @@ func DeriveSnapdDBusConfig(s *snap.Info) (sessionContent, systemContent map[stri
 	return sessionContent, systemContent, nil
 }
 
+func isReadOnlyFsError(err error) bool {
+	if err == nil {
+		return false
+	}
+	if e, ok := err.(*os.PathError); ok {
+		err = e.Err
+	}
+	if e, ok := err.(syscall.Errno); ok {
+		return e == syscall.EROFS
+	}
+	return false
+}
+
+var ensureDirState = osutil.EnsureDirState
+
 func writeSnapdDbusConfigOnCore(s *snap.Info) error {
 	sessionContent, systemContent, err := DeriveSnapdDBusConfig(s)
 	if err != nil {
 		return err
 	}
 
-	_, _, err = osutil.EnsureDirState(dirs.SnapDBusSessionPolicyDir, "snapd.*.conf", sessionContent)
+	_, _, err = ensureDirState(dirs.SnapDBusSessionPolicyDir, "snapd.*.conf", sessionContent)
 	if err != nil {
-		return err
+		if isReadOnlyFsError(err) {
+			// If /etc/dbus-1/session.d is read-only (which may be the case on very old core18), then
+			// err is os.PathError with syscall.Errno underneath. Hitting this prevents snapd refresh,
+			// so log the error but carry on. This fixes LP: 1899664.
+			// XXX: ideally we should regenerate session files elsewhere if we fail here (otherwise
+			// this will only happen on future snapd refresh), but realistically this
+			// is not relevant on core18 devices.
+			logger.Noticef("%s appears to be read-only, could not write snapd dbus config files", dirs.SnapDBusSessionPolicyDir)
+		} else {
+			return err
+		}
 	}
 
 	_, _, err = osutil.EnsureDirState(dirs.SnapDBusSystemPolicyDir, "snapd.*.conf", systemContent)
@@ -558,7 +584,7 @@ func RemoveSnapdSnapServicesOnCore(s *snap.Info, inter interacter) error {
 		return nil
 	}
 
-	sysd := systemd.New(dirs.GlobalRootDir, systemd.SystemMode, inter)
+	sysd := systemd.NewUnderRoot(dirs.GlobalRootDir, systemd.SystemMode, inter)
 
 	if err := undoSnapdDbusConfigOnCore(s); err != nil {
 		return err
