@@ -23,10 +23,12 @@ import (
 	"fmt"
 	"os"
 	"path/filepath"
+	"syscall"
 
 	. "gopkg.in/check.v1"
 
 	"github.com/snapcore/snapd/dirs"
+	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/progress"
 	"github.com/snapcore/snapd/release"
@@ -100,9 +102,9 @@ func (s *servicesTestSuite) TestAddSnapServicesForSnapdOnCore(c *C) {
 			s := fmt.Sprintf("Type=oneshot\nId=%s\nActiveState=inactive\nUnitFileState=enabled\n", cmd[2])
 			return []byte(s), nil
 		}
-		if len(cmd) == 4 && cmd[2] == "is-enabled" {
+		if len(cmd) == 2 && cmd[0] == "is-enabled" {
 			// pretend snapd.socket is disabled
-			if cmd[3] == "snapd.socket" {
+			if cmd[1] == "snapd.socket" {
 				return []byte("disabled"), &mockSystemctlError{msg: "disabled", exitCode: 1}
 			}
 			return []byte("enabled"), nil
@@ -173,34 +175,34 @@ WantedBy=snapd.service
 	// check the systemctl calls
 	c.Check(s.sysdLog, DeepEquals, [][]string{
 		{"daemon-reload"},
-		{"--root", dirs.GlobalRootDir, "enable", "usr-lib-snapd.mount"},
+		{"enable", "usr-lib-snapd.mount"},
 		{"stop", "usr-lib-snapd.mount"},
 		{"show", "--property=ActiveState", "usr-lib-snapd.mount"},
 		{"start", "usr-lib-snapd.mount"},
 		{"daemon-reload"},
-		{"--root", dirs.GlobalRootDir, "is-enabled", "snapd.autoimport.service"},
-		{"--root", dirs.GlobalRootDir, "is-enabled", "snapd.service"},
-		{"--root", dirs.GlobalRootDir, "is-enabled", "snapd.snap-repair.timer"},
+		{"is-enabled", "snapd.autoimport.service"},
+		{"is-enabled", "snapd.service"},
+		{"is-enabled", "snapd.snap-repair.timer"},
 		// test pretends snapd.socket is disabled and needs enabling
-		{"--root", dirs.GlobalRootDir, "is-enabled", "snapd.socket"},
-		{"--root", dirs.GlobalRootDir, "enable", "snapd.socket"},
-		{"--root", dirs.GlobalRootDir, "is-enabled", "snapd.system-shutdown.service"},
-		{"--root", dirs.GlobalRootDir, "is-active", "snapd.autoimport.service"},
+		{"is-enabled", "snapd.socket"},
+		{"enable", "snapd.socket"},
+		{"is-enabled", "snapd.system-shutdown.service"},
+		{"is-active", "snapd.autoimport.service"},
 		{"stop", "snapd.autoimport.service"},
 		{"show", "--property=ActiveState", "snapd.autoimport.service"},
 		{"start", "snapd.autoimport.service"},
-		{"--root", dirs.GlobalRootDir, "is-active", "snapd.snap-repair.timer"},
+		{"is-active", "snapd.snap-repair.timer"},
 		{"stop", "snapd.snap-repair.timer"},
 		{"show", "--property=ActiveState", "snapd.snap-repair.timer"},
 		{"start", "snapd.snap-repair.timer"},
-		{"--root", dirs.GlobalRootDir, "is-active", "snapd.socket"},
+		{"is-active", "snapd.socket"},
 		{"start", "snapd.service"},
 		{"start", "--no-block", "snapd.seeded.service"},
 		{"start", "--no-block", "snapd.autoimport.service"},
-		{"--user", "--global", "--root", dirs.GlobalRootDir, "disable", "snapd.session-agent.service"},
-		{"--user", "--global", "--root", dirs.GlobalRootDir, "enable", "snapd.session-agent.service"},
-		{"--user", "--global", "--root", dirs.GlobalRootDir, "disable", "snapd.session-agent.socket"},
-		{"--user", "--global", "--root", dirs.GlobalRootDir, "enable", "snapd.session-agent.socket"},
+		{"--user", "--global", "disable", "snapd.session-agent.service"},
+		{"--user", "--global", "enable", "snapd.session-agent.service"},
+		{"--user", "--global", "disable", "snapd.session-agent.socket"},
+		{"--user", "--global", "enable", "snapd.session-agent.socket"},
 	})
 }
 
@@ -227,6 +229,39 @@ func (s *servicesTestSuite) TestAddSnapServicesForSnapdOnClassic(c *C) {
 	c.Check(s.sysdLog, IsNil)
 }
 
+func (s *servicesTestSuite) TestAddSessionServicesWithReadOnlyFilesystem(c *C) {
+	restore := release.MockOnClassic(false)
+	defer restore()
+
+	restoreEnsureDirState := wrappers.MockEnsureDirState(func(dir string, glob string, content map[string]osutil.FileState) (changed, removed []string, err error) {
+		return nil, nil, &os.PathError{Err: syscall.EROFS}
+	})
+	defer restoreEnsureDirState()
+
+	info := makeMockSnapdSnap(c)
+
+	logBuf, restore := logger.MockLogger()
+	defer restore()
+
+	// add the snapd service
+	err := wrappers.AddSnapdSnapServices(info, progress.Null)
+
+	// didn't fail despite of read-only SnapDBusSessionPolicyDir
+	c.Assert(err, IsNil)
+
+	// check that snapd services were *not* created
+	c.Check(osutil.FileExists(filepath.Join(dirs.SnapServicesDir, "snapd.service")), Equals, true)
+	c.Check(osutil.FileExists(filepath.Join(dirs.SnapServicesDir, "snapd.autoimport.service")), Equals, true)
+	c.Check(osutil.FileExists(filepath.Join(dirs.SnapServicesDir, "snapd.system-shutdown.service")), Equals, true)
+	c.Check(osutil.FileExists(filepath.Join(dirs.SnapServicesDir, "usr-lib-snapd.mount")), Equals, true)
+	c.Check(osutil.FileExists(filepath.Join(dirs.SnapUserServicesDir, "snapd.session-agent.service")), Equals, true)
+	c.Check(osutil.FileExists(filepath.Join(dirs.SnapUserServicesDir, "snapd.session-agent.socket")), Equals, true)
+	c.Check(osutil.FileExists(filepath.Join(dirs.SnapDBusSystemPolicyDir, "snapd.system-services.conf")), Equals, true)
+	c.Check(osutil.FileExists(filepath.Join(dirs.SnapDBusSessionPolicyDir, "snapd.session-services.conf")), Equals, false)
+
+	c.Assert(logBuf.String(), testutil.Contains, "/etc/dbus-1/session.d appears to be read-only, could not write snapd dbus config files")
+}
+
 func (s *servicesTestSuite) TestAddSnapdServicesWithNonSnapd(c *C) {
 	restore := release.MockOnClassic(false)
 	defer restore()
