@@ -21,6 +21,8 @@ package wrappers
 
 import (
 	"time"
+
+	"github.com/snapcore/snapd/osutil"
 )
 
 // some internal helper exposed for testing
@@ -50,3 +52,11 @@ func MockKillWait(wait time.Duration) (restore func()) {
 		killWait = oldKillWait
 	}
 }
+
+func MockEnsureDirState(f func(dir string, glob string, content map[string]osutil.FileState) (changed, removed []string, err error)) (restore func()) {
+	oldEnsureDirState := ensureDirState
+	ensureDirState = f
+	return func() {
+		ensureDirState = oldEnsureDirState
+	}
+}
