@@ -31,7 +31,7 @@ import (
 	"text/template"
 	"time"
 
-	"github.com/snapcore/snapd/dirs"
+	"github.com/snapcore/snapd/interfaces"
 	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/osutil/sys"
@@ -66,7 +66,7 @@ func generateSnapServiceFile(app *snap.AppInfo, opts *AddSnapServicesOptions) ([
 		return nil, err
 	}
 
-	return genServiceFile(app, opts), nil
+	return genServiceFile(app, opts)
 }
 
 func stopUserServices(cli *client.Client, inter interacter, services ...string) error {
@@ -144,8 +144,8 @@ func enableServices(apps []*snap.AppInfo, inter interacter) (disable func(), err
 	var enabled []string
 	var userEnabled []string
 
-	systemSysd := systemd.New(dirs.GlobalRootDir, systemd.SystemMode, inter)
-	userSysd := systemd.New(dirs.GlobalRootDir, systemd.GlobalUserMode, inter)
+	systemSysd := systemd.New(systemd.SystemMode, inter)
+	userSysd := systemd.New(systemd.GlobalUserMode, inter)
 
 	disableEnabledServices := func() {
 		for _, srvName := range enabled {
@@ -204,8 +204,8 @@ type StartServicesFlags struct {
 // are services. Service units will be started in the order provided by the
 // caller.
 func StartServices(apps []*snap.AppInfo, disabledSvcs []string, flags *StartServicesFlags, inter interacter, tm timings.Measurer) (err error) {
-	systemSysd := systemd.New(dirs.GlobalRootDir, systemd.SystemMode, inter)
-	userSysd := systemd.New(dirs.GlobalRootDir, systemd.GlobalUserMode, inter)
+	systemSysd := systemd.New(systemd.SystemMode, inter)
+	userSysd := systemd.New(systemd.GlobalUserMode, inter)
 	cli := client.New()
 
 	systemServices := make([]string, 0, len(apps))
@@ -372,7 +372,8 @@ func AddSnapServices(s *snap.Info, disabledSvcs []string, opts *AddSnapServicesO
 	// TODO: remove once services get enabled on start and not when created.
 	preseeding := opts.Preseeding
 
-	sysd := systemd.New(dirs.GlobalRootDir, systemd.SystemMode, inter)
+	// note, sysd is not used when preseeding
+	sysd := systemd.New(systemd.SystemMode, inter)
 	var written []string
 	var writtenSystem, writtenUser bool
 	var disableEnabledServices func()
@@ -467,6 +468,7 @@ func AddSnapServices(s *snap.Info, disabledSvcs []string, opts *AddSnapServicesO
 		toEnable = append(toEnable, app)
 	}
 
+	// this is no-op when preseeding because of empty toEnable list
 	disableEnabledServices, err = enableServices(toEnable, inter)
 	if err != nil {
 		return err
@@ -492,7 +494,7 @@ func AddSnapServices(s *snap.Info, disabledSvcs []string, opts *AddSnapServicesO
 // the first boot of a pre-seeded image with service files already in place but not enabled.
 // XXX: it should go away once services are fixed and enabled on start.
 func EnableSnapServices(s *snap.Info, inter interacter) (err error) {
-	sysd := systemd.New(dirs.GlobalRootDir, systemd.SystemMode, inter)
+	sysd := systemd.New(systemd.SystemMode, inter)
 	for _, app := range s.Apps {
 		if app.IsService() {
 			svcName := app.ServiceName()
@@ -512,7 +514,7 @@ type StopServicesFlags struct {
 // StopServices stops and optionally disables service units for the applications
 // from the snap which are services.
 func StopServices(apps []*snap.AppInfo, flags *StopServicesFlags, reason snap.ServiceStopReason, inter interacter, tm timings.Measurer) error {
-	sysd := systemd.New(dirs.GlobalRootDir, systemd.SystemMode, inter)
+	sysd := systemd.New(systemd.SystemMode, inter)
 	if flags == nil {
 		flags = &StopServicesFlags{}
 	}
@@ -567,7 +569,7 @@ func StopServices(apps []*snap.AppInfo, flags *StopServicesFlags, reason snap.Se
 // ServicesEnableState returns a map of service names from the given snap,
 // together with their enable/disable status.
 func ServicesEnableState(s *snap.Info, inter interacter) (map[string]bool, error) {
-	sysd := systemd.New(dirs.GlobalRootDir, systemd.SystemMode, inter)
+	sysd := systemd.New(systemd.SystemMode, inter)
 
 	// loop over all services in the snap, querying systemd for the current
 	// systemd state of the snaps
@@ -596,8 +598,8 @@ func RemoveSnapServices(s *snap.Info, inter interacter) error {
 	if s.Type() == snap.TypeSnapd {
 		return fmt.Errorf("internal error: removing explicit services for snapd snap is unexpected")
 	}
-	systemSysd := systemd.New(dirs.GlobalRootDir, systemd.SystemMode, inter)
-	userSysd := systemd.New(dirs.GlobalRootDir, systemd.GlobalUserMode, inter)
+	systemSysd := systemd.New(systemd.SystemMode, inter)
+	userSysd := systemd.New(systemd.GlobalUserMode, inter)
 	var removedSystem, removedUser bool
 
 	for _, app := range s.Apps {
@@ -677,11 +679,43 @@ func genServiceNames(snap *snap.Info, appNames []string) []string {
 	return names
 }
 
-func genServiceFile(appInfo *snap.AppInfo, opts *AddSnapServicesOptions) []byte {
+func genServiceFile(appInfo *snap.AppInfo, opts *AddSnapServicesOptions) ([]byte, error) {
 	if opts == nil {
 		opts = &AddSnapServicesOptions{}
 	}
 
+	// assemble all of the service directive snippets for all interfaces that
+	// this service needs to include in the generated systemd file
+
+	// use an ordered set to ensure we don't duplicate any keys from interfaces
+	// that specify the same snippet
+
+	// TODO: maybe we should error if multiple interfaces specify different
+	// values for the same directive, otherwise one of them will overwrite the
+	// other? What happens right now is that the snippet from the plug that
+	// comes last will win in the case of directives that can have only one
+	// value, but for some directives, systemd combines their values into a
+	// list.
+	ifaceServiceSnippets := &strutil.OrderedSet{}
+
+	for _, plug := range appInfo.Plugs {
+		iface, err := interfaces.ByName(plug.Interface)
+		if err != nil {
+			return nil, fmt.Errorf("error processing plugs while generating service unit for %v: %v", appInfo.SecurityTag(), err)
+		}
+		snips, err := interfaces.PermanentPlugServiceSnippets(iface, plug)
+		if err != nil {
+			return nil, fmt.Errorf("error processing plugs while generating service unit for %v: %v", appInfo.SecurityTag(), err)
+		}
+		for _, snip := range snips {
+			ifaceServiceSnippets.Put(snip)
+		}
+	}
+
+	// join the service snippets into one string to be included in the
+	// template
+	ifaceSpecifiedServiceSnippet := strings.Join(ifaceServiceSnippets.Items(), "\n")
+
 	serviceTemplate := `[Unit]
 # Auto-generated, DO NOT EDIT
 Description=Service for snap application {{.App.Snap.InstanceName}}.{{.App.Name}}
@@ -742,6 +776,9 @@ KillSignal={{.KillSignal}}
 {{- if .OOMAdjustScore }}
 OOMScoreAdjust={{.OOMAdjustScore}}
 {{- end}}
+{{- if .InterfaceServiceSnippets}}
+{{.InterfaceServiceSnippets}}
+{{- end}}
 {{- if not .App.Sockets}}
 
 [Install]
@@ -786,25 +823,28 @@ WantedBy={{.ServicesTarget}}
 	wrapperData := struct {
 		App *snap.AppInfo
 
-		Restart            string
-		WorkingDir         string
-		StopTimeout        time.Duration
-		StartTimeout       time.Duration
-		ServicesTarget     string
-		PrerequisiteTarget string
-		MountUnit          string
-		Remain             string
-		KillMode           string
-		KillSignal         string
-		OOMAdjustScore     int
-		Before             []string
-		After              []string
+		Restart                  string
+		WorkingDir               string
+		StopTimeout              time.Duration
+		StartTimeout             time.Duration
+		ServicesTarget           string
+		PrerequisiteTarget       string
+		MountUnit                string
+		Remain                   string
+		KillMode                 string
+		KillSignal               string
+		OOMAdjustScore           int
+		Before                   []string
+		After                    []string
+		InterfaceServiceSnippets string
 
 		Home    string
 		EnvVars string
 	}{
 		App: appInfo,
 
+		InterfaceServiceSnippets: ifaceSpecifiedServiceSnippet,
+
 		Restart:        restartCond,
 		StopTimeout:    serviceStopTimeout(appInfo),
 		StartTimeout:   time.Duration(appInfo.StartTimeout),
@@ -848,7 +888,7 @@ WantedBy={{.ServicesTarget}}
 		logger.Panicf("Unable to execute template: %v", err)
 	}
 
-	return templateOut.Bytes()
+	return templateOut.Bytes(), nil
 }
 
 func genServiceSocketFile(appInfo *snap.AppInfo, socketName string) []byte {
@@ -1225,7 +1265,7 @@ type RestartServicesFlags struct {
 
 // Restart or reload services; if reload flag is set then "systemctl reload-or-restart" is attempted.
 func RestartServices(svcs []*snap.AppInfo, flags *RestartServicesFlags, inter interacter, tm timings.Measurer) error {
-	sysd := systemd.New(dirs.GlobalRootDir, systemd.SystemMode, inter)
+	sysd := systemd.New(systemd.SystemMode, inter)
 
 	for _, srv := range svcs {
 		// they're *supposed* to be all services, but checking doesn't hurt
